@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2002-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2002-2016 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /**
@@ -25,6 +25,9 @@
  * Research program.
  */
 
+#if 0  // Broken program which uses old getfem brick. Should be adapted.
+       // Kept because it is the only example of 3D delamination.
+
 #include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
 #include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
 #include "getfem/getfem_derivatives.h"
@@ -39,11 +42,13 @@
 #include "getfem/getfem_import.h"
 #include "gmm/gmm.h"
 
+
+
 using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
 
 
-/* some Getfem++ types that we will be using */
+/* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_vector;
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
@@ -74,9 +79,9 @@ struct exact_solution_3D {
 	    getfem::level_set &ls) {
     std::vector<getfem::pglobal_function> cfun(4);
     for (unsigned j=0; j < 4; ++j) {
-      getfem::crack_singular_xy_function *s = 
-	new getfem::crack_singular_xy_function(j);
-      cfun[j] = getfem::global_function_on_level_set(ls, *s);
+      getfem::pxy_function s = 
+	std::make_shared<getfem::crack_singular_xy_function>(j);
+      cfun[j] = getfem::global_function_on_level_set(ls, s);
     }
     mf.set_functions(cfun);
     
@@ -125,8 +130,8 @@ base_small_vector sol_f(const base_node &x) {
 
 scalar_type cutoff_C2(scalar_type r, scalar_type r1, scalar_type r2) {
   if (r < r1) return 1.0;
-  if (r < r2) return pow(r2 - r, 3) *
-    (r2*r2 - 5*r1*r2 + 3*r*r2 + 10*r1*r1 + 6*r*r - 15*r*r1) / pow(r2 - r1, 5);
+  if (r < r2) return pow(r2 - r, 3.) *
+    (r2*r2 - 5*r1*r2 + 3*r*r2 + 10*r1*r1 + 6*r*r - 15*r*r1) / pow(r2 - r1, 5.);
   return 0.0;
 }
 
@@ -349,7 +354,7 @@ public:
     mf.extend_vector(U_, U);
   }
   
-  const bgeot::multi_index &sizes() const { return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const { return sizes_; }
   
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
@@ -546,7 +551,7 @@ namespace getfem {
       sizes_[0] = short_type((version == 1) ? 1 : N);
     }
     
-    const bgeot::multi_index &sizes() const { return sizes_; }
+    const bgeot::multi_index &sizes(size_type) const { return sizes_; }
     
     virtual void compute(getfem::fem_interpolation_context& ctx,
 			 bgeot::base_tensor &t) {
@@ -800,9 +805,9 @@ bool crack_problem::solve(plain_vector &U) {
   mfls_u.adapt();
   std::vector<getfem::pglobal_function> vfunc(4);
   for (unsigned j=0; j < 4; ++j) {
-    getfem::crack_singular_xy_function *s = 
-      new getfem::crack_singular_xy_function(j);
-    vfunc[j] = getfem::global_function_on_level_set(ls, *s);
+    getfem::pxy_function s = 
+      std::make_shared<getfem::crack_singular_xy_function>(j);
+    vfunc[j] = getfem::global_function_on_level_set(ls, s);
   }
   
   mf_sing_u.set_functions(vfunc);
@@ -1029,7 +1034,15 @@ void crack_problem::save_U(plain_vector &U) {
 /*  main program.                                                         */
 /**************************************************************************/
 
-int main(int argc, char *argv[]) {
+#endif
+
+
+int main(
+#if 0     
+         int argc, char *argv[]
+#endif
+         ) {
+#if 0
 
   GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
@@ -1073,5 +1086,7 @@ int main(int argc, char *argv[]) {
   }
   GMM_STANDARD_CATCH_ERROR;
 
+#endif
+
   return 0; 
 }
