@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2009-2012 Konstantinos Poulios.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2009-2016 Konstantinos Poulios.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,22 +16,24 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 #include "getfem/getfem_model_solvers.h"
 #include "getfem/getfem_import.h"   /* import functions (load a mesh from file)    */
 #include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
-#include "getfem/getfem_Coulomb_friction.h"
+#include "getfem/getfem_contact_and_friction_nodal.h"
+#include "getfem/getfem_contact_and_friction_integral.h"
+#include "getfem/getfem_contact_and_friction_large_sliding.h"
 #include "gmm/gmm.h"
 
 using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
 
-
-/* some Getfem++ types that we will be using */
+/* some GetFEM++ types that we will be using */
 using bgeot::dim_type;
 using bgeot::size_type;   /* = unsigned long */
+using bgeot::short_type;
 using bgeot::scalar_type; /* = double */
 using bgeot::base_node;   /* geometrical nodes (derived from base_small_vector)*/
 using bgeot::base_matrix; /* small dense matrix. */
@@ -46,23 +48,27 @@ typedef getfem::model_real_plain_vector  plain_vector;
 */
 struct elastostatic_contact_problem {
 
-  enum { DIRICHLET_BOUNDARY_1 = 0, DIRICHLET_BOUNDARY_2 = 1};
-  getfem::mesh mesh1, mesh2; /* the meshes */
-  getfem::mesh_im mim1, mim2;/* the integration methods */
-  getfem::mesh_fem mf_u1;    /* 1st mesh_fem, for the elastostatic solution  */
-  getfem::mesh_fem mf_u2;    /* 2nd mesh_fem, for the elastostatic solution  */
-  getfem::mesh_fem mf_rhs1;  /* 1st mesh_fem for the right hand side         */
-  getfem::mesh_fem mf_rhs2;  /* 2nd mesh_fem for the right hand side         */
-  scalar_type lambda, mu;    /* elastic coefficients.                        */
+  enum { DIRICHLET_BOUNDARY_1 = 0, DIRICHLET_BOUNDARY_2 = 1,
+         CONTACT_BOUNDARY_1 = 1001, CONTACT_BOUNDARY_2 = 1002 };
+  getfem::mesh mesh1, mesh2;   /* the meshes */
+  getfem::mesh_im mim1, mim2;  /* the integration methods */
+  getfem::mesh_fem mf_u1;      /* 1st mesh_fem, for the elastostatic solution  */
+  getfem::mesh_fem mf_u2;      /* 2nd mesh_fem, for the elastostatic solution  */
+  getfem::mesh_fem mf_rhs1;    /* 1st mesh_fem for the right hand side         */
+  getfem::mesh_fem mf_rhs2;    /* 2nd mesh_fem for the right hand side         */
+  getfem::mesh_fem mf_mult1;   /* 1st mesh_fem for the multipliers.            */
+  scalar_type lambda, mu;      /* elastic coefficients.                        */
 
-  scalar_type residual;      /* max residual for the iterative solvers       */
-  scalar_type rot_angle;     /* rotation angle of the pinion gear            */
-  scalar_type frict_coeff;   /* friction coefficient                         */
-//scalar_type threshold;     /* threshold distance for contact finding       */
+  scalar_type residual;        /* max residual for the iterative solvers       */
+  scalar_type rot_angle;       /* rotation angle of the pinion gear            */
+  scalar_type frict_coeff;     /* friction coefficient                         */
+//scalar_type threshold;       /* threshold distance for contact finding       */
 
-  size_type N;               /* dimension of the problem                     */
+  size_type N;                 /* dimension of the problem                     */
 
-  bool frictionless;         /* flag for frictionless model                  */
+  bool frictionless;           /* flag for frictionless model                  */
+  int contact_algo;            /* contact algorithm (0:nodal, 1-4: integral
+                                  >=5:integral large sliding)                  */
 
   // Vectors holding the ids of mesh region pairs expected to come in contact
   // with each other
@@ -74,7 +80,8 @@ struct elastostatic_contact_problem {
   bool solve(void);
   void init(void);
   elastostatic_contact_problem(void) : mim1(mesh1), mim2(mesh2),
-    mf_u1(mesh1), mf_u2(mesh2), mf_rhs1(mesh1), mf_rhs2(mesh2) {}
+    mf_u1(mesh1), mf_u2(mesh2), mf_rhs1(mesh1), mf_rhs2(mesh2),
+    mf_mult1(mesh1) {}
 };
 
 
@@ -150,7 +157,7 @@ void elastostatic_contact_problem::init(void) {
           mesh.add_convex_by_points(tmpmesh.trans_of_convex(cv),
                                     tmpmesh.points_of_convex(cv).begin());
       }
-      for (size_type f = 0; f < tmpmesh.structure_of_convex(cv)->nb_faces(); f++) {
+      for (short_type f = 0; f < tmpmesh.structure_of_convex(cv)->nb_faces(); f++) {
         for (size_type i = 0; i < dfs.size(); ++i)
           if (tmpmesh.region(dfs[i]).is_in(cv,f)) {
             dirichlet_boundary.add(newcv,f);
@@ -196,13 +203,35 @@ void elastostatic_contact_problem::init(void) {
   mim2.set_integration_method(ppi);
 
   /* set the finite element on mf_rhs */
-  mf_rhs1.set_finite_element(mesh1.convex_index(),
-                             getfem::fem_descriptor(FEM_TYPE));
-  mf_rhs2.set_finite_element(mesh2.convex_index(),
-                             getfem::fem_descriptor(FEM_TYPE));
+  mf_rhs1.set_finite_element(getfem::fem_descriptor(FEM_TYPE));
+  mf_rhs2.set_finite_element(getfem::fem_descriptor(FEM_TYPE));
 
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
 
+  contact_algo = int(PARAM.int_value("CONTACT_ALGO","Algorithm for imposing the contact condition."));
+
+  if (contact_algo != 0) { // integral contact
+    std::string MULT_FEM_TYPE  = PARAM.string_value("MULT_FEM_TYPE","FEM name for the multipliers");
+    if (frictionless && contact_algo >= 1 && contact_algo <= 4)
+      mf_mult1.set_qdim(dim_type(1));
+    else
+      mf_mult1.set_qdim(dim_type(N));
+    getfem::pfem pf_mult = getfem::fem_descriptor(MULT_FEM_TYPE);
+    mf_mult1.set_finite_element(pf_mult);
+
+    getfem::mesh_region &mr1 = mesh1.region(CONTACT_BOUNDARY_1);
+    getfem::mesh_region &mr2 = mesh2.region(CONTACT_BOUNDARY_2);
+    for (std::vector<size_type>::const_iterator rg_it=cb_rgs1.begin();
+         rg_it != cb_rgs1.end(); rg_it++)
+      mr1 = getfem::mesh_region::merge(mr1, mesh1.region(*rg_it));
+    for (std::vector<size_type>::const_iterator rg_it=cb_rgs2.begin();
+         rg_it != cb_rgs2.end(); rg_it++)
+      mr2 = getfem::mesh_region::merge(mr2, mesh2.region(*rg_it));
+
+    dal::bit_vector dol1 = mf_mult1.basic_dof_on_region(CONTACT_BOUNDARY_1);
+    mf_mult1.reduce_to_basic_dof(dol1);
+  }
+
 }
 
 /*  Construction and solution of the Model.
@@ -224,28 +253,53 @@ bool elastostatic_contact_problem::solve() {
   getfem::add_isotropic_linearized_elasticity_brick(md, mim1, "u1", "lambda", "mu");
   getfem::add_isotropic_linearized_elasticity_brick(md, mim2, "u2", "lambda", "mu");
 
-  // Nonlinear elasticity brick.
-//  base_vector p(2); p[0] = lambda; p[1] = mu;
-//  getfem::SaintVenant_Kirchhoff_hyperelastic_law pl;
-//  getfem::mdbrick_nonlinear_elasticity<>  ELAS(pl, mim, mf_u, p);
-  
   // Defining the contact condition.
-  std::string varname_u1="u1";
-  std::string varname_u2="u2";
-  std::string dataname_r="r";
   md.add_initialized_scalar_data
-    (dataname_r, mu * (3*lambda + 2*mu) / (lambda + mu) );  // r ~= Young modulus
+    ("r", mu * (3*lambda + 2*mu) / (lambda + mu) );  // r ~= Young modulus
   std::string multname_n, multname_t;
-  if (frictionless) {
-    getfem::add_nodal_contact_between_nonmatching_meshes_brick
-      (md, mim1, mim2, varname_u1, varname_u2, multname_n, dataname_r,
-       cb_rgs1, cb_rgs2);
+  if (contact_algo == 0) {
+    if (frictionless) {
+      getfem::add_nodal_contact_between_nonmatching_meshes_brick
+        (md, mim1, mim2, "u1", "u2", multname_n, "r",
+         cb_rgs1, cb_rgs2);
+    } else {
+      std::string dataname_frict_coeff="friction_coefficient";
+      md.add_initialized_scalar_data(dataname_frict_coeff, frict_coeff);
+      getfem::add_nodal_contact_between_nonmatching_meshes_brick
+        (md, mim1, mim2, "u1", "u2", multname_n, multname_t,
+         "r", dataname_frict_coeff, cb_rgs1, cb_rgs2);
+    }
   } else {
-    std::string dataname_frict_coeff="friction_coefficient";
-    md.add_initialized_scalar_data(dataname_frict_coeff, frict_coeff);
-    getfem::add_nodal_contact_between_nonmatching_meshes_brick
-      (md, mim1, mim2, varname_u1, varname_u2, multname_n, multname_t,
-       dataname_r, dataname_frict_coeff, cb_rgs1, cb_rgs2);
+    md.add_fem_variable("mult1", mf_mult1);
+    if (contact_algo >= 1 && contact_algo <= 4) { // integral contact
+      if (frictionless)
+        getfem::add_integral_contact_between_nonmatching_meshes_brick
+          (md, mim1, "u1", "u2", "mult1", "r",
+           CONTACT_BOUNDARY_1, CONTACT_BOUNDARY_2, contact_algo);
+      else {
+        md.add_initialized_scalar_data("f_coeff", frict_coeff);
+        getfem::add_integral_contact_between_nonmatching_meshes_brick
+          (md, mim1, "u1", "u2", "mult1", "r", "f_coeff",
+           CONTACT_BOUNDARY_1, CONTACT_BOUNDARY_2, contact_algo);
+      }
+    }
+    else { // large sliding is for the moment always frictionless
+      std::string u01_str(""), u02_str("");
+//      if (frict_coeff > scalar_type(0)) {
+//        u01_str = "u01";
+//        u02_str = "u02";
+//        md.add_fem_variable(u01_str, mf_u1);
+//        md.add_fem_variable(u02_str, mf_u2);
+//      }
+      md.add_initialized_scalar_data("f_coeff", frict_coeff);
+      size_type indb =
+      getfem::add_integral_large_sliding_contact_brick_raytracing
+      (md, "r", 20., "f_coeff", "1", false, false);
+      getfem::add_contact_boundary_to_large_sliding_contact_brick
+      (md, indb, mim1, CONTACT_BOUNDARY_1, false, true, "u1", "mult1", u01_str);
+      getfem::add_contact_boundary_to_large_sliding_contact_brick
+      (md, indb, mim2, CONTACT_BOUNDARY_2, true, false, "u2", "", u02_str);
+    }
   }
 
   // Defining the DIRICHLET condition.
@@ -271,8 +325,14 @@ bool elastostatic_contact_problem::solve() {
 
   gmm::iteration iter(residual, 1, 40000);
 
-  getfem::default_newton_line_search ls;
-  getfem::standard_solve(md, iter, getfem::rselect_linear_solver(md,"superlu"), ls);
+//  getfem::default_newton_line_search ls;
+  getfem::simplest_newton_line_search ls(50, 5., 5., 0.6, 1e-1);
+
+  #ifdef GMM_USES_MUMPS
+  getfem::standard_solve(md, iter, getfem::rselect_linear_solver(md,"mumps"), ls);
+  #else
+  getfem::standard_solve(md, iter, getfem::default_linear_solver<getfem::model_real_sparse_matrix, getfem::model_real_plain_vector>(md), ls);
+  #endif
 
   if (!iter.converged()) return false; // Solution has not converged
 
@@ -282,6 +342,9 @@ bool elastostatic_contact_problem::solve() {
   getfem::compute_isotropic_linearized_Von_Mises_or_Tresca
       (md, "u2", "lambda", "mu", mf_rhs2, VM2, false);
 
+  if (!getfem::MPI_IS_MASTER())
+    return true;
+
   // Prepare results
   plain_vector U1(mf_u1.nb_dof()), U2(mf_u2.nb_dof());
   plain_vector RHS(md.nb_dof());
@@ -298,36 +361,6 @@ bool elastostatic_contact_problem::solve() {
   gmm::copy(gmm::sub_vector(RHS, md.interval_of_variable("u2")), Forces2);
   gmm::scale(Forces2, -1.0);
 
-  gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
-                            md.interval_of_variable("u1"),
-                            md.interval_of_variable(multname_n) ),
-            md.real_variable(multname_n),
-            NCForces1);
-  gmm::scale(NCForces1, -1.0);
-  gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
-                            md.interval_of_variable("u2"),
-                            md.interval_of_variable(multname_n) ),
-            md.real_variable(multname_n),
-            NCForces2);
-  gmm::scale(NCForces2, -1.0);
-
-  if (!frictionless) {
-    gmm::resize(TCForces1, mf_u1.nb_dof());
-    gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
-                              md.interval_of_variable("u1"),
-                              md.interval_of_variable(multname_t) ),
-              md.real_variable(multname_t),
-              TCForces1);
-    gmm::scale(TCForces1, -1.0);
-    gmm::resize(TCForces2, mf_u2.nb_dof());
-    gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
-                              md.interval_of_variable("u2"),
-                              md.interval_of_variable(multname_t) ),
-              md.real_variable(multname_t),
-              TCForces2);
-    gmm::scale(TCForces2, -1.0);
-  }
-
   // Export results
   mesh1.write_to_file(datafilename + "1.mesh");
   mesh2.write_to_file(datafilename + "2.mesh");
@@ -346,15 +379,46 @@ bool elastostatic_contact_problem::solve() {
   exp2.write_point_data(mf_u2, U2, "elastostatic_displacement_2");
   exp1.write_point_data(mf_u1, Forces1, "forces_1");
   exp2.write_point_data(mf_u2, Forces2, "forces_2");
-  exp1.write_point_data(mf_u1, NCForces1, "normal_contact_forces_1");
-  exp2.write_point_data(mf_u2, NCForces2, "normal_contact_forces_2");
-  if (!frictionless) {
-    exp1.write_point_data(mf_u1, TCForces1, "tangential_contact_forces_1");
-    exp2.write_point_data(mf_u2, TCForces2, "tangential_contact_forces_2");
-  }
   exp1.write_point_data(mf_rhs1, VM1, "von_mises_stresses_1");
   exp2.write_point_data(mf_rhs2, VM2, "von_mises_stresses_2");
 
+  if (contact_algo == 0) {
+    gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
+                              md.interval_of_variable("u1"),
+                              md.interval_of_variable(multname_n) ),
+              md.real_variable(multname_n),
+              NCForces1);
+    gmm::scale(NCForces1, -1.0);
+    gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
+                              md.interval_of_variable("u2"),
+                              md.interval_of_variable(multname_n) ),
+              md.real_variable(multname_n),
+              NCForces2);
+    gmm::scale(NCForces2, -1.0);
+
+    exp1.write_point_data(mf_u1, NCForces1, "normal_contact_forces_1");
+    exp2.write_point_data(mf_u2, NCForces2, "normal_contact_forces_2");
+
+    if (!frictionless) {
+      gmm::resize(TCForces1, mf_u1.nb_dof());
+      gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
+                                md.interval_of_variable("u1"),
+                                md.interval_of_variable(multname_t) ),
+                md.real_variable(multname_t),
+                TCForces1);
+      gmm::scale(TCForces1, -1.0);
+      gmm::resize(TCForces2, mf_u2.nb_dof());
+      gmm::mult(gmm::sub_matrix(md.real_tangent_matrix(),
+                                md.interval_of_variable("u2"),
+                                md.interval_of_variable(multname_t) ),
+                md.real_variable(multname_t),
+                TCForces2);
+      gmm::scale(TCForces2, -1.0);
+      exp1.write_point_data(mf_u1, TCForces1, "tangential_contact_forces_1");
+      exp2.write_point_data(mf_u2, TCForces2, "tangential_contact_forces_2");
+    }
+  }
+
   return true; // Solution has converged
 }
 
@@ -364,10 +428,14 @@ bool elastostatic_contact_problem::solve() {
 
 int main(int argc, char *argv[]) {
 
+  GETFEM_MPI_INIT(argc, argv); // For parallelized version
+
   elastostatic_contact_problem p;
   p.PARAM.read_command_line(argc, argv);
   p.init();
   if (!p.solve()) cout << "Solve has failed\n";
 
+  GETFEM_MPI_FINALIZE;
+
   return 0;
 }
