@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2002-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -34,7 +34,6 @@
 #include "getfem/getfem_partial_mesh_fem.h"
 #include "getfem/getfem_contact_and_friction_nodal.h"
 #include "getfem/getfem_import.h"
-#include "getfem/getfem_inter_element.h"
 #include "gmm/gmm.h"
 
 using std::endl; using std::cout; using std::cerr;
@@ -333,7 +332,7 @@ void compute_mass_matrix_extra_element
   size_type nbd1 = pf1->nb_dof(cv1);
   
   if (pf1 != pf1_old || pai1 != pai1_old) {
-    pfp1 = fem_precomp(pf1, &pai1->integration_points(), pim1);
+    pfp1 = fem_precomp(pf1, pai1->pintegration_points(), pim1);
     pf1_old = pf1; pai1_old = pai1;
   }
   
