@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2002-2012 Yves Renard, Julien Pommier.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2002-2016 Yves Renard, Julien Pommier.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 /**
@@ -36,24 +36,27 @@
 #include "getfem/getfem_mesh_fem_product.h"
 #include "getfem/getfem_mesh_fem_global_function.h"
 #include "getfem/getfem_partial_mesh_fem.h"
-#include "getfem/getfem_Coulomb_friction.h"
+#include "getfem/getfem_contact_and_friction_nodal.h"
 #include "getfem/getfem_import.h"
-#include "getfem/getfem_inter_element.h"
 #include "getfem/getfem_mesh_fem_sum.h"
 #include "gmm/gmm_inoutput.h"
 
+#if GETFEM_HAVE_METIS_OLD_API
 extern "C" void METIS_PartGraphKway(int *, int *, int *, int *, int *, int *,
-			    int *, int *, int *, int *, int *);
+                                    int *, int *, int *, int *, int *);
 extern "C" void METIS_PartGraphRecursive(int *, int *, int *, int *, int *, int *,
-			    int *, int *, int *, int *, int *);
-
-
+                                         int *, int *, int *, int *, int *);
 extern "C" void METIS_mCPartGraphKway(int *, int *, int *, int *, int *, int *, int *,
-				      int *, int *, float *, int *, int *, int *);
+                                      int *, int *, float *, int *, int *, int *);
 extern "C" void METIS_mCPartGraphRecursive(int *, int *, int *, int *, int *, int *, int *,
-				      int *, int *, int *, int *, int *);
+                                           int *, int *, int *, int *, int *);
+#elif GETFEM_HAVE_METIS
+# include <metis.h>
+#endif
 
-/* some Getfem++ types that we will be using */
+using std::endl; using std::cout; using std::cerr;
+
+/* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_vector;
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
@@ -90,7 +93,7 @@ public:
     sizes_.resize(1); sizes_[0] = short_type(N);
     mf.extend_vector(U_, U);
   }
-  const bgeot::multi_index &sizes() const {  return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
     size_type cv = ctx.convex_num();
@@ -121,7 +124,7 @@ public:
     sizes_.resize(1); sizes_[0] = 1;
     cv_old = size_type(-1); h = 0.;
   }
-  const bgeot::multi_index &sizes() const {  return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
     size_type cv = ctx.convex_num();
@@ -154,7 +157,7 @@ public:
     sizes_.resize(1); sizes_[0] = short_type(N);
     mf.extend_vector(U_, U);
   }
-  const bgeot::multi_index &sizes() const {  return sizes_; }
+  const bgeot::multi_index &sizes(size_type) const {  return sizes_; }
   virtual void compute(getfem::fem_interpolation_context& ctx,
 		       bgeot::base_tensor &t) {
     size_type cv = ctx.convex_num();
@@ -438,7 +441,11 @@ void asm_patch_vector
 template<class MAT>
 void asm_stabilization_patch_term
 (const MAT &RM_, const getfem::mesh &mesh, const getfem::mesh_im &mimbounddown, const getfem::mesh_fem &mf_mult,
- scalar_type ratio_size, scalar_type h ){
+ scalar_type ratio_size, scalar_type
+#ifdef GETFEM_HAVE_METIS
+h
+#endif
+){
   MAT &M1 = const_cast<MAT &>(RM_);
   
   /****************************************************/
@@ -490,30 +497,43 @@ void asm_stabilization_patch_term
   }
   
   xadj[j] = k;
-  std::vector<int> adjwgt(k);
   // cout<<"xadj="<<xadj<<endl;
   //cout<<"adjncy="<<adjncy<<endl;
   //cout<<"vwgt="<<vwgt<<endl;
   
   cout<<"ratio size beween mesh and coarse mesh= "<< ratio_size <<endl;
   
-  int wgtflag = 2, edgecut, nparts=int(size_of_crack/(ratio_size*h)), numflag = 0;
-      // float ubvec[1] = {1.03f};
+  int nparts = 1;
+#ifdef GETFEM_HAVE_METIS
+  nparts = int(size_of_crack/(ratio_size*h));
+# ifdef GETFEM_HAVE_METIS_OLD_API
+  std::vector<int> adjwgt(k); // actually Metis would also accept NULL instead of an empty array
+  int wgtflag = 2, numflag = 0, edgecut;
   int  options[5] = {0,0,0,0,0};
+  // float ubvec[1] = {1.03f};
   //METIS_mCPartGraphKway(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //		    &numflag, &nparts, &(ubvec[0]),  options, &edgecut, &(part[0]));
-  // METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
+  //METIS_mCPartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //			 &numflag, &nparts,  options, &edgecut, &(part[0]));
   //METIS_PartGraphKway(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
   //	  &numflag, &nparts, options, &edgecut, &(part[0]));
   METIS_PartGraphRecursive(&ne, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), &(adjwgt[0]), &wgtflag,
-			   &numflag, &nparts, options, &edgecut, &(part[0]));
-  
+                           &numflag, &nparts, options, &edgecut, &(part[0]));
+# else
+  int ncon = 1, edgecut;
+  int options[METIS_NOPTIONS] = { 0 };
+  METIS_SetDefaultOptions(options);
+  METIS_PartGraphRecursive(&ne, &ncon, &(xadj[0]), &(adjncy[0]), &(vwgt[0]), 0, 0,
+                           &nparts, 0, 0, options, &edgecut, &(part[0]));
+# endif
   //cout<<"size_of_mesh="<<h<<endl;
   cout<<"size_of_crack="<< size_of_crack <<endl;
   cout<<"nb_partition="<<nparts<<endl;
   cout<<"partition="<<part<<endl;
   cout<<"edgecut="<<edgecut<<endl;
+#else
+  GMM_ASSERT1(false, "METIS not linked");
+#endif
   
 
   /**************************************************************/
@@ -1027,26 +1047,24 @@ bool  unilateral_contact_problem::solve(plain_vector &U, plain_vector &LAMBDA, p
   cout << dgr << endl;
   
   std::vector<getfem::pglobal_function> vfunc(dgr*4);
-  //std::vector<getfem::pglobal_function> vfunc_u(6);
-  //std::vector<getfem::pglobal_function> vfunc_p(dgrp*2);
+
   if (!load_global_fun) {
     std::cout << "Using default singular functions\n";
     for (unsigned i = 0; i < vfunc.size(); ++i){
       /* use the singularity */
       
-      getfem::abstract_xy_function *s = 
-	new getfem::crack_singular_xy_function(i);
+      getfem::pxy_function
+	s = std::make_shared<getfem::crack_singular_xy_function>(i);
       
       if (enrichment_option != FIXED_ZONE ) {
 	/* use the product of the singularity function
 	   with a cutoff */
-	getfem::abstract_xy_function *c = 
-	  new getfem::cutoff_xy_function(int(cutoff_func),
-					 cutoff_radius, 
-					 cutoff_radius1,cutoff_radius0);
-	s  = new getfem::product_of_xy_functions(*s, *c);
+	getfem::pxy_function
+	  c = std::make_shared<getfem::cutoff_xy_function>
+	  (int(cutoff_func), cutoff_radius, cutoff_radius1,cutoff_radius0);
+	s  = std::make_shared<getfem::product_of_xy_functions>(s, c);
        }
-      vfunc[i]=getfem::global_function_on_level_set(ls, *s);           
+      vfunc[i]=getfem::global_function_on_level_set(ls, s);     
     }
   }
   
@@ -1289,11 +1307,11 @@ bool  unilateral_contact_problem::solve(plain_vector &U, plain_vector &LAMBDA, p
 						  "augmentation_parameter", BN, MA);
     }else{
       if (Tresca_version){
-	getfem::add_Hughes_stab_with_friction_contact_brick
+	getfem::add_Hughes_stab_basic_contact_brick
 	  (model, "u", "Lambda", "Lambda_t", "augmentation_parameter",
 	   BN, BT, MA, MAT, "Friction_coeff","","",1, Tresca_version, "Tresca_threshold");
       }else{
-	getfem::add_Hughes_stab_with_friction_contact_brick
+	getfem::add_Hughes_stab_basic_contact_brick
 	  (model, "u", "Lambda", "Lambda_t", "augmentation_parameter",
 	   BN, BT, MA, MAT, "Friction_coeff");
       }
@@ -1305,11 +1323,11 @@ bool  unilateral_contact_problem::solve(plain_vector &U, plain_vector &LAMBDA, p
 				      "augmentation_parameter", BN);
     }else{
       if (Tresca_version){
-     	getfem::add_basic_contact_with_friction_brick
+     	getfem::add_basic_contact_brick
 	  (model, "u", "Lambda", "Lambda_t",
 	   "augmentation_parameter", BN, BT, "Friction_coeff","","",1,Tresca_version,"Tresca_threshold");
       }else{
-	getfem::add_basic_contact_with_friction_brick
+	getfem::add_basic_contact_brick
 	  (model, "u", "Lambda", "Lambda_t",
 	   "augmentation_parameter", BN, BT, "Friction_coeff");
       }
@@ -1333,7 +1351,7 @@ bool  unilateral_contact_problem::solve(plain_vector &U, plain_vector &LAMBDA, p
 //       getfem::add_explicit_matrix(model, "u", "u", KAT);
 //       // Defining the contact condition.
 //       gmm::add(CAT, BT); 
-//       getfem::add_Hughes_stab_with_friction_contact_brick
+//       getfem::add_Hughes_stab_basic_contact_brick
 // 	(model, "u", "Lambda", "Lambda_t", "augmentation_parameter",
 // 	 BN, BT, MA, MAT, "friction_coeff");
 //     }
@@ -1342,7 +1360,7 @@ bool  unilateral_contact_problem::solve(plain_vector &U, plain_vector &LAMBDA, p
 // 				    "augmentation_parameter", BN);
     
 //     if (!contact_only){
-//       getfem::add_basic_contact_with_friction_brick
+//       getfem::add_basic_contact_brick
 // 	(model, "u", "Lambda", "Lambda_t",
 // 	 "augmentation_parameter", BN, BT, "friction_coeff","","",0);
       
