@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -18,7 +18,7 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: getfem_interface.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id: getfem_interface.cc 5353 2016-07-26 13:36:19Z renard $
 #include <getfem_interface.h>
 #include <getfemint.h>
 
@@ -26,7 +26,6 @@ using namespace getfemint;
 
 void gf_workspace(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
 void gf_delete(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
-void gf_undelete(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
 void gf_eltm(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
 void gf_geotrans(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
 void gf_geotrans_get(getfemint::mexargs_in& in, getfemint::mexargs_out& out);
@@ -127,7 +126,6 @@ char* getfem_interface_main(int config_id, const char *function,
   if (subc_tab.size() == 0) {
     subc_tab["workspace"] = gf_workspace;
     subc_tab["delete"] = gf_delete;
-    subc_tab["undelete"] = gf_undelete;
     subc_tab["eltm"] = gf_eltm;
     subc_tab["geotrans"] = gf_geotrans;
     subc_tab["geotrans_get"] = gf_geotrans_get;
@@ -213,7 +211,7 @@ char* getfem_interface_main(int config_id, const char *function,
     std::copy(out.args().begin(), out.args().end(), *pout_args);
   }
   catch (getfemint_bad_arg e) {
-    //cerr << "Bad argument!\n";
+    // cerr << "Bad argument!\n";
     return strdup(e.what());
   }
   catch (getfemint_interrupted) {
@@ -221,6 +219,7 @@ char* getfem_interface_main(int config_id, const char *function,
     return strdup("Interrupted (Ctrl-C)");
   }
   catch (getfemint_error e) {
+    // cerr << "Error!\n";
     return strdup(e.what());
   }
   catch (std::logic_error e) {
