@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -23,7 +23,7 @@
 #include <getfem/getfem_derivatives.h>
 #include <getfem/getfem_interpolation.h>
 #include <getfem/getfem_assembling.h>
-#include <getfemint_mesh_slice.h>
+#include <getfem/getfem_mesh_slice.h>
 #include <getfem/getfem_error_estimate.h>
 #include <getfem/getfem_convect.h>
 using namespace getfemint;
@@ -115,8 +115,8 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
 	       const getfem::mesh_fem& mf, const garray<T> &U) {
   array_dimensions dims;
   dims.push_back(U,0,U.ndim()-1,true);
-  if (in.front().is_mesh_fem()) {
-    const getfem::mesh_fem& mf_dest = *in.pop().to_const_mesh_fem();
+  if (is_meshfem_object(in.front())) {
+      const getfem::mesh_fem& mf_dest = *to_meshfem_object(in.pop());
     error_for_non_lagrange_elements(mf_dest, true);
     size_type qmult = mf.get_qdim() / mf_dest.get_qdim();
     if (qmult == 0)
@@ -129,9 +129,8 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
     garray<T> V = out.pop().create_array(dims,T());
     getfem::interpolation(mf, mf_dest, U, V);
   }
-  else if (in.front().is_mesh_slice()) {
-    getfem::stored_mesh_slice *sl =
-      &in.pop().to_getfemint_mesh_slice()->mesh_slice();
+  else if (is_slice_object(in.front())) {
+    getfem::stored_mesh_slice *sl = to_slice_object(in.pop());
 
     for (size_type i=0; i < sl->nb_convex(); ++i)
       if (!mf.linked_mesh().convex_index().is_in(sl->convex_num(i)))
@@ -201,7 +200,7 @@ struct sub_gf_compute : virtual public dal::static_stored_object {
 		   rcarray U) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_compute> psub_command;
+typedef std::shared_ptr<sub_gf_compute> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -213,7 +212,7 @@ template <typename T> static inline void dummy_func(T &) {}
 		       const getfem::mesh_fem *mf, rcarray U)		\
       { dummy_func(in); dummy_func(out); code }				\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
@@ -237,7 +236,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("L2 norm", 1, 2, 0, 1,
        U_is_a_vector(U, "L2 norm");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
        dal::bit_vector bv = in.remaining() ?
        in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
        if (!U.is_complex())
@@ -253,8 +252,8 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("L2 dist", 3, 4, 0, 1,
        U_is_a_vector(U, "L2 dist");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
-       const getfem::mesh_fem *mf_2 = in.pop().to_const_mesh_fem();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
+       const getfem::mesh_fem *mf_2 = to_meshfem_object(in.pop());
        if (!U.is_complex()) {
          darray st = in.pop().to_darray();
          std::vector<double> V(st.begin(), st.end());
@@ -284,7 +283,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("H1 semi norm", 1, 2, 0, 1,
        U_is_a_vector(U, "H1 semi norm");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
        dal::bit_vector bv = in.remaining() ?
        in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
        if (!U.is_complex())
@@ -303,8 +302,8 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("H1 semi dist", 3, 4, 0, 1,
        U_is_a_vector(U, "H1 semi dist");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
-       const getfem::mesh_fem *mf_2 = in.pop().to_const_mesh_fem();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
+       const getfem::mesh_fem *mf_2 = to_meshfem_object(in.pop());
        if (!U.is_complex()) {
          darray st = in.pop().to_darray();
          std::vector<double> V(st.begin(), st.end());
@@ -334,7 +333,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("H1 norm", 1, 2, 0, 1,
        U_is_a_vector(U, "H1 norm");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
        dal::bit_vector bv = in.remaining() ?
        in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
        if (!U.is_complex())
@@ -352,7 +351,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("H2 semi norm", 1, 2, 0, 1,
        U_is_a_vector(U, "H2 semi norm");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
        dal::bit_vector bv = in.remaining() ?
        in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
        if (!U.is_complex())
@@ -371,7 +370,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("H2 norm", 1, 2, 0, 1,
        U_is_a_vector(U, "H2 norm");
-       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_im *mim = to_meshim_object(in.pop());
        dal::bit_vector bv = in.remaining() ?
        in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
        if (!U.is_complex())
@@ -399,7 +398,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     `mf` and Qdim_mf_du is the Qdim of `mf_du`.@*/
     sub_command
       ("gradient", 1, 1, 0, 1,
-       const getfem::mesh_fem *mf_grad = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_grad = to_meshfem_object(in.pop());
        error_for_non_lagrange_elements(*mf_grad, true);
        size_type qm
          = (mf_grad->get_qdim() == mf->get_qdim()) ? 1 : mf->get_qdim();
@@ -415,7 +414,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     See also ::COMPUTE('gradient', @tmf mf_du).@*/
     sub_command
       ("hessian", 1, 1, 0, 1,
-       const getfem::mesh_fem *mf_hess = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_hess = to_meshfem_object(in.pop());
        error_for_non_lagrange_elements(*mf_hess, true);
        size_type qm = (mf_hess->get_qdim() == mf->get_qdim()) ? 1 : mf->get_qdim();
        if (!U.is_complex())
@@ -488,7 +487,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     See also ::ASM('extrapolation matrix')@*/
     sub_command
       ("extrapolate on", 1, 1, 0, 1,
-       const getfem::mesh_fem *mf_dest = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_dest = to_meshfem_object(in.pop());
        error_for_non_lagrange_elements(*mf_dest, true);
        if (!U.is_complex()) {
 	 darray V = out.pop().create_darray(1, unsigned(mf_dest->nb_dof()));
@@ -507,14 +506,20 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     the jump of the normal derivative is integrated on its faces.@*/
     sub_command
       ("error_estimate", 1, 1, 0, 1,
-       const getfem::mesh_im &mim = *in.pop().to_const_mesh_im();
+       const getfem::mesh_im &mim = *(to_meshim_object(in.pop()));
        darray err =
        out.pop().create_darray_h
-               (unsigned(mim.linked_mesh().convex_index().last_true()+1));
+       (unsigned(mim.linked_mesh().convex_index().last_true()+1));
        if (!U.is_complex())
 	 getfem::error_estimate(mim, *mf, U.real(), err, mim.convex_index());
-       else
-	 getfem::error_estimate(mim, *mf, U.cplx(), err, mim.convex_index());
+       else {
+	 getfem::base_vector err_imag(gmm::vect_size(err));
+	 getfem::error_estimate(mim, *mf, gmm::imag_part(U.cplx()), err_imag,
+				mim.convex_index());
+	 getfem::error_estimate(mim, *mf, gmm::real_part(U.cplx()), err,
+				mim.convex_index());
+	 gmm::add(err_imag, err);
+       }
        );
 
       
@@ -527,7 +532,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     the jump of the normal derivative is integrated on its faces.@*/
     sub_command
       ("error_estimate_nitsche", 8, 8, 0, 1,
-       const getfem::mesh_im &mim = *in.pop().to_const_mesh_im();
+       const getfem::mesh_im &mim = *to_meshim_object(in.pop());
        int GAMMAC = in.pop().to_integer();
        int GAMMAN = in.pop().to_integer();
        scalar_type lambda = in.pop().to_scalar();
@@ -569,7 +574,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     @*/
     sub_command
       ("convect", 4, 7, 0, 0,
-       const getfem::mesh_fem *mf_v = in.pop().to_const_mesh_fem();
+       const getfem::mesh_fem *mf_v = to_meshfem_object(in.pop());
        rcarray V              = in.pop().to_rcarray();
        scalar_type dt = in.pop().to_scalar();
        size_type nt = in.pop().to_integer(0,100000);
@@ -614,7 +619,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
   
   if (m_in.narg() < 3)  THROW_BADARG( "Wrong number of input arguments");
 
-  const getfem::mesh_fem *mf   = m_in.pop().to_const_mesh_fem();
+  const getfem::mesh_fem *mf   = to_meshfem_object(m_in.pop());
   rcarray U              = m_in.pop().to_rcarray();
   m_in.last_popped().check_trailing_dimension(int(mf->nb_dof()));
   std::string init_cmd   = m_in.pop().to_string();
