@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2009-2015 Luis Saavedra.
+ Copyright (C) 2009-2016 Luis Saavedra.
 
  This file is a part of GetFEM++
 
@@ -18,11 +18,10 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_global_function.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id: gf_global_function.cc 5353 2016-07-26 13:36:19Z renard $
 #include <getfemint.h>
 #include <getfemint_workspace.h>
-#include <getfemint_global_function.h>
-#include <getfem/getfem_mesh_fem_global_function.h>
+#include <getfem/getfem_global_function.h>
 #include <getfem/getfem_arch_config.h>
 
 using namespace getfemint;
@@ -41,16 +40,14 @@ using namespace getfemint;
 
 // Object for the declaration of a new sub-command.
 
-typedef getfemint_global_function *pgetfemint_global_function;
-
 struct sub_gf_globfunc : virtual public dal::static_stored_object {
   int arg_in_min, arg_in_max, arg_out_min, arg_out_max;
   virtual void run(getfemint::mexargs_in& in,
 		   getfemint::mexargs_out& out,
-		   pgetfemint_global_function &ggf) = 0;
+		   getfem::pxy_function &ggf) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_globfunc> psub_command;
+typedef std::shared_ptr<sub_gf_globfunc> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -59,20 +56,16 @@ template <typename T> static inline void dummy_func(T &) {}
     struct subc : public sub_gf_globfunc {				\
       virtual void run(getfemint::mexargs_in& in,			\
 		       getfemint::mexargs_out& out,			\
-		       pgetfemint_global_function &ggf)			\
+		       getfem::pxy_function &ggf)			\
       { dummy_func(in); dummy_func(out); code }				\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
   }
 
 
-
-
-
-
 void gf_global_function(getfemint::mexargs_in& m_in,
 			getfemint::mexargs_out& m_out) {
   typedef std::map<std::string, psub_command > SUBC_TAB;
@@ -90,8 +83,7 @@ void gf_global_function(getfemint::mexargs_in& m_in,
        scalar_type r1 = in.pop().to_scalar();
        scalar_type r0 = in.pop().to_scalar();
 
-       getfem::abstract_xy_function *cutoff = new getfem::cutoff_xy_function(int(fn),r,r1,r0);
-       ggf = getfemint_global_function::get_from(cutoff);
+       ggf = std::make_shared<getfem::cutoff_xy_function>(int(fn),r,r1,r0);
        );
 
 
@@ -100,9 +92,7 @@ void gf_global_function(getfemint::mexargs_in& m_in,
     sub_command
       ("crack", 1, 1, 0, 1,
        size_type fn = in.pop().to_integer(0,11);
-       getfem::abstract_xy_function *crack
-       = new getfem::crack_singular_xy_function(unsigned(fn));
-       ggf = getfemint_global_function::get_from(crack);
+       ggf = std::make_shared<getfem::crack_singular_xy_function>(unsigned(fn));
        );
 
     /*@INIT GF = ('parser', @str val[, @str grad[, @str hess]])
@@ -114,22 +104,20 @@ void gf_global_function(getfemint::mexargs_in& m_in,
        std::string sval = in.pop().to_string();
        std::string sgrad = "[0;0]";
        std::string shess = "[0,0;0,0]";
-       if (in.remaining() && in.front().is_string()) sgrad = in.pop().to_string();
-       if (in.remaining() && in.front().is_string()) shess = in.pop().to_string();
-
-       getfem::abstract_xy_function *parser = new getfem::parser_xy_function(sval,sgrad,shess);
-       ggf = getfemint_global_function::get_from(parser);
+       if (in.remaining() && in.front().is_string())
+	 sgrad = in.pop().to_string();
+       if (in.remaining() && in.front().is_string())
+	 shess = in.pop().to_string();
+       ggf = std::make_shared<getfem::parser_xy_function>(sval,sgrad,shess);
        );
 
     /*@INIT GF = ('product', @tgf F, @tgf G)
       Create a product of two global functions.@*/
     sub_command
       ("product", 2, 2, 0, 1,
-       getfem::abstract_xy_function *af1 = in.pop().to_global_function();
-       getfem::abstract_xy_function *af2 = in.pop().to_global_function();
-
-       getfem::abstract_xy_function *product = new getfem::product_of_xy_functions(*af1,*af2);
-       ggf = getfemint_global_function::get_from(product);
+       getfem::pxy_function af1 = to_global_function_object(in.pop());
+       getfem::pxy_function af2 = to_global_function_object(in.pop());
+       ggf = std::make_shared<getfem::product_of_xy_functions>(af1,af2);
        );
 
 
@@ -137,11 +125,9 @@ void gf_global_function(getfemint::mexargs_in& m_in,
       Create a add of two global functions.@*/
     sub_command
       ("add", 2, 2, 0, 1,
-       getfem::abstract_xy_function *af1 = in.pop().to_global_function();
-       getfem::abstract_xy_function *af2 = in.pop().to_global_function();
-       getfem::abstract_xy_function *add
-       = new getfem::add_of_xy_functions(*af1,*af2);
-       ggf = getfemint_global_function::get_from(add);
+       getfem::pxy_function af1 = to_global_function_object(in.pop());
+       getfem::pxy_function af2 = to_global_function_object(in.pop());
+       ggf = std::make_shared<getfem::add_of_xy_functions>(af1,af2);
        );
 
   }
@@ -150,7 +136,7 @@ void gf_global_function(getfemint::mexargs_in& m_in,
 
   if (m_in.narg() < 1)  THROW_BADARG( "Wrong number of input arguments");
 
-  getfemint_global_function *ggf = NULL;
+  getfem::pxy_function ggf;
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
@@ -164,8 +150,8 @@ void gf_global_function(getfemint::mexargs_in& m_in,
   }
   else bad_cmd(init_cmd);
 
-  m_out.pop().from_object_id(ggf->get_id(), GLOBAL_FUNCTION_CLASS_ID);
-
+  m_out.pop().from_object_id(store_global_function_object(ggf),
+			     GLOBAL_FUNCTION_CLASS_ID);
 }
 
 
