@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -19,12 +19,11 @@
 
 ===========================================================================*/
 
+#include <getfem/getfem_mesh_level_set.h>
+#include <getfem/getfem_mesh_im_level_set.h>
+#include <getfem/getfem_integration.h>
 #include <getfemint_misc.h>
 #include <getfemint_workspace.h>
-#include <getfemint_mesh_im.h>
-#include <getfemint_mesh.h>
-#include <getfemint_mesh_levelset.h>
-#include <getfem/getfem_mesh_im_level_set.h>
 
 using namespace getfemint;
 
@@ -38,16 +37,15 @@ void gf_mesh_im_set_integ(getfem::mesh_im *mim, getfemint::mexargs_in& in);
 
 // Object for the declaration of a new sub-command.
 
-typedef  getfemint_mesh_im *pgetfemint_mesh_im;
-
 struct sub_gf_mim : virtual public dal::static_stored_object {
   int arg_in_min, arg_in_max, arg_out_min, arg_out_max;
   virtual void run(getfemint::mexargs_in& in,
 		   getfemint::mexargs_out& out,
-		   getfemint_mesh *mm, pgetfemint_mesh_im &mim) = 0;
+		   const getfem::mesh * &mm,
+		   std::shared_ptr<getfem::mesh_im> &mim) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_mim> psub_command;
+typedef std::shared_ptr<sub_gf_mim> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -56,10 +54,11 @@ template <typename T> static inline void dummy_func(T &) {}
     struct subc : public sub_gf_mim {					\
       virtual void run(getfemint::mexargs_in& in,			\
 		       getfemint::mexargs_out& out,			\
-		       getfemint_mesh *mm, pgetfemint_mesh_im &mim)	\
+		       const getfem::mesh * &mm,			\
+		       std::shared_ptr<getfem::mesh_im> &mim)		\
       { dummy_func(in); dummy_func(out); dummy_func(mm); code }		\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
@@ -84,14 +83,18 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("load", 1, 2, 0, 1,
        std::string fname = in.pop().to_string();
-       if (in.remaining()) mm = in.pop().to_getfemint_mesh();
-       else {
-	 getfem::mesh *m = new getfem::mesh();
-	 m->read_from_file(fname);
-	 mm = getfemint_mesh::get_from(m);
+       if (in.remaining()) {
+	 mm = extract_mesh_object(in.pop());
+	 mim = std::make_shared<getfem::mesh_im>(*mm);
+       } else {
+	  auto m = std::make_shared<getfem::mesh>();
+	  m->read_from_file(fname);
+	  store_mesh_object(m);
+	  mm = m.get();
+	  mim = std::make_shared<getfem::mesh_im>(*mm);
+	  workspace().add_hidden_object(store_meshim_object(mim), m);
        }
-       mim = getfemint_mesh_im::new_from(mm);
-       mim->mesh_im().read_from_file(fname);
+       mim->read_from_file(fname);
        );
 
 
@@ -102,14 +105,18 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
     sub_command
       ("from string", 1, 2, 0, 1,
        std::stringstream ss(in.pop().to_string());
-       if (in.remaining()) mm = in.pop().to_getfemint_mesh();
-       else {
-	 getfem::mesh *m = new getfem::mesh();
+       if (in.remaining()) {
+	 mm = extract_mesh_object(in.pop());
+	 mim = std::make_shared<getfem::mesh_im>(*mm);
+       } else {
+	 auto m = std::make_shared<getfem::mesh>();
 	 m->read_from_file(ss);
-	 mm = getfemint_mesh::get_from(m);
+	 store_mesh_object(m);
+	 mm = m.get();
+	 mim = std::make_shared<getfem::mesh_im>(*mm);
+	 workspace().add_hidden_object(store_meshim_object(mim), m);
        }
-       mim = getfemint_mesh_im::new_from(mm);
-       mim->mesh_im().read_from_file(ss);
+       mim->read_from_file(ss);
        );
 
 
@@ -117,12 +124,9 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       Create a copy of a @tmim.@*/
     sub_command
       ("clone", 1, 1, 0, 1,
-       getfemint_mesh_im *mim2 = in.pop().to_getfemint_mesh_im();
-       mm = object_to_mesh(workspace().object(mim2->linked_mesh_id()));
-       mim = getfemint_mesh_im::new_from(mm);
-       std::stringstream ss; /* not very elegant ! */
-       mim2->mesh_im().write_to_file(ss);
-       mim->mesh_im().read_from_file(ss);
+       getfem::mesh_im *mim2 = to_meshim_object(in.pop());
+       mm = &mim2->linked_mesh();
+       mim = std::make_shared<getfem::mesh_im>(*mim2);
        );
 
 
@@ -164,13 +168,13 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       @*/
     sub_command
       ("levelset", 3, 5, 0, 1,
-       getfemint_mesh_levelset *gmls = in.pop().to_getfemint_mesh_levelset();
+       getfem::mesh_level_set &mls = *(to_mesh_levelset_object(in.pop()));
        std::string swhere = in.pop().to_string();
-       getfem::pintegration_method pim  = in.pop().to_integration_method();
+       getfem::pintegration_method pim  = to_integ_object(in.pop());
        getfem::pintegration_method pim2 = 0;
        getfem::pintegration_method pim3 = 0;
-       if (in.remaining()) pim2 = in.pop().to_integration_method();
-       if (in.remaining()) pim3 = in.pop().to_integration_method();
+       if (in.remaining()) pim2 = to_integ_object(in.pop());
+       if (in.remaining()) pim3 = to_integ_object(in.pop());
        int where = 0;
        std::string csg_description;
        if (cmd_strmatch(swhere, "all"))
@@ -202,10 +206,9 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        if (pim->type() != getfem::IM_APPROX) {
  	 THROW_BADARG("expecting an approximate integration method");
        }
-
-       getfem::mesh_im_level_set *mimls =
-       new getfem::mesh_im_level_set(gmls->mesh_levelset(),
- 				     where, pim, pim2);
+       
+       auto mimls = std::make_shared<getfem::mesh_im_level_set>(mls, where,
+								pim, pim2);
        if (pim3)
 	 mimls->set_integration_method(mimls->linked_mesh().convex_index(),
 				       pim3);
@@ -214,19 +217,20 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        if (csg_description.size()) {
  	 mimls->set_level_set_boolean_operations(csg_description);
        }
-       mim = getfemint_mesh_im::get_from(mimls);
+       mim = mimls;
        mimls->adapt();
-       workspace().set_dependance(mim, gmls);
+       mm = &mls.linked_mesh();
+       store_meshim_object(mim);
+       workspace().set_dependence(mim.get(), &mls);
        );
 
   }
 
 
   if (m_in.narg() < 1) THROW_BADARG("Wrong number of input arguments");
-  getfemint_mesh *mm = NULL;
-  getfemint_mesh_im *mim = NULL;
+  const getfem::mesh *mm = NULL;
+  std::shared_ptr<getfem::mesh_im> mim;
   if (m_in.front().is_string()) {
-
     std::string init_cmd   = m_in.pop().to_string();
     std::string cmd        = cmd_normalize(init_cmd);
 
@@ -249,13 +253,17 @@ void gf_mesh_im(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       with these arguments.@*/
     if (!m_out.narg_in_range(1, 1))
       THROW_BADARG("Wrong number of output arguments");
-    mm = m_in.pop().to_getfemint_mesh();
-    mim = getfemint_mesh_im::new_from(mm);
+    mm = extract_mesh_object(m_in.pop());
+    mim = std::make_shared<getfem::mesh_im>(*mm);
     if (m_in.remaining()) {
-      gf_mesh_im_set_integ(&mim->mesh_im(), m_in);
+      gf_mesh_im_set_integ(mim.get(), m_in);
     }
     if (m_in.remaining()) THROW_BADARG("Wrong number of input arguments");
-  }
+    
 
-  m_out.pop().from_object_id(mim->get_id(), MESHIM_CLASS_ID);
+  }
+  if (!mim.get()) THROW_INTERNAL_ERROR;
+  id_type id = store_meshim_object(mim);
+  workspace().set_dependence(id, mm);
+  m_out.pop().from_object_id(id, MESHIM_CLASS_ID);
 }
