@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -18,14 +18,14 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_slice.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id: gf_slice.cc 5353 2016-07-26 13:36:19Z renard $
+
+#include <memory>
+#include <getfem/getfem_mesh_level_set.h>
+#include <getfem/getfem_mesh_slice.h>
+#include <getfem/getfem_mesh_fem.h>
 #include <getfemint_misc.h>
 #include <getfemint_workspace.h>
-#include <getfemint_mesh.h>
-#include <getfemint_mesh_fem.h>
-#include <getfemint_mesh_slice.h>
-#include <getfemint_mesh_levelset.h>
-#include <memory>
 
 
 using namespace getfemint;
@@ -142,7 +142,7 @@ namespace getfem {
       do {
         if (change_convex) {
           //cerr << "changement de convexe, ancien=" << cv << endl;
-          /* init convex-dependant data */
+          /* init convex-dependent data */
           if (!first) {
 
             cv = find_convex_of_point(mfU->pmf->linked_mesh(), cv, P0, refP0, gti);
@@ -219,74 +219,74 @@ namespace getfem {
 }
 
 static getfem::slicer_action*
-build_slicers(const getfem::mesh& m, dal::ptr_collection<getfem::slicer_action> & slicers,
+build_slicers(const getfem::mesh& m, std::vector<std::unique_ptr<getfem::slicer_action>> &slicers,
               const gfi_array *arg) {
   GMM_ASSERT1(gfi_array_get_class(arg) == GFI_CELL, "slices must be "
               "described as imbricated cell arrays");
   mexargs_in in(1, &arg, true);
   std::string cmd = in.pop().to_string();
   if (check_cmd(cmd, "none", in, 0, 0)) {
-    slicers.push_back(new getfem::slicer_none());
+    slicers.push_back(std::make_unique<getfem::slicer_none>());
   } else if (check_cmd(cmd, "planar", in, 3, 3)) {
     int orient = in.pop().to_integer(-1,2);
     getfem::base_node x0 = in.pop().to_base_node();
-    slicers.push_back(new getfem::slicer_half_space(x0, in.pop().to_base_node(), orient));
+    slicers.push_back(std::make_unique<getfem::slicer_half_space>(x0, in.pop().to_base_node(), orient));
   } else if (check_cmd(cmd, "ball", in, 3, 3)) {
     int orient = in.pop().to_integer(-1,2);
     getfem::base_node x0 = in.pop().to_base_node();
-    slicers.push_back(new getfem::slicer_sphere(x0, in.pop().to_scalar(1e-5), orient));
+    slicers.push_back(std::make_unique<getfem::slicer_sphere>(x0, in.pop().to_scalar(1e-5), orient));
   } else if (check_cmd(cmd, "cylinder", in, 4, 4)) {
     int orient = in.pop().to_integer(-1,2);
     getfem::base_node x0 = in.pop().to_base_node();
     getfem::base_node x1 = in.pop().to_base_node();
-    slicers.push_back(new getfem::slicer_cylinder(x0, x1, in.pop().to_scalar(1e-5), orient));
+    slicers.push_back(std::make_unique<getfem::slicer_cylinder>(x0, x1, in.pop().to_scalar(1e-5), orient));
   } else if (check_cmd(cmd, "isovalues", in, 4, 4)) {
     int orient = in.pop().to_integer(-1,2);
-    const getfem::mesh_fem &mf = *in.pop().to_const_mesh_fem();
+    const getfem::mesh_fem &mf = *to_meshfem_object(in.pop());
     darray U = in.pop().to_darray(int(mf.nb_dof()));
-    slicers.push_back(new getfem::slicer_isovalues(getfem::mesh_slice_cv_dof_data<darray>(mf,U),
+    slicers.push_back(std::make_unique<getfem::slicer_isovalues>(getfem::mesh_slice_cv_dof_data<darray>(mf,U),
                                                    in.pop().to_scalar(), orient));
   } else if (check_cmd(cmd, "boundary", in, 0, 1)) {
     getfem::slicer_action *s1 = 0;
     if (in.remaining()) {
       s1 = build_slicers(m, slicers, in.pop().arg);
     } else {
-      slicers.push_back(new getfem::slicer_none());
-      s1 = slicers.back();
+      slicers.push_back(std::make_unique<getfem::slicer_none>());
+      s1 = slicers.back().get();
     }
     getfem::mesh_region cvflst;
     getfem::outer_faces_of_mesh(m, m.convex_index(), cvflst);
-    slicers.push_back(new getfem::slicer_boundary(m,*s1,cvflst));
+    slicers.push_back(std::make_unique<getfem::slicer_boundary>(m,*s1,cvflst));
   } else if (check_cmd(cmd, "explode", in, 1, 1)) {
     scalar_type c = in.pop().to_scalar();
-    slicers.push_back(new getfem::slicer_explode(c));
+    slicers.push_back(std::make_unique<getfem::slicer_explode>(c));
   } else if (check_cmd(cmd, "union", in, 1, -1)) {
     getfem::slicer_action *s1 = build_slicers(m, slicers, in.pop().arg);
     while (in.remaining()) {
       getfem::slicer_action *s2 = build_slicers(m, slicers, in.pop().arg);
-      slicers.push_back(new getfem::slicer_union(*s1,*s2));
-      s1 = slicers.back();
+      slicers.push_back(std::make_unique<getfem::slicer_union>(*s1,*s2));
+      s1 = slicers.back().get();
     }
   } else if (check_cmd(cmd, "intersection", in, 1, -1)) {
     getfem::slicer_action *s1 = build_slicers(m, slicers, in.pop().arg);
     while (in.remaining()) {
       getfem::slicer_action *s2 = build_slicers(m, slicers, in.pop().arg);
-      slicers.push_back(new getfem::slicer_intersect(*s1,*s2));
-      s1 = slicers.back();
+      slicers.push_back(std::make_unique<getfem::slicer_intersect>(*s1,*s2));
+      s1 = slicers.back().get();
     }
   } else if (check_cmd(cmd, "diff", in, 2, 2)) {
     getfem::slicer_action *s1 = build_slicers(m, slicers, in.pop().arg);
     getfem::slicer_action *s2 = build_slicers(m, slicers, in.pop().arg);
-    slicers.push_back(new getfem::slicer_complementary(*s2));
-    slicers.push_back(new getfem::slicer_intersect(*s1, *slicers.back()));
+    slicers.push_back(std::make_unique<getfem::slicer_complementary>(*s2));
+    slicers.push_back(std::make_unique<getfem::slicer_intersect>(*s1, *slicers.back()));
   } else if (check_cmd(cmd, "comp", in, 1, 1)) {
     getfem::slicer_action *s = build_slicers(m, slicers, in.pop().arg);
-    slicers.push_back(new getfem::slicer_complementary(*s));
+    slicers.push_back(std::make_unique<getfem::slicer_complementary>(*s));
   } else if (check_cmd(cmd, "mesh", in, 1, 1)) {
-    const getfem::mesh &m2 = *in.pop().to_const_mesh();
-    slicers.push_back(new getfem::slicer_mesh_with_mesh(m2));
+    const getfem::mesh &m2 = *extract_mesh_object(in.pop());
+    slicers.push_back(std::make_unique<getfem::slicer_mesh_with_mesh>(m2));
   } else bad_cmd(cmd);
-  return slicers.back();
+  return slicers.back().get();
 }
 
 
@@ -384,9 +384,9 @@ void gf_slice(getfemint::mexargs_in& in, getfemint::mexargs_out& out)
   if (in.narg()  <  2) THROW_BADARG("Wrong number of input arguments");
   if (!out.narg_in_range(1,1)) THROW_BADARG("Wrong number of output arguments");
 
-  getfemint_mesh *mm = 0;
+  const getfem::mesh *mm = 0;
   getfem::mesh_level_set *pmls = 0;
-  std::unique_ptr<getfem::stored_mesh_slice> pstored;
+  auto pstored = std::make_shared<getfem::stored_mesh_slice>();
 
   /* "normal" slices */
   if (in.front().is_cell()) {
@@ -397,38 +397,35 @@ void gf_slice(getfemint::mexargs_in& in, getfemint::mexargs_out& out)
     /* check the source argument (mesh/mesh_fem or slice) */
     std::unique_ptr<getfem::mesh_slice_cv_dof_data<darray> > mfdef;
     std::unique_ptr<getfem::slicer_action> slicer_def;
-    getfemint_mesh_slice *source_slice = 0;
-    if (in.front().is_mesh_fem() && in.remaining()  >=  3) {
-      mm = object_to_mesh(workspace().object(in.front().to_getfemint_mesh_fem()->linked_mesh_id()));
-      const getfem::mesh_fem& mf = *in.pop().to_const_mesh_fem();
+    getfem::stored_mesh_slice *source_slice = 0;
+    if (is_meshfem_object(in.front()) && in.remaining()  >=  3) {
+      const getfem::mesh_fem& mf = *to_meshfem_object(in.pop());
+      mm = &mf.linked_mesh();
       darray Udef = in.pop().to_darray(-2, int(mf.nb_dof()));
-      if (!(mf.get_qdim() == mm->mesh().dim() && Udef.getm() == 1) &&
-          !(mf.get_qdim() == 1 && Udef.getm() == mm->mesh().dim())) {
-        THROW_BADARG("either the mesh_fem must have a Qdim=" << int(mm->mesh().dim()) <<
-                     ", either the data must have " << int(mm->mesh().dim()) << " rows");
+      if (!(mf.get_qdim() == mm->dim() && Udef.getm() == 1) &&
+          !(mf.get_qdim() == 1 && Udef.getm() == mm->dim())) {
+        THROW_BADARG("either the mesh_fem must have a Qdim=" << int(mm->dim()) <<
+                     ", either the data must have " << int(mm->dim()) << " rows");
       }
       mfdef.reset(new getfem::mesh_slice_cv_dof_data<darray>(mf,Udef));
       slicer_def.reset(new getfem::slicer_apply_deformation(*mfdef.get()));
-    } else if (in.front().is_mesh_slice()) {
-      source_slice = in.pop().to_getfemint_mesh_slice(false);
-      mm = object_to_mesh(workspace().object(source_slice->linked_mesh_id()));
-    } else if (in.front().is_mesh_levelset()) {
-      pmls = &in.pop().to_getfemint_mesh_levelset()->mesh_levelset();
-      mm = getfemint_mesh::get_from(&pmls->linked_mesh());
+    } else if (is_slice_object(in.front())) {
+      source_slice = to_slice_object(in.pop());
+      mm = &(source_slice->linked_mesh());
+    } else if (is_mesh_levelset_object(in.front())) {
+      pmls = to_mesh_levelset_object(in.pop());
+      mm = &(pmls->linked_mesh());
     } else {
-      id_type id; in.pop().to_const_mesh(id); mm = object_to_mesh(workspace().object(id));
+      mm = extract_mesh_object(in.pop());
     }
 
-    dal::ptr_collection<getfem::slicer_action> slicers;
-    getfem::slicer_action * s = build_slicers(mm->mesh(), slicers, arg);
-
-    /* build the slice */
-    pstored.reset(new getfem::stored_mesh_slice());
+    std::vector<std::unique_ptr<getfem::slicer_action>> slicers;
+    getfem::slicer_action * s = build_slicers(*mm, slicers, arg);
 
     /* create the slicer and registers the actions */
-    getfem::mesh_slicer slicer(mm->mesh());
+    getfem::mesh_slicer slicer(*mm);
     if (pmls) slicer.using_mesh_level_set(*pmls);
-    getfem::slicer_build_stored_mesh_slice slicer_store(*pstored.get());
+    getfem::slicer_build_stored_mesh_slice slicer_store(*pstored);
     if (slicer_def.get()) slicer.push_back_action(*slicer_def.get());
     slicer.push_back_action(*s);
     slicer.push_back_action(slicer_store);
@@ -440,12 +437,12 @@ void gf_slice(getfemint::mexargs_in& in, getfemint::mexargs_out& out)
       /*std::vector<convex_face> cvf;
       if (in.remaining()) {
         iarray v = in.pop().to_iarray(-2, -1);
-        build_convex_face_lst(mm->mesh(), cvf, &v);
-        } else build_convex_face_lst(mm->mesh(), cvf, 0);*/
-      getfem::mesh_region rg = to_mesh_region(mm->mesh(), in);
+        build_convex_face_lst(*mm, cvf, &v);
+        } else build_convex_face_lst(*mm, cvf, 0);*/
+      getfem::mesh_region rg = to_mesh_region(*mm, in);
       slicer.exec(nrefine, rg);
     } else {
-      slicer.exec(source_slice->mesh_slice());
+      slicer.exec(*source_slice);
     }
     if (in.remaining()) THROW_BADARG("too much input arguments");
 
@@ -456,27 +453,28 @@ void gf_slice(getfemint::mexargs_in& in, getfemint::mexargs_out& out)
     /*@INIT sl = ('streamlines', @tmf mf, @mat U, @dmat S)
       Compute streamlines of the (vector) field `U`, with seed points given
       by the columns of `S`.@*/
-      const getfem::mesh_fem *mf = in.front().to_const_mesh_fem();
-      id_type id; in.pop().to_const_mesh(id); mm = object_to_mesh(workspace().object(id));
+      const getfem::mesh_fem *mf = to_meshfem_object(in.front());
+      mm = extract_mesh_object(in.pop());
       darray U = in.pop().to_darray(int(mf->nb_dof()));
-      darray v = in.pop().to_darray(mm->mesh().dim(), -1);
+      darray v = in.pop().to_darray(mm->dim(), -1);
       std::vector<getfem::base_node> seeds(v.getn());
       for (unsigned j=0; j < v.getn(); ++j)
         seeds[j] = v.col_to_bn(j);
       getfem::mesh_slice_cv_dof_data<darray> mfU(*mf,U);
-      pstored.reset(new getfem::mesh_slice_streamline(&mfU, seeds, true, true));
+      pstored = std::make_shared<getfem::mesh_slice_streamline>(&mfU, seeds,
+								true, true);
     } else if (check_cmd(cmd, "points", in, 2, 2)) {
     /*@INIT sl = ('points', @tmesh m, @dmat Pts)
       Return the "slice" composed of points given by the columns of `Pts`
       (useful for interpolation on a given set of sparse points, see
       ``::COMPUTE('interpolate on',sl)``.@*/
-      id_type id; in.pop().to_const_mesh(id); mm = object_to_mesh(workspace().object(id));
-      pstored.reset(new getfem::stored_mesh_slice());
-      getfem::mesh_slicer slicer(mm->mesh());
-      getfem::slicer_build_stored_mesh_slice slicer_store(*pstored.get());
+      mm = extract_mesh_object(in.pop());
+      pstored = std::make_shared<getfem::stored_mesh_slice>();
+      getfem::mesh_slicer slicer(*mm);
+      getfem::slicer_build_stored_mesh_slice slicer_store(*pstored);
       slicer.push_back_action(slicer_store);
 
-      darray w = in.pop().to_darray(mm->mesh().dim(), -1);
+      darray w = in.pop().to_darray(mm->dim(), -1);
       std::vector<getfem::base_node> N(w.getn());
       for (unsigned i=0; i < w.getn(); ++i) N[i] = w.col_to_bn(i);
       slicer.exec(N);
@@ -485,19 +483,22 @@ void gf_slice(getfemint::mexargs_in& in, getfemint::mexargs_out& out)
       Load the slice (and its linked mesh if it is not given as an argument)
       from a text file.@*/
       std::string fname = in.pop().to_string();
-      if (in.remaining()) mm = in.pop().to_getfemint_mesh();
+      pstored = std::make_shared<getfem::stored_mesh_slice>();
+      if (in.remaining()) mm = extract_mesh_object(in.pop());
       else {
-        getfem::mesh *m = new getfem::mesh();
-        m->read_from_file(fname);
-        mm = getfemint_mesh::get_from(m);
+	auto m = std::make_shared<getfem::mesh>();
+	m->read_from_file(fname);
+	store_mesh_object(m);
+	mm = m.get();
+	workspace().add_hidden_object(store_slice_object(pstored), m);
       }
-      pstored.reset(new getfem::stored_mesh_slice());
-      pstored->read_from_file(fname, mm->mesh());
+      pstored->read_from_file(fname, *mm);
     } else bad_cmd(cmd);
   } else THROW_BADARG("a slicer specification (i.e. cell array) or a string "
                       "was expected as the first argument");
   if (mm == 0 || pstored.get() == 0) THROW_INTERNAL_ERROR;
-  getfemint_mesh_slice *mms = new getfemint_mesh_slice(*mm, pstored.release());
-  out.pop().from_object_id(workspace().push_object(mms), SLICE_CLASS_ID);
-  workspace().set_dependance(mms, mm);
+
+  id_type id = store_slice_object(pstored);
+  out.pop().from_object_id(id, SLICE_CLASS_ID);
+  workspace().set_dependence(pstored.get(), mm);
 }
