@@ -1,4 +1,4 @@
-% Copyright (C) 2010-2015 Amandine Cottaz, Yves Renard, Farshid Dabaghi.
+% Copyright (C) 2010-2016 Amandine Cottaz, Yves Renard, Farshid Dabaghi.
 %
 % This file is a part of GetFEM++
 %
@@ -21,8 +21,7 @@ clc;
 
 
 
-% We compute a plasticity problem with a Von Mises criterion with or
-% without kinematic hardening
+% Plasticity problem with a Von Mises criterion with or without kinematic hardening
 % For convenience we consider an homogenous Dirichlet condition on the left
 % of the domain and an easy computed Neumann Condition on the right
 
@@ -115,8 +114,7 @@ end
 md = gfModel('real');
 
 % Declare that u is the unknown of the system on mf_u
-% 2 is the number of version of the data stored, for the time integration scheme 
-set(md, 'add fem variable', 'u', mf_u, 2);
+set(md, 'add fem variable', 'u', mf_u);
 
 % Declare that lambda is a data of the system on mf_data
 set(md, 'add initialized fem data', 'lambda', mf_data, lambda);
@@ -160,13 +158,13 @@ if (with_hardening)
     B, '*sigma), von_mises_threshold) + (H)*(((1-alpha)*',En,')+(alpha*', Enp1, '))))');
   
   gf_model_set(md, 'add nonlinear generic assembly brick', mim, strcat(expr_sigma, ':Grad_Test_u'));
-  % gf_model_set(md, 'add finite strain elasticity brick', mim, 'u', 'SaintVenant Kirchhoff', '[lambda; mu]');
+  % gf_model_set(md, 'add finite strain elasticity brick', mim, 'SaintVenant Kirchhoff', 'u', '[lambda; mu]');
 else
-    
+  gf_model_set(md, 'add fem data', 'previous_u', mf_u);
   % Declare that sigma is a data of the system on mf_sigma
   set(md, 'add fem data', 'sigma', mf_sigma);
   % Add plasticity brick on u
-  set(md, 'add elastoplasticity brick', mim, 'VM', 'u', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
+  set(md, 'add elastoplasticity brick', mim, 'VM', 'u', 'previous_u', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
 end
 
 % Add homogeneous Dirichlet condition to u on the left hand side of the domain
@@ -193,7 +191,7 @@ for step=1:size(t,2),
     % get(md, 'solve', 'noisy', 'max_iter', 80);
 
     % Retrieve the solution U
-    U = get(md, 'variable', 'u', 0);
+    U = get(md, 'variable', 'u');
     
     % Compute new plasticity constraints used to compute 
     % the Von Mises or Tresca stress
@@ -217,7 +215,7 @@ for step=1:size(t,2),
       Epsilon_u = gf_model_get(md, 'interpolation', '((Grad_u+Grad_u'')/2)', mim_data);
       gf_model_set(md, 'variable', 'u', U);
       ind_gauss_pt = 22500;
-      if (size(sigma, 2) <= N*(ind_gauss_pt + 1))
+      if (size(sigma, 2) <= N*N*(ind_gauss_pt + 1))
         ind_gauss_pt = floor(3*size(sigma, 2) / (4*N*N));
       end
       sigma_fig(1,step)=sigma(N*N*ind_gauss_pt + 1);
@@ -227,8 +225,8 @@ for step=1:size(t,2),
       gf_model_set(md, 'variable', 'sigma', sigma);
       gf_model_set(md, 'variable', 'Previous_u', U);
     else
-      get(md, 'elastoplasticity next iter', mim, 'u', 'VM', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
-      plast = get(md, 'compute plastic part', mim, mf_vm, 'u', 'VM', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
+      get(md, 'elastoplasticity next iter', mim, 'u', 'previous_u', 'VM', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
+      plast = get(md, 'compute plastic part', mim, mf_vm, 'u', 'previous_u', 'VM', 'lambda', 'mu', 'von_mises_threshold', 'sigma');
       % Compute Von Mises or Tresca stress
       VM = get(md, 'compute elastoplasticity Von Mises or Tresca', 'sigma', mf_vm, 'Von Mises');
     end
