@@ -1,8 +1,8 @@
-% Copyright (C) 2009-2012 Yves Renard.
+% Copyright (C) 2009-2016 Yves Renard.
 %
-% This file is a part of GETFEM++
+% This file is a part of GetFEM++
 %
-% Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+% GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
 % under  the  terms  of the  GNU  Lesser General Public License as published
 % by  the  Free Software Foundation;  either version 3 of the License,  or
 % (at your option) any later version along with the GCC Runtime Library
@@ -26,8 +26,8 @@ gf_workspace('clear all');
 clear all;
 
 % Import the mesh : disc
-m=gf_mesh('load', '../../../tests/meshes/disc_P2_h4.mesh');
-% m=gf_mesh('load', '../../../tests/meshes/disc_P2_h2.mesh');
+% m=gf_mesh('load', '../../../tests/meshes/disc_P2_h4.mesh');
+m=gf_mesh('load', '../../../tests/meshes/disc_P2_h2.mesh');
 % m=gf_mesh('load', '../../../tests/meshes/disc_P2_h1.mesh');
 % m=gf_mesh('load', '../../../tests/meshes/disc_P2_h0_5.mesh');
 % m=gf_mesh('load', '../../../tests/meshes/disc_P2_h0_3.mesh');
@@ -47,9 +47,16 @@ d = gf_mesh_get(m, 'dim'); % Mesh dimension
 clambda = 1;           % Lame coefficient
 cmu = 1;               % Lame coefficient
 friction_coeff = 0.4;  % coefficient of friction
-vertical_force = 0.05; % Volumic load in the vertical direction
-r = 1;                % Augmentation parameter
-condition_type = 2; % 0 = Explicitely kill horizontal rigid displacements
+vertical_force = 0.015; % Volumic load in the vertical direction
+u_degree = 2;
+lambda_degree = 2;
+incompressibility = 0;
+p_degree = 1;
+r = 40;                % Augmentation parameter
+gamma0 = 0.001;        % Nitsche's method gamma0 parameter
+theta = 1;             % Nitsche's method theta parameter
+
+condition_type = 1; % 0 = Explicitely kill horizontal rigid displacements
                     % 1 = Kill rigid displacements using a global penalization
                     % 2 = Add a Dirichlet condition on the top of the structure
 penalty_parameter = 1E-6;    % Penalization coefficient for the global penalization
@@ -63,12 +70,12 @@ end;
 
 niter = 100;   % Maximum number of iterations for Newton's algorithm.
 plot_mesh = true;
-version = 1;  % 1 : frictionless contact and the basic contact brick
+version = 1; % 1 : frictionless contact and the basic contact brick
               % 2 : contact with 'static' Coulomb friction and basic contact brick
               % 3 : frictionless contact and the contact with a
               %     rigid obstacle brick
-              % 4 : contact with 'static' Coulomb friction and the contact with a
-              %     rigid obstacle brick
+              % 4 : contact with 'static' Coulomb friction and the contact
+              %     with a rigid obstacle brick
               % 5 : frictionless contact and the integral brick
               %     Newton and Alart-Curnier augmented lagrangian,
               %     unsymmetric version
@@ -93,6 +100,8 @@ version = 1;  % 1 : frictionless contact and the basic contact brick
               %     on the Lagrangian augmented by the penalization term.
               % 15 : penalized contact with 'static' Coulomb friction (r is the penalization
               %     coefficient).
+              % 16 : contact without friction and integral Nitsche approach
+              % 17 : contact with friction and integral Nitsche approach
  % Signed distance representing the obstacle
 if (d == 2) obstacle = 'y'; else obstacle = 'z'; end;
 
@@ -103,18 +112,20 @@ border = gf_mesh_get(m,'outer faces');
 normals = gf_mesh_get(m, 'normal of faces', border);
 contact_boundary=border(:, find(normals(d, :) < -0.01));
 gf_mesh_set(m, 'region', GAMMAC, contact_boundary);
-contact_boundary=border(:, find(normals(d, :) > 0.01));
-gf_mesh_set(m, 'region', GAMMAD, contact_boundary);
+dirichlet_boundary=border(:, find(normals(d, :) > 0.01));
+gf_mesh_set(m, 'region', GAMMAD, dirichlet_boundary);
 
 
 
 
 % Finite element methods
-u_degree = 2;
-lambda_degree = 2;
 
 mfu=gf_mesh_fem(m, d);
 gf_mesh_fem_set(mfu, 'classical fem', u_degree);
+if (incompressibility)
+  mfp=gf_mesh_fem(m, 1);
+  gf_mesh_fem_set(mfp, 'classical fem', p_degree);
+end
 mfd=gf_mesh_fem(m, 1);
 gf_mesh_fem_set(mfd, 'classical fem', u_degree);
 mflambda=gf_mesh_fem(m, 1); % used only by versions 5 to 13
@@ -153,6 +164,10 @@ gf_model_set(md, 'add initialized data', 'cmu', [cmu]);
 gf_model_set(md, 'add initialized data', 'clambda', [clambda]);
 gf_model_set(md, 'add isotropic linearized elasticity brick', mim, 'u', ...
                  'clambda', 'cmu');
+if (incompressibility)
+  gf_model_set(md, 'add fem variable', 'p', mfp);
+  gf_model_set(md, 'add linear incompressibility brick', mim, 'u', 'p');
+end
 gf_model_set(md, 'add initialized fem data', 'volumicload', mfd, F);
 gf_model_set(md, 'add source term brick', mim, 'u', 'volumicload');
 
@@ -323,13 +338,32 @@ elseif (version == 15)
   gf_model_set(md, 'add penalized contact with rigid obstacle brick', mim_friction, 'u', ...
 	         'obstacle', 'r', 'friction_coeff', GAMMAC);
     
+         
+elseif (version == 16 || version == 17)
+ 
+  gf_model_set(md, 'add initialized data', 'gamma0', [gamma0]);
+  gf_model_set(md, 'add initialized data', 'theta', [theta]);
+  
+  if (version == 16)
+    gf_model_set(md, 'add initialized data', 'friction_coeff', [0]);
+  else
+    gf_model_set(md, 'add initialized data', 'friction_coeff', [friction_coeff]);
+  end
+  OBS = gf_mesh_fem_get(mfd, 'eval', { obstacle });
+  gf_model_set(md, 'add initialized fem data', 'obstacle', mfd, OBS);
+  % gf_model_set(md, 'add Nitsche contact with rigid obstacle brick old', mim_friction, 'u', 'obstacle', 'gamma0', 'theta', 'friction_coeff', 'clambda', 'cmu', GAMMAC);    
+  if (version == 16)
+    gf_model_set(md, 'add Nitsche contact with rigid obstacle brick', mim_friction, 'u', 'obstacle', 'gamma0', GAMMAC, theta);    
+  else
+    gf_model_set(md, 'add Nitsche contact with rigid obstacle brick', mim_friction, 'u', 'obstacle', 'gamma0', GAMMAC, theta, 'friction_coeff'); 
+  end
 else
   error('Inexistent version');
 end
 
 % Solve the problem
 if (~solved)
-  gf_model_get(md, 'test tangent matrix', 1e-6, 10, 0.01);
+  gf_model_get(md, 'test tangent matrix', 1e-6, 10, 0.1);
   gf_model_get(md, 'solve', 'max_res', 1E-9, 'very noisy', 'max_iter', niter); % ,  'lsearch', 'simplest'); % , 'with pseudo potential');
 end;
 
@@ -360,7 +394,7 @@ if (d == 3)
   set(gcf,'renderer','zbuffer');
   h=gf_plot_slice(sl2,'mesh','on','mesh_slice_edges','off','data',VMsl);
   view(-80,-15); axis on; camlight; gf_colormap('chouette');
-  % map=[1:-1/10:0]'*[1 1 1]; colormap(map); % for NB
+  
     
   % gf_plot(mfvm, VM, 'mesh', 'off', 'cvlst', ...
   %        gf_mesh_get(mfu,'outer faces'), 'deformation', U, ...
@@ -371,6 +405,7 @@ if (d == 3)
 else
   gf_plot(mfvm, VM, 'deformed_mesh', 'on', 'deformation', U, ...
           'deformation_mf', mfu, 'deformation_scale', 1, 'refine', 8);
+  map=[1:-1/10:0]'*[0.6 0.6 0.4]; colormap(map); % for NB
   xlabel('x'); ylabel('y');
   title('Deformed configuration (not really a small deformation of course ...)');
 end;
