@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2000-2015 Yves Renard
+ Copyright (C) 2000-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -24,6 +24,7 @@
 #include "getfem/dal_tree_sorted.h"
 #include "getfem/bgeot_geometric_trans.h"
 #include "getfem/bgeot_poly_composite.h"
+#include "getfem/getfem_torus.h"
 
 namespace bgeot {
 
@@ -68,9 +69,9 @@ namespace bgeot {
       size_type P = pgt_->structure()->dim();
       K_.resize(N(), P);
       if (have_pgp()) {
-        GMM_ASSERT1(ii_ < pgp_->get_point_tab().size(),
+        GMM_ASSERT1(ii_ < pgp_->get_ppoint_tab()->size(),
                     "Invalid index " << ii_ << " should be < "
-                    << pgp_->get_point_tab().size());
+                    << pgp_->get_ppoint_tab()->size());
 
         if (&pgp_->grad(ii_) == 0) { cerr << "OULA!! " << ii_ << "\n"; }
         else if (pgp_->grad(ii_).size() == 0) { cerr << "OUCH\n"; }
@@ -170,7 +171,7 @@ namespace bgeot {
   geotrans_interpolation_context::geotrans_interpolation_context
   (bgeot::pgeotrans_precomp pgp__, size_type ii__, const base_matrix& G__) :
     G_(&G__), pgt_(pgp__->get_trans()), pgp_(pgp__),
-    pspt_(&pgp__->get_point_tab()), ii_(ii__), J_(-1) {}
+    pspt_(pgp__->get_ppoint_tab()), ii_(ii__), J_(-1) {}
   geotrans_interpolation_context::geotrans_interpolation_context
   (bgeot::pgeometric_trans pgt__, bgeot::pstored_point_tab pspt__,
    size_type ii__,  const base_matrix& G__) :
@@ -328,7 +329,7 @@ namespace bgeot {
                 double(n) == params[0].num() && double(k) == params[1].num(),
                 "Bad parameters");
     dependencies.push_back(simplex_of_reference(dim_type(n), dim_type(k)));
-    return new simplex_trans_(dim_type(n), dim_type(k));
+    return std::make_shared<simplex_trans_>(dim_type(n), dim_type(k));
   }
 
   /* ******************************************************************** */
@@ -371,7 +372,7 @@ namespace bgeot {
       = dynamic_cast<const poly_geometric_trans *>(b.get());
     GMM_ASSERT1(aa && bb, "The product of geometric transformations "
                 "is only defined for polynomial ones");
-    return new cv_pr_t_(aa, bb);
+    return std::make_shared<cv_pr_t_>(aa, bb);
   }
 
   /* ******************************************************************** */
@@ -422,7 +423,7 @@ namespace bgeot {
       = dynamic_cast<const poly_geometric_trans *>(b.get());
     GMM_ASSERT1(aa && bb, "The product of geometric transformations "
                 "is only defined for polynomial ones");
-    return new cv_pr_tl_(aa, bb);
+    return std::make_shared<cv_pr_tl_>(aa, bb);
   }
 
   /* ******************************************************************** */
@@ -540,21 +541,20 @@ namespace bgeot {
   static pgeometric_trans
     Q2_incomplete_gt(gt_param_list& params,
                      std::vector<dal::pstatic_stored_object> &dependencies) {
-    GMM_ASSERT1(params.size() == 1, "Bad number of parameters : " << params.size() << " should be 1.");
+    GMM_ASSERT1(params.size() == 1, "Bad number of parameters : "
+		<< params.size() << " should be 1.");
     GMM_ASSERT1(params[0].type() == 0, "Bad type of parameters");
     int n = int(::floor(params[0].num() + 0.01));
     GMM_ASSERT1(n == 2 || n == 3, "Bad parameter, expected value 2 or 3");
     
     dependencies.push_back(Q2_incomplete_reference(dim_type(n)));
-    return new Q2_incomplete_trans_(dim_type(n));
+    return std::make_shared<Q2_incomplete_trans_>(dim_type(n));
   }
   
   pgeometric_trans Q2_incomplete_geotrans(dim_type nc) {
-    static pgeometric_trans pgt = 0;
     std::stringstream name;
     name << "GT_Q2_INCOMPLETE(" << nc << ")";
-    pgt = geometric_trans_descriptor(name.str());
-    return pgt;
+    return geometric_trans_descriptor(name.str());
   }
 
 
@@ -651,7 +651,10 @@ namespace bgeot {
   }
 
   std::string name_of_geometric_trans(pgeometric_trans p) {
-    return dal::singleton<geometric_trans_naming_system>::instance().shorter_name_of_method(p);
+    auto &instance = dal::singleton<geometric_trans_naming_system>::instance();
+    const torus_geom_trans *pgt_torus = dynamic_cast<const torus_geom_trans *>(p.get());
+    if (pgt_torus) return instance.shorter_name_of_method(pgt_torus->get_original_transformation());
+    return instance.shorter_name_of_method(p);
   }
 
   /* Fonctions pour la ref. directe.                                     */
@@ -758,7 +761,8 @@ namespace bgeot {
 
   geotrans_precomp_::geotrans_precomp_(pgeometric_trans pg,
                                        pstored_point_tab ps)
-    : pgt(pg), pspt(ps) {}
+    : pgt(pg), pspt(ps)
+  { DAL_STORED_OBJECT_DEBUG_CREATED(this, "Geotrans precomp"); }
 
   void geotrans_precomp_::init_val() const {
     c.clear();
@@ -801,12 +805,11 @@ namespace bgeot {
   pgeotrans_precomp geotrans_precomp(pgeometric_trans pg,
                                      pstored_point_tab pspt,
                                      dal::pstatic_stored_object dep) {
-    dal::pstatic_stored_object o
-      = dal::search_stored_object(pre_geot_key_(pg, pspt));
-    if (o) return dal::stored_cast<geotrans_precomp_>(o);
-    pgeotrans_precomp p = new geotrans_precomp_(pg, pspt);
-    dal::add_stored_object(new pre_geot_key_(pg, pspt), p, pg, pspt,
-                           dal::AUTODELETE_STATIC_OBJECT);
+    dal::pstatic_stored_object_key pk= std::make_shared<pre_geot_key_>(pg,pspt);
+    dal::pstatic_stored_object o = dal::search_stored_object(pk);
+    if (o) return std::dynamic_pointer_cast<const geotrans_precomp_>(o);
+    pgeotrans_precomp p = std::make_shared<geotrans_precomp_>(pg, pspt);
+    dal::add_stored_object(pk, p, pg, pspt, dal::AUTODELETE_STATIC_OBJECT);
     if (dep) dal::add_dependency(p, dep);
     return p;
   }
