@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2015 Yves Renard
+ Copyright (C) 2002-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -30,7 +30,7 @@ namespace bgeot {
 
 
   int imbricated_box_less::operator()(const base_node &x,
-				      const base_node &y) const {
+                                      const base_node &y) const {
     size_type s = x.size(); 
     scalar_type c1 = c_max, c2 = c_max * scalar_type(base);
     GMM_ASSERT2(y.size() == s, "dimension error");
@@ -40,23 +40,23 @@ namespace bgeot {
     for (; itx != itex; ++itx, ++ity) {
       long a = long(sfloor((*itx) * c1)), b = long(sfloor((*ity) * c1));
       if ((gmm::abs(a) > scalar_type(base))
-	  || (gmm::abs(b) > scalar_type(base))) { 
-	exp_max++; c_max /= scalar_type(base);
-	return (*this)(x,y);
+          || (gmm::abs(b) > scalar_type(base))) { 
+        exp_max++; c_max /= scalar_type(base);
+        return (*this)(x,y);
       }
       if (ret == 0) { if (a < b) ret = -1; else if (a > b) ret = 1; }
     }
     if (ret) return ret;
     
     for (int e = exp_max; e >= exp_min; --e, c1 *= scalar_type(base),
-	   c2 *= scalar_type(base)) {
+           c2 *= scalar_type(base)) {
       itx = x.begin(), itex = x.end(), ity = y.begin();
       for (; itx != itex; ++itx, ++ity) {
-	int a = int(sfloor(((*itx) * c2) - sfloor((*itx) * c1)
-			   * scalar_type(base)));
-	int b = int(sfloor(((*ity) * c2) - sfloor((*ity) * c1)
-			   * scalar_type(base)));
-	if (a < b) return -1; else if (a > b) return 1;
+        int a = int(sfloor(((*itx) * c2) - sfloor((*itx) * c1)
+                           * scalar_type(base)));
+        int b = int(sfloor(((*ity) * c2) - sfloor((*ity) * c1)
+                           * scalar_type(base)));
+        if (a < b) return -1; else if (a > b) return 1;
       }
     }
     return 0;
@@ -187,7 +187,7 @@ namespace bgeot {
   static void structured_mesh_for_simplex_(pconvex_structure cvs, 
     pgeometric_trans opt_gt,
     const std::vector<base_node> *opt_gt_pts,
-    short_type k, pbasic_mesh &pm) {
+    short_type k, pbasic_mesh pm) {
       scalar_type h = 1./k;
       switch (cvs->dim()) {
       case 1 :
@@ -293,7 +293,7 @@ namespace bgeot {
                 }
               }
             }
-          }	
+          }
         }
         break;
       default : 
@@ -304,7 +304,7 @@ namespace bgeot {
 
   static void structured_mesh_for_parallelepiped_
     (pconvex_structure cvs, pgeometric_trans opt_gt,
-    const std::vector<base_node> *opt_gt_pts, short_type k, pbasic_mesh &pm) {
+    const std::vector<base_node> *opt_gt_pts, short_type k, pbasic_mesh pm) {
       scalar_type h = 1./k;
       size_type n = cvs->dim();
       size_type pow2n = (size_type(1) << n);
@@ -320,7 +320,7 @@ namespace bgeot {
       while (kcnt[n] == 0) {
         for (size_type z = 0; z < n; ++z)
           pt[z] = h*kcnt[z];
-        if (opt_gt) pt = opt_gt->transform(pt, *opt_gt_pts);	  
+        if (opt_gt) pt = opt_gt->transform(pt, *opt_gt_pts);
         pids.push_back(pm->add_point(pt));
         kk=0;
         while (kk <= n)
@@ -350,20 +350,20 @@ namespace bgeot {
   static void structured_mesh_for_convex_(pconvex_structure cvs, 
     pgeometric_trans opt_gt,
     const std::vector<base_node> *opt_gt_pts,
-    short_type k, pbasic_mesh &pm) {
-      size_type nbp = cvs->basic_structure()->nb_points();
+    short_type k, pbasic_mesh pm) {
+      size_type nbp = basic_structure(cvs)->nb_points();
       dim_type n = cvs->dim();
       /* Identifying simplexes.                                           */    
       if (nbp == size_type(n+1) && 
-        cvs->basic_structure()==simplex_structure(n)) {
+          basic_structure(cvs)==simplex_structure(n)) {
           // smc.pm->write_to_file(cout);
           structured_mesh_for_simplex_(cvs,opt_gt,opt_gt_pts,k,pm);
           /* Identifying parallelepipeds.                                     */
       } else if (nbp == (size_type(1) << n) && 
-        cvs->basic_structure() == parallelepiped_structure(n)) {
+          basic_structure(cvs) == parallelepiped_structure(n)) {
           structured_mesh_for_parallelepiped_(cvs,opt_gt,opt_gt_pts,k,pm);
       } else if (nbp == size_type(2 * n) && 
-        cvs->basic_structure() == prism_structure(n)) {
+          basic_structure(cvs) == prism_structure(n)) {
           GMM_ASSERT1(false, "Sorry, structured_mesh not implemented for prisms.");
       } else {
         GMM_ASSERT1(false, "This element is not taken into account. Contact us");
@@ -410,20 +410,17 @@ namespace bgeot {
     short_type n;
     bool simplex_mesh; /* true if the convex has been splited into simplexes,
                        which were refined */
-    basic_mesh *pm;
-    std::vector<mesh_structure *> pfacem; /* array of mesh_structures for faces */
+    std::unique_ptr<basic_mesh> pm;
+    std::vector<std::unique_ptr<mesh_structure>> pfacem; /* array of mesh_structures for faces */
     dal::bit_vector nodes_on_edges;
-    mesh_precomposite *pmp;
-    str_mesh_cv__(void) : pm(0), pmp(0) {}
+    std::unique_ptr<mesh_precomposite> pmp;
     str_mesh_cv__(pconvex_structure c, short_type k, bool smesh_) : 
-      cvs(c), n(k), simplex_mesh(smesh_) {}
-    ~str_mesh_cv__() { 
-      if (pm) delete pm; if (pmp) delete pmp; pm = 0; pmp = 0;
-      for (size_type i=0; i < pfacem.size(); ++i) delete pfacem[i];
-    }
+      cvs(c), n(k), simplex_mesh(smesh_)
+    { DAL_STORED_OBJECT_DEBUG_CREATED(this, "cv mesh"); }
+    ~str_mesh_cv__() { DAL_STORED_OBJECT_DEBUG_DESTROYED(this,"cv mesh"); }
   };
 
-  typedef boost::intrusive_ptr<const str_mesh_cv__> pstr_mesh_cv__;
+  typedef std::shared_ptr<const str_mesh_cv__> pstr_mesh_cv__;
 
   /**
   * This function returns a mesh in pm which contains a refinement of the convex
@@ -436,82 +433,79 @@ namespace bgeot {
     pbasic_mesh &pm, pmesh_precomposite &pmp, 
     bool force_simplexification) {
       size_type n = cvr->structure()->dim();
-      size_type nbp = cvr->structure()->basic_structure()->nb_points();
+      size_type nbp = basic_structure(cvr->structure())->nb_points();
 
       force_simplexification = force_simplexification || (nbp == n+1);
+      dal::pstatic_stored_object_key
+        pk = std::make_shared<str_mesh_key>(basic_structure(cvr->structure()),
+                                            k, force_simplexification);
 
-      dal::pstatic_stored_object o
-        = dal::search_stored_object(str_mesh_key(cvr->structure()
-        ->basic_structure(), k,
-        force_simplexification));
+      dal::pstatic_stored_object o = dal::search_stored_object(pk);
       pstr_mesh_cv__ psmc;
-      if (o) {
-        psmc = dal::stored_cast<str_mesh_cv__>(o);
-      } 
+      if (o)
+        psmc = std::dynamic_pointer_cast<const str_mesh_cv__>(o);
       else {
-        str_mesh_cv__ &smc(*(new str_mesh_cv__(cvr->structure()
-          ->basic_structure(), k,
-          force_simplexification)));
-        psmc = &smc;
 
-        smc.pm = new basic_mesh();
+        auto ppsmc = std::make_shared<str_mesh_cv__>
+          (basic_structure(cvr->structure()), k, force_simplexification);
+        str_mesh_cv__ &smc(*ppsmc);
+        psmc = ppsmc;
+
+        smc.pm = std::make_unique<basic_mesh>();
 
         if (force_simplexification) {
           // cout << "cvr = " << int(cvr->structure()->dim()) << " : "
           //      << cvr->structure()->nb_points() << endl;
           const mesh_structure* splx_mesh
-            = cvr->basic_convex_ref()->simplexified_convex();
+            = basic_convex_ref(cvr)->simplexified_convex();
           /* splx_mesh is correctly oriented */
           for (size_type ic=0; ic < splx_mesh->nb_convex(); ++ic) {
             std::vector<base_node> cvpts(splx_mesh->nb_points_of_convex(ic));
             pgeometric_trans sgt
               = simplex_geotrans(cvr->structure()->dim(), 1);
             for (size_type j=0; j < cvpts.size(); ++j) {
-              cvpts[j] = cvr->basic_convex_ref()->points()
+              cvpts[j] = basic_convex_ref(cvr)->points()
                 [splx_mesh->ind_points_of_convex(ic)[j]];
               //cerr << "cvpts[" << j << "]=" << cvpts[j] << endl;
             }
             structured_mesh_for_convex_(splx_mesh->structure_of_convex(ic),
-              sgt, &cvpts, k, smc.pm);
+                                        sgt, &cvpts, k, smc.pm.get());
           }
         } else {
-          structured_mesh_for_convex_(cvr->structure(), 0, 0, k, smc.pm);
+          structured_mesh_for_convex_(cvr->structure(), 0, 0, k, smc.pm.get());
         }
         smc.pfacem.resize(cvr->structure()->nb_faces());
         for (dim_type f=0; f < cvr->structure()->nb_faces(); ++f) {
-          smc.pfacem[f] = new mesh_structure();
-          structured_mesh_of_faces_(cvr, f, *smc.pm, *smc.pfacem[f]);
+          smc.pfacem[f] = std::make_unique<mesh_structure>();
+          structured_mesh_of_faces_(cvr, f, *(smc.pm), *(smc.pfacem[f]));
         }
 
-        smc.pmp = new mesh_precomposite(*(smc.pm));
-        dal::add_stored_object(new str_mesh_key(cvr->structure()
-          ->basic_structure(), k,
-          force_simplexification), psmc,
-          cvr->structure()->basic_structure());
+        smc.pmp = std::make_unique<mesh_precomposite>(*(smc.pm));
+        dal::add_stored_object(pk, psmc, basic_structure(cvr->structure()));
       }
-      pm  = psmc->pm;
-      pmp = psmc->pmp;
+      pm  = psmc->pm.get();
+      pmp = psmc->pmp.get();
   }
 
   const basic_mesh *
     refined_simplex_mesh_for_convex(pconvex_ref cvr, short_type k) {
       pbasic_mesh pm; pmesh_precomposite pmp; 
-      structured_mesh_for_convex(cvr,k,pm,pmp,true);
+      structured_mesh_for_convex(cvr, k, pm, pmp, true);
       return pm;
   }
 
-  const std::vector<mesh_structure*>& 
-    refined_simplex_mesh_for_convex_faces(pconvex_ref cvr, 
-    short_type k) {
-      dal::pstatic_stored_object o
-        = dal::search_stored_object(str_mesh_key(cvr->structure()
-        ->basic_structure(), k, true));
-      if (o) {
-        pstr_mesh_cv__ psmc = dal::stored_cast<str_mesh_cv__>(o);
-        return psmc->pfacem;
-      } 
-      else GMM_ASSERT1(false,
-        "call refined_simplex_mesh_for_convex first (or fix me)");
+  const std::vector<std::unique_ptr<mesh_structure>>&
+    refined_simplex_mesh_for_convex_faces(pconvex_ref cvr, short_type k) {
+    dal::pstatic_stored_object_key
+      pk = std::make_shared<str_mesh_key>(basic_structure(cvr->structure()),
+                                          k, true);
+    dal::pstatic_stored_object o = dal::search_stored_object(pk);
+    if (o) {
+      pstr_mesh_cv__ psmc = std::dynamic_pointer_cast<const str_mesh_cv__>(o);
+      return psmc->pfacem;
+    } 
+    else GMM_ASSERT1(false,
+                     "call refined_simplex_mesh_for_convex first (or fix me)");
   }
 
 }  /* end of namespace getfem.                                            */
