@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2000-2015 Yves Renard
+ Copyright (C) 2000-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -45,8 +45,6 @@
 
 namespace bgeot {
 
-  typedef std::vector<short_type> convex_ind_ct;
-
   /**  Description of a geometric transformation between a
    * reference element and a real element.
    *
@@ -128,7 +126,7 @@ namespace bgeot {
     pconvex_structure structure(void) const { return cvr->structure(); }
     /// Basic structure of the reference element.
     pconvex_structure basic_structure(void) const
-    { return cvr->structure()->basic_structure(); }
+    { return bgeot::basic_structure(cvr->structure()); }
     /// Gives the value of the functions vector at a certain point.
     virtual void poly_vector_val(const base_node &pt, base_vector &val) const = 0;
     /// Gives the value of a subgroup of the functions vector at a certain point.
@@ -150,6 +148,9 @@ namespace bgeot {
     /// Gives the array of geometric nodes (on reference convex)
     const stored_point_tab &geometric_nodes(void) const
     { return cvr->points(); }
+    /// Gives the array of geometric nodes (on reference convex)
+    pstored_point_tab pgeometric_nodes(void) const
+    { return cvr->pspt(); }
     /// Gives the array of the normals to faces (on reference convex)
     const std::vector<base_small_vector> &normals(void) const
     { return cvr->normals(); }
@@ -161,7 +162,10 @@ namespace bgeot {
     /** Compute the gradient at point x, pc is resized to [nb_points() x dim()]
         if the transformation is linear, x is not used at all */
     size_type complexity(void) const { return complexity_; }
-    virtual ~geometric_trans() {}
+    virtual ~geometric_trans()
+    { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Geometric transformation"); }
+    geometric_trans()
+      { DAL_STORED_OBJECT_DEBUG_CREATED(this, "Geometric transformation"); }
   };
 
   template<class CONT>
@@ -176,7 +180,7 @@ namespace bgeot {
   }
 
   /** pointer type for a geometric transformation */
-  typedef boost::intrusive_ptr<const bgeot::geometric_trans> pgeometric_trans;
+  typedef std::shared_ptr<const bgeot::geometric_trans> pgeometric_trans;
   class geotrans_interpolation_context;
 
   template<class CONT>
@@ -263,7 +267,7 @@ namespace bgeot {
 
 
   class geotrans_precomp_;
-  typedef boost::intrusive_ptr<const geotrans_precomp_> pgeotrans_precomp;
+  typedef std::shared_ptr<const geotrans_precomp_> pgeotrans_precomp;
 
 
   /**
@@ -305,9 +309,11 @@ namespace bgeot {
 
     base_node transform(size_type i, const base_matrix &G) const;
     pgeometric_trans get_trans() const { return pgt; }
-    inline const stored_point_tab& get_point_tab() const { return *pspt; }
-  protected:
+    // inline const stored_point_tab& get_point_tab() const { return *pspt; }
+    inline pstored_point_tab get_ppoint_tab() const { return pspt; }
     geotrans_precomp_(pgeometric_trans pg, pstored_point_tab ps);
+    ~geotrans_precomp_()
+      { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Geotrans precomp"); }
 
   private:
     void init_val() const;
