@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2011-2015 Tomas Ligursky, Yves Renard, Konstantinos Poulios
+ Copyright (C) 2011-2016 Tomas Ligursky, Yves Renard, Konstantinos Poulios
 
  This file is a part of GetFEM++
 
@@ -334,6 +334,10 @@ namespace getfem {
         solve_grad(X, Gamma, Delta_X, y, f, g);                // y = F_x(X, Gamma)^-1 * g
                                                                // Delta_X = F_x(X, Gamma)^-1 * f
         Delta_Gamma = sp(tX, Delta_X) / (sp(tX, y) - tGamma);  // Delta_Gamma = tX.Delta_X / (tX.y - tGamma)
+        if (isnan(Delta_Gamma)) {
+          if (noisy() > 0) cout << "Newton correction failed with NaN" << endl;
+          return false;
+        }
         gmm::add(gmm::scaled(y, -Delta_Gamma), Delta_X);       // Delta_X -= Delta_Gamma * y
         scaled_add(X, Gamma, Delta_X, Delta_Gamma, -1,
                    X, Gamma);                                  // [X,Gamma] -= [Delta_X,Delta_Gamma]
@@ -969,6 +973,7 @@ namespace getfem {
         tau_lp(0.), tau_bp_1(tau_bp_init), tau_bp_2(tau_bp_init),
         gamma_sing(0.), gamma_next(0.)
     {}
+    virtual ~virtual_cont_struct() {}
 
   };
 
@@ -985,7 +990,8 @@ namespace getfem {
 #ifdef GETFEM_MODELS_H__
 
   class cont_struct_getfem_model
-  : public virtual_cont_struct<base_vector, model_real_sparse_matrix> {
+    : public virtual_cont_struct<base_vector, model_real_sparse_matrix>,
+      virtual public dal::static_stored_object {
 
   private:
     mutable model *md;
