@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 1999-2015 Yves Renard
+ Copyright (C) 1999-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -63,16 +63,16 @@ namespace getfem {
 
     iterator &operator ++()
     { ++ii; if (ii == N) { ii = 0; ++it; } return *this; }
-    iterator &operator --() 
+    iterator &operator --()
     { if (ii == 0) { ii = N-1; --it; } else --ii; return *this; }
     iterator operator ++(int) { iterator tmp = *this; ++(*this); return tmp; }
     iterator operator --(int) { iterator tmp = *this; --(*this); return tmp; }
-   
+
     iterator &operator +=(difference_type i)
     { it += (i+ii)/N; ii = dim_type((ii + i) % N); return *this; }
     iterator &operator -=(difference_type i)
     { it -= (i+N-ii-1)/N; ii = (ii - i + N * i) % N; return *this; }
-    iterator operator +(difference_type i) const 
+    iterator operator +(difference_type i) const
     { iterator itt = *this; return (itt += i); }
     iterator operator -(difference_type i) const
     { iterator itt = *this; return (itt -= i); }
@@ -117,7 +117,7 @@ namespace getfem {
   protected :
     ITER it, ite;
     dim_type N;
-    
+
   public :
 
     bool empty(void) const { return it == ite; }
@@ -129,28 +129,30 @@ namespace getfem {
     { return const_reverse_iterator(end()); }
     const_reverse_iterator rend(void) const
     { return const_reverse_iterator(begin()); }
-    
+
     value_type front(void) const { return *begin(); }
     value_type back(void) const { return *(--(end())); }
 
     tab_scal_to_vect(void) : N(0) {}
     tab_scal_to_vect(const CONT &cc, dim_type n)
       : it(cc.begin()), ite(cc.end()), N(n) {}
-    
+
     value_type operator [](size_type ii) const { return *(begin() + ii);}
   };
-  
+
   /** Describe a finite element method linked to a mesh.
-   *    
+   *
    *  @see mesh
    *  @see mesh_im
    */
-  class mesh_fem : public context_dependencies {
+  class mesh_fem : public context_dependencies, virtual public dal::static_stored_object {
   protected :
-    
     typedef gmm::csc_matrix<scalar_type> REDUCTION_MATRIX;
     typedef gmm::csr_matrix<scalar_type> EXTENSION_MATRIX;
-    
+
+    void copy_from(const mesh_fem &mf); /* Remember to change copy_from if
+                                           adding components to mesh_fem */
+
     dal::dynamic_array<pfem> f_elems;
     dal::bit_vector fe_convex;
     const mesh *linked_mesh_;
@@ -166,13 +168,13 @@ namespace getfem {
     bool auto_add_elt_disc;
     scalar_type auto_add_elt_alpha;
     dim_type Qdim;         /* this is the "total" target_dim. */
-    bgeot::multi_index mi; /* multi dimensions for matrix/tensor field. */ 
+    bgeot::multi_index mi; /* multi dimensions for matrix/tensor field. */
     // dim_type QdimM, QdimN; /* for matrix field with QdimM lines and QdimN */
     //                       /* columnsQdimM * QdimN = Qdim.                */
     std::vector<size_type> dof_partition;
     mutable gmm::uint64_type v_num_update, v_num;
     bool use_reduction;    /* A reduction matrix is applied or not.       */
-    
+
   public :
     typedef base_node point_type;
     typedef tab_scal_to_vect<mesh::ind_cv_ct> ind_dof_ct;
@@ -187,25 +189,25 @@ namespace getfem {
      */
     inline const dal::bit_vector &convex_index(void) const
     { context_check(); return fe_convex; }
-    
+
     /// Return true if a reduction matrix is applied to the dofs.
     bool is_reduced(void) const { return use_reduction; }
-    
+
     /// Return the reduction matrix applied to the dofs.
     const REDUCTION_MATRIX &reduction_matrix(void) const { return R_; }
 
     /// Return the extension matrix corresponding to reduction applied (RE=I).
     const EXTENSION_MATRIX &extension_matrix(void) const { return E_; }
-    
+
     /** Allows to set the reduction and the extension matrices.
      * Should satify (RR*EE=I). */
     template <typename MATR, typename MATE>
     void set_reduction_matrices(const MATR &RR, const MATE &EE) {
       context_check();
       GMM_ASSERT1(gmm::mat_ncols(RR) == nb_basic_dof() &&
-		  gmm::mat_nrows(EE) == nb_basic_dof() &&
-		  gmm::mat_nrows(RR) == gmm::mat_ncols(EE),
-		  "Wrong dimension of reduction and/or extension matrices");
+                  gmm::mat_nrows(EE) == nb_basic_dof() &&
+                  gmm::mat_nrows(RR) == gmm::mat_ncols(EE),
+                  "Wrong dimension of reduction and/or extension matrices");
       R_ = REDUCTION_MATRIX(gmm::mat_nrows(RR), gmm::mat_ncols(RR));
       E_ = EXTENSION_MATRIX(gmm::mat_nrows(EE), gmm::mat_ncols(EE));
       gmm::copy(RR, R_);
@@ -222,32 +224,32 @@ namespace getfem {
     /// Validate or invalidate the reduction (keeping the reduction matrices).
     void set_reduction(bool r) {
       if (r != use_reduction) {
-	use_reduction = r;
-	if (use_reduction) {
-	  context_check();
-	  GMM_ASSERT1(gmm::mat_ncols(R_) == nb_basic_dof() &&
-		      gmm::mat_nrows(E_) == nb_basic_dof() &&
-		      gmm::mat_nrows(R_) == gmm::mat_ncols(E_),
-		    "Wrong dimension of reduction and/or extension matrices");
-	}
-	touch(); v_num = act_counter();
+        use_reduction = r;
+        if (use_reduction) {
+          context_check();
+          GMM_ASSERT1(gmm::mat_ncols(R_) == nb_basic_dof() &&
+                      gmm::mat_nrows(E_) == nb_basic_dof() &&
+                      gmm::mat_nrows(R_) == gmm::mat_ncols(E_),
+                    "Wrong dimension of reduction and/or extension matrices");
+        }
+        touch(); v_num = act_counter();
       }
     }
 
     template<typename VECT1, typename VECT2>
     void reduce_vector(const VECT1 &V1, const VECT2 &V2) const {
       if (is_reduced()) {
-	size_type qqdim = gmm::vect_size(V1) / nb_basic_dof();
-	if (qqdim == 1)
-	  gmm::mult(reduction_matrix(), V1, const_cast<VECT2 &>(V2));
-	else
-	  for (size_type k = 0; k < qqdim; ++k)
-	    gmm::mult(reduction_matrix(),
-		      gmm::sub_vector(V1, gmm::sub_slice(k, nb_basic_dof(),
-							 qqdim)),
-		      gmm::sub_vector(const_cast<VECT2 &>(V2),
-				      gmm::sub_slice(k, nb_dof(),
-						     qqdim)));
+        size_type qqdim = gmm::vect_size(V1) / nb_basic_dof();
+        if (qqdim == 1)
+          gmm::mult(reduction_matrix(), V1, const_cast<VECT2 &>(V2));
+        else
+          for (size_type k = 0; k < qqdim; ++k)
+            gmm::mult(reduction_matrix(),
+                      gmm::sub_vector(V1, gmm::sub_slice(k, nb_basic_dof(),
+                                                         qqdim)),
+                      gmm::sub_vector(const_cast<VECT2 &>(V2),
+                                      gmm::sub_slice(k, nb_dof(),
+                                                     qqdim)));
       }
       else gmm::copy(V1, const_cast<VECT2 &>(V2));
     }
@@ -255,21 +257,21 @@ namespace getfem {
     template<typename VECT1, typename VECT2>
     void extend_vector(const VECT1 &V1, const VECT2 &V2) const {
       if (is_reduced()) {
-	size_type qqdim = gmm::vect_size(V1) / nb_dof();
-	if (qqdim == 1)
-	  gmm::mult(extension_matrix(), V1, const_cast<VECT2 &>(V2));
-	else
-	  for (size_type k = 0; k < qqdim; ++k)
-	    gmm::mult(extension_matrix(),
-		      gmm::sub_vector(V1, gmm::sub_slice(k, nb_dof(),
-							 qqdim)),
-		      gmm::sub_vector(const_cast<VECT2 &>(V2),
-				      gmm::sub_slice(k, nb_basic_dof(),
-						     qqdim)));
+        size_type qqdim = gmm::vect_size(V1) / nb_dof();
+        if (qqdim == 1)
+          gmm::mult(extension_matrix(), V1, const_cast<VECT2 &>(V2));
+        else
+          for (size_type k = 0; k < qqdim; ++k)
+            gmm::mult(extension_matrix(),
+                      gmm::sub_vector(V1, gmm::sub_slice(k, nb_dof(),
+                                                         qqdim)),
+                      gmm::sub_vector(const_cast<VECT2 &>(V2),
+                                      gmm::sub_slice(k, nb_basic_dof(),
+                                                     qqdim)));
       }
       else gmm::copy(V1, const_cast<VECT2 &>(V2));
     }
-    
+
 
     /// Return a reference to the underlying mesh.
     const mesh &linked_mesh(void) const { return *linked_mesh_; }
@@ -278,7 +280,7 @@ namespace getfem {
      *  of element option. K=-1 disables the automatic addition.
      */
     void set_auto_add(dim_type K, bool disc = false,
-		      scalar_type alpha = scalar_type(0)) {
+                      scalar_type alpha = scalar_type(0)) {
       auto_add_elt_K = K; auto_add_elt_disc = disc;
       auto_add_elt_alpha = alpha; auto_add_elt_pf = 0;
     }
@@ -290,8 +292,8 @@ namespace getfem {
     { auto_add_elt_pf = pf; auto_add_elt_K = dim_type(-1);}
 
     /** Return the Q dimension. A mesh_fem used for scalar fields has
-	Q=1, for vector fields, Q is typically equal to
-	linked_mesh().dim().
+        Q=1, for vector fields, Q is typically equal to
+        linked_mesh().dim().
      */
     virtual dim_type get_qdim() const { return Qdim; }
     virtual const bgeot::multi_index &get_qdims() const { return mi; }
@@ -308,7 +310,7 @@ namespace getfem {
     virtual void set_qdim(dim_type M, dim_type N) {
       if (mi.size() != 2 || mi[0] != M || mi[1] != N) {
         mi.resize(2); mi[0] = M; mi[1] = N; Qdim = dim_type(M*N);
-	dof_enumeration_made = false; touch(); v_num = act_counter();
+        dof_enumeration_made = false; touch(); v_num = act_counter();
       }
     }
 
@@ -318,7 +320,7 @@ namespace getfem {
           || mi[3] != P) {
         mi.resize(4); mi[0] = M; mi[1] = N; mi[2] = O; mi[3] = P;
         Qdim = dim_type(M*N*O*P);
-	dof_enumeration_made = false; touch(); v_num = act_counter();
+        dof_enumeration_made = false; touch(); v_num = act_counter();
       }
     }
 
@@ -336,71 +338,71 @@ namespace getfem {
         dof_enumeration_made = false; touch(); v_num = act_counter();
       }
     }
-    
+
 
     void set_qdim_mn(dim_type M, dim_type N) IS_DEPRECATED
     { set_qdim(M,N); }
-    
+
     /** for matrix fields, return the number of rows. */
     dim_type get_qdim_m() const IS_DEPRECATED { return dim_type(mi[0]); }
     /** for matrix fields, return the number of columns. */
     dim_type get_qdim_n() const IS_DEPRECATED { return dim_type(mi[1]); }
-     
+
 
     /** Set the finite element method of a convex.
-	@param cv the convex number.
-	@param pf the finite element.
+        @param cv the convex number.
+        @param pf the finite element.
     */
-    void set_finite_element(size_type cv, pfem pf);	
+    void set_finite_element(size_type cv, pfem pf);
     /** Set the finite element on a set of convexes.
-	@param cvs the set of convex indexes, as a dal::bit_vector.
-	@param pf the finite element, typically obtained with
-	@code getfem::fem_descriptor("FEM_SOMETHING(..)") 
-	@endcode
+        @param cvs the set of convex indexes, as a dal::bit_vector.
+        @param pf the finite element, typically obtained with
+        @code getfem::fem_descriptor("FEM_SOMETHING(..)")
+        @endcode
     */
     void set_finite_element(const dal::bit_vector &cvs, pfem pf);
     /** shortcut for set_finite_element(linked_mesh().convex_index(),pf);
         and set_auto_add(pf). */
     void set_finite_element(pfem pf);
     /** Set a classical (i.e. lagrange polynomial) finite element on
-	a convex.
-	@param cv is the convex number.
-	@param fem_degree the polynomial degree of the finite element.
+        a convex.
+        @param cv is the convex number.
+        @param fem_degree the polynomial degree of the finite element.
     */
     void set_classical_finite_element(size_type cv, dim_type fem_degree);
     /** Set a classical (i.e. lagrange polynomial) finite element on
-	a set of convexes.
-	@param cvs the set of convexes, as a dal::bit_vector.
-	@param fem_degree the polynomial degree of the finite element.
+        a set of convexes.
+        @param cvs the set of convexes, as a dal::bit_vector.
+        @param fem_degree the polynomial degree of the finite element.
     */
-    void set_classical_finite_element(const dal::bit_vector &cvs, 
-				      dim_type fem_degree);
+    void set_classical_finite_element(const dal::bit_vector &cvs,
+                                      dim_type fem_degree);
     /** Similar to set_classical_finite_element, but uses
-	discontinuous lagrange elements. 
+        discontinuous lagrange elements.
 
-	@param cv the convex index.
-	@param fem_degree the polynomial degree of the finite element.
-	@param alpha the node inset, 0 <= alpha < 1, where 0 implies
-	usual dof nodes, greater values move the nodes toward the
-	center of gravity, and 1 means that all degrees of freedom
-	collapse on the center of gravity.
+        @param cv the convex index.
+        @param fem_degree the polynomial degree of the finite element.
+        @param alpha the node inset, 0 <= alpha < 1, where 0 implies
+        usual dof nodes, greater values move the nodes toward the
+        center of gravity, and 1 means that all degrees of freedom
+        collapse on the center of gravity.
      */
     void set_classical_discontinuous_finite_element(size_type cv,
-						    dim_type fem_degree,
-						    scalar_type alpha=0);
+                                                    dim_type fem_degree,
+                                                    scalar_type alpha=0);
     /** Similar to set_classical_finite_element, but uses
-	discontinuous lagrange elements. 
+        discontinuous lagrange elements.
 
-	@param cvs the set of convexes, as a dal::bit_vector.
-	@param fem_degree the polynomial degree of the finite element.
-	@param alpha the node inset, 0 <= alpha < 1, where 0 implies
-	usual dof nodes, greater values move the nodes toward the
-	center of gravity, and 1 means that all degrees of freedom
-	collapse on the center of gravity.
+        @param cvs the set of convexes, as a dal::bit_vector.
+        @param fem_degree the polynomial degree of the finite element.
+        @param alpha the node inset, 0 <= alpha < 1, where 0 implies
+        usual dof nodes, greater values move the nodes toward the
+        center of gravity, and 1 means that all degrees of freedom
+        collapse on the center of gravity.
      */
     void set_classical_discontinuous_finite_element(const dal::bit_vector &cvs,
-						    dim_type fem_degree,
-						    scalar_type alpha=0);
+                                                    dim_type fem_degree,
+                                                    scalar_type alpha=0);
     /** Shortcut for
      * set_classical_finite_element(linked_mesh().convex_index(),...)
      */
@@ -410,9 +412,9 @@ namespace getfem {
      *  ,...)
      */
     void set_classical_discontinuous_finite_element(dim_type fem_degree,
-						    scalar_type alpha=0);   
+                                                    scalar_type alpha=0);
     /** Return the basic fem associated with an element (if no fem is
-     *	associated, the function will crash! use the convex_index() of
+     *        associated, the function will crash! use the convex_index() of
      *  the mesh_fem to check that a fem is associated to a given
      *  convex). This fem does not take into account the optional
      *  vectorization due to qdim nor the optional reduction.
@@ -426,43 +428,43 @@ namespace getfem {
     virtual ind_dof_ct ind_basic_dof_of_element(size_type cv) const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       return ind_dof_ct(dof_structure.ind_points_of_convex(cv),
-			dim_type(Qdim/fem_of_element(cv)->target_dim()));
+                        dim_type(Qdim/fem_of_element(cv)->target_dim()));
     }
     ind_dof_ct ind_dof_of_element(size_type cv) const IS_DEPRECATED
     { return ind_basic_dof_of_element(cv); }
     /** Give an array of the dof numbers lying of a convex face (all
-      	degrees of freedom whose associated base function is non-zero
-	on the convex face).
-	@param cv the convex number.
-	@param f the face number.
-	@return a pseudo-container of the dof number.
+              degrees of freedom whose associated base function is non-zero
+        on the convex face).
+        @param cv the convex number.
+        @param f the face number.
+        @return a pseudo-container of the dof number.
     */
     virtual ind_dof_face_ct
     ind_basic_dof_of_face_of_element(size_type cv, short_type f) const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       return ind_dof_face_ct
-	(dof_structure.ind_points_of_face_of_convex(cv, f),
-	 dim_type(Qdim/fem_of_element(cv)->target_dim()));
+        (dof_structure.ind_points_of_face_of_convex(cv, f),
+         dim_type(Qdim/fem_of_element(cv)->target_dim()));
     }
     ind_dof_face_ct
     ind_dof_of_face_of_element(size_type cv,short_type f) const IS_DEPRECATED
     { return ind_basic_dof_of_face_of_element(cv, f); }
     /** Return the number of dof lying on the given convex face.
-	@param cv the convex number.
-	@param f the face number.
+        @param cv the convex number.
+        @param f the face number.
     */
     virtual size_type nb_basic_dof_of_face_of_element(size_type cv,
-					      short_type f) const {
+                                              short_type f) const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       pfem pf = f_elems[cv];
       return dof_structure.structure_of_convex(cv)->nb_points_of_face(f)
-	* Qdim / pf->target_dim();
+        * Qdim / pf->target_dim();
     }
     size_type nb_dof_of_face_of_element(size_type cv,
-					short_type f) const IS_DEPRECATED
+                                        short_type f) const IS_DEPRECATED
     { return nb_basic_dof_of_face_of_element(cv, f); }
     /** Return the number of  degrees of freedom attached to a given convex.
-	@param cv the convex number.
+        @param cv the convex number.
     */
     virtual size_type nb_basic_dof_of_element(size_type cv) const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
@@ -470,25 +472,25 @@ namespace getfem {
     }
     size_type nb_dof_of_element(size_type cv) const IS_DEPRECATED
     { return nb_basic_dof_of_element(cv); }
-    
+
     /* Return the geometrical location of a degree of freedom in the
        reference convex.
-	@param cv the convex number.
-	@param i the local dof number.
+        @param cv the convex number.
+        @param i the local dof number.
     const base_node &reference_point_of_dof(size_type cv,size_type i) const {
       pfem pf = f_elems[cv];
       return pf->node_of_dof(cv, i * pf->target_dim() / Qdim);
     }
     */
     /** Return the geometrical location of a degree of freedom.
-	@param cv the convex number.
-	@param i the local dof number.
+        @param cv the convex number.
+        @param i the local dof number.
      */
     virtual base_node point_of_basic_dof(size_type cv, size_type i) const;
     base_node point_of_dof(size_type cv, size_type i) const IS_DEPRECATED
     { return point_of_basic_dof(cv, i); }
     /** Return the geometrical location of a degree of freedom.
-	@param d the global dof number.
+        @param d the global dof number.
     */
     virtual base_node point_of_basic_dof(size_type d) const;
     base_node point_of_dof(size_type d) const IS_DEPRECATED
@@ -497,15 +499,15 @@ namespace getfem {
     virtual dim_type basic_dof_qdim(size_type d) const;
     dim_type dof_qdim(size_type d) const IS_DEPRECATED
     { return basic_dof_qdim(d); }
-    /** Shortcut for convex_to_dof(d)[0] 
- 	@param d the global dof number.
+    /** Shortcut for convex_to_dof(d)[0]
+         @param d the global dof number.
     */
     virtual size_type first_convex_of_basic_dof(size_type d) const;
     size_type first_convex_of_dof(size_type d) const IS_DEPRECATED
     { return first_convex_of_basic_dof(d); }
-    /** Return the list of convexes attached to the specified dof 
-	@param d the global dof number.
-	@return an array of convex numbers.
+    /** Return the list of convexes attached to the specified dof
+        @param d the global dof number.
+        @return an array of convex numbers.
      */
     virtual const mesh::ind_cv_ct &convex_to_basic_dof(size_type d) const;
     const mesh::ind_cv_ct &convex_to_dof(size_type d) const IS_DEPRECATED
@@ -536,51 +538,54 @@ namespace getfem {
       return use_reduction ? gmm::mat_nrows(R_) : nb_total_dof;
     }
     /** Get a list of basic dof lying on a given mesh_region.
-	@param b the mesh_region.
-	@return the list in a dal::bit_vector.
+        @param b the mesh_region.
+        @return the list in a dal::bit_vector.
     */
     virtual dal::bit_vector basic_dof_on_region(const mesh_region &b) const;
     /** Get a list of dof lying on a given mesh_region. For a reduced mesh_fem
-	a dof is lying on a region if its potential corresponding shape
-	function is nonzero on this region. The extension matrix is used
-	to make the correspondance between basic and reduced dofs.
-	@param b the mesh_region.
-	@return the list in a dal::bit_vector.
-    */    
+        a dof is lying on a region if its potential corresponding shape
+        function is nonzero on this region. The extension matrix is used
+        to make the correspondance between basic and reduced dofs.
+        @param b the mesh_region.
+        @return the list in a dal::bit_vector.
+    */
     dal::bit_vector dof_on_region(const mesh_region &b) const;
     dal::bit_vector dof_on_set(const mesh_region &b) const IS_DEPRECATED
     { return dof_on_region(b); }
 
     void set_dof_partition(size_type cv, unsigned partition_num) {
-      if (dof_partition.empty() && partition_num == 0) return;      
-      if (dof_partition.size() < linked_mesh().convex_index().last_true()+1) 
-	dof_partition.resize(linked_mesh().convex_index().last_true()+1);
+      if (dof_partition.empty() && partition_num == 0) return;
+      if (dof_partition.size() < linked_mesh().convex_index().last_true()+1)
+        dof_partition.resize(linked_mesh().convex_index().last_true()+1);
       if (dof_partition.at(cv) != partition_num) {
-	dof_partition[cv] = partition_num;
-	dof_enumeration_made = false;
+        dof_partition[cv] = partition_num;
+        dof_enumeration_made = false;
       }
     }
     unsigned get_dof_partition(size_type cv) const {
-      return (cv < dof_partition.size() ? unsigned(dof_partition[cv]) : 0); 
+      return (cv < dof_partition.size() ? unsigned(dof_partition[cv]) : 0);
     }
     void clear_dof_partition() { dof_partition.clear(); }
-    
+
     size_type memsize() const {
-      return dof_structure.memsize() + 
-	sizeof(mesh_fem) - sizeof(bgeot::mesh_structure) +
-	f_elems.memsize() + fe_convex.memsize();
+      return dof_structure.memsize() +
+        sizeof(mesh_fem) - sizeof(bgeot::mesh_structure) +
+        f_elems.memsize() + fe_convex.memsize();
     }
     void init_with_mesh(const mesh &me, dim_type Q = 1);
-    /** Build a new mesh_fem. A mesh object must be supplied. 
-	@param me the linked mesh.
-	@param Q the Q dimension (see mesh_fem::get_qdim).
+    /** Build a new mesh_fem. A mesh object must be supplied.
+        @param me the linked mesh.
+        @param Q the Q dimension (see mesh_fem::get_qdim).
     */
     explicit mesh_fem(const mesh &me, dim_type Q = 1);
     mesh_fem(void);
+    mesh_fem(const mesh_fem &mf);
+    mesh_fem &operator=(const mesh_fem &mf);
+
     virtual ~mesh_fem();
     virtual void clear(void);
-    /** Read the mesh_fem from a stream. 
-	@param ist the stream.
+    /** Read the mesh_fem from a stream.
+        @param ist the stream.
      */
     virtual void read_from_file(std::istream &ist);
     /** Read the mesh_fem from a file.
@@ -592,30 +597,29 @@ namespace getfem {
     void write_reduction_matrices_to_file(std::ostream &ost) const;
     /** Write the mesh_fem to a stream. */
     virtual void write_to_file(std::ostream &ost) const;
-    /** Write the mesh_fem to a file. 
+    /** Write the mesh_fem to a file.
 
-	@param name the file name
+        @param name the file name
 
-	@param with_mesh if set, then the linked_mesh() will also be
-	saved to the file.
+        @param with_mesh if set, then the linked_mesh() will also be
+        saved to the file.
     */
     void write_to_file(const std::string &name, bool with_mesh=false) const;
   };
 
   /** Gives the descriptor of a classical finite element method of degree K
-      on mesh.  
-      
+      on mesh.
+
       The mesh_fem won't be destroyed until its linked_mesh is
       destroyed. All the mesh_fem built by this function are stored
       in a cache, which means that calling this function twice with
       the same arguments will return the same mesh_fem object. A
       consequence is that you should NEVER modify this mesh_fem!
    */
-  const mesh_fem &classical_mesh_fem(const mesh &mesh, dim_type degree,
-				     dim_type qdim = 1);
+  const mesh_fem &classical_mesh_fem(const mesh &mesh, dim_type degree, dim_type qdim = 1);
 
-  /* Dummy mesh_fem for default parameter of functions. */
-  const mesh_fem &dummy_mesh_fem(void);
+  /** Dummy mesh_fem for default parameter of functions. */
+  const mesh_fem &dummy_mesh_fem();
 
 
   /** Given a mesh_fem @param mf and a vector @param vec of size equal to
@@ -644,6 +648,13 @@ namespace getfem {
     }
   }
 
+  void vectorize_base_tensor(const base_tensor &t, base_matrix &vt,
+                             size_type ndof, size_type qdim, size_type N);
+
+  void vectorize_grad_base_tensor(const base_tensor &t, base_tensor &vt,
+                                  size_type ndof, size_type qdim, size_type N);
+
+
 }  /* end of namespace getfem.                                             */
 
 
