@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2004-2015 Yves Renard
+ Copyright (C) 2004-2016 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -47,10 +47,6 @@
 
 //#include "gmm/gmm_inoutput.h"
 
-// metis necessary in this header only due to the old bricks system
-#if defined GETFEM_HAVE_METIS && !defined GETFEM_HAVE_METIS_OLD_API
-#  include <metis.h>
-#endif
 
 namespace getfem {
 
@@ -263,7 +259,7 @@ namespace getfem {
       conv_r = r;
       return ((it <= 1 && r < first_res)
               || (r <= first_res * alpha_max_ratio && r <= alpha_threshold_res)
-              || (conv_alpha <= alpha_min)
+              || (conv_alpha <= alpha_min && r < first_res * 1e5)
               || it >= itmax);
     }
     simplest_newton_line_search
@@ -554,28 +550,29 @@ namespace getfem {
 
   typedef abstract_linear_solver<model_real_sparse_matrix,
                                  model_real_plain_vector> rmodel_linear_solver;
-  typedef dal::shared_ptr<rmodel_linear_solver> rmodel_plsolver_type;
+  typedef std::shared_ptr<rmodel_linear_solver> rmodel_plsolver_type;
   typedef abstract_linear_solver<model_complex_sparse_matrix,
                                  model_complex_plain_vector>
           cmodel_linear_solver;
-  typedef dal::shared_ptr<cmodel_linear_solver> cmodel_plsolver_type;
+  typedef std::shared_ptr<cmodel_linear_solver> cmodel_plsolver_type;
 
 
   template<typename MATRIX, typename VECTOR>
-  dal::shared_ptr<abstract_linear_solver<MATRIX, VECTOR> >
+  std::shared_ptr<abstract_linear_solver<MATRIX, VECTOR>>
   default_linear_solver(const model &md) {
-    dal::shared_ptr<abstract_linear_solver<MATRIX, VECTOR> > p;
 
 #if GETFEM_PARA_LEVEL == 1 && GETFEM_PARA_SOLVER == MUMPS_PARA_SOLVER
     if (md.is_symmetric())
-      p.reset(new linear_solver_mumps_sym<MATRIX, VECTOR>);
+      return std::make_shared<linear_solver_mumps_sym<MATRIX, VECTOR>>();
     else
-      p.reset(new linear_solver_mumps<MATRIX, VECTOR>);
+      return std::make_shared<linear_solver_mumps<MATRIX, VECTOR>>();
 #elif GETFEM_PARA_LEVEL > 1 && GETFEM_PARA_SOLVER == MUMPS_PARA_SOLVER
     if (md.is_symmetric())
-      p.reset(new linear_solver_distributed_mumps_sym<MATRIX, VECTOR>);
-    else
-      p.reset(new linear_solver_distributed_mumps<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_distributed_mumps_sym<MATRIX, VECTOR>>();
+      else
+	return std::make_shared
+	  <linear_solver_distributed_mumps<MATRIX, VECTOR>>();
 #else
     size_type ndof = md.nb_dof(), max3d = 15000, dim = md.leading_dimension();
 # ifdef GMM_USES_MUMPS
@@ -584,59 +581,65 @@ namespace getfem {
     if ((ndof<300000 && dim<=2) || (ndof<max3d && dim<=3) || (ndof<1000)) {
 # ifdef GMM_USES_MUMPS
       if (md.is_symmetric())
-        p.reset(new linear_solver_mumps_sym<MATRIX, VECTOR>);
+	return std::make_shared<linear_solver_mumps_sym<MATRIX, VECTOR>>();
       else
-        p.reset(new linear_solver_mumps<MATRIX, VECTOR>);
+	return std::make_shared<linear_solver_mumps<MATRIX, VECTOR>>();
 # else
-      p.reset(new linear_solver_superlu<MATRIX, VECTOR>);
+      return std::make_shared<linear_solver_superlu<MATRIX, VECTOR>>();
 # endif
     }
     else {
       if (md.is_coercive())
-        p.reset(new linear_solver_cg_preconditioned_ildlt<MATRIX, VECTOR>);
+	return std::make_shared
+	  <linear_solver_cg_preconditioned_ildlt<MATRIX, VECTOR>>();
       else {
         if (dim <= 2)
-          p.reset(new
-                  linear_solver_gmres_preconditioned_ilut<MATRIX,VECTOR>);
-        else
-          p.reset(new
-                  linear_solver_gmres_preconditioned_ilu<MATRIX,VECTOR>);
+	  return std::make_shared
+	    <linear_solver_gmres_preconditioned_ilut<MATRIX,VECTOR>>();
+	  else
+	    return std::make_shared
+	      <linear_solver_gmres_preconditioned_ilu<MATRIX,VECTOR>>();
       }
     }
 #endif
-    return p;
+    return std::shared_ptr<abstract_linear_solver<MATRIX, VECTOR>>();
   }
 
   template <typename MATRIX, typename VECTOR>
-  dal::shared_ptr<abstract_linear_solver<MATRIX, VECTOR> >
+  std::shared_ptr<abstract_linear_solver<MATRIX, VECTOR>>
   select_linear_solver(const model &md, const std::string &name) {
-    dal::shared_ptr<abstract_linear_solver<MATRIX, VECTOR> > p;
+    std::shared_ptr<abstract_linear_solver<MATRIX, VECTOR>> p;
     if (bgeot::casecmp(name, "superlu") == 0)
-      p.reset(new linear_solver_superlu<MATRIX, VECTOR>);
+      return std::make_shared<linear_solver_superlu<MATRIX, VECTOR>>();
     else if (bgeot::casecmp(name, "mumps") == 0) {
 #ifdef GMM_USES_MUMPS
 # if GETFEM_PARA_LEVEL <= 1
-      p.reset(new linear_solver_mumps<MATRIX, VECTOR>);
+      return std::make_shared<linear_solver_mumps<MATRIX, VECTOR>>();
 # else
-      p.reset(new linear_solver_distributed_mumps<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_distributed_mumps<MATRIX, VECTOR>>();
 # endif
 #else
       GMM_ASSERT1(false, "Mumps is not interfaced");
 #endif
     }
     else if (bgeot::casecmp(name, "cg/ildlt") == 0)
-      p.reset(new linear_solver_cg_preconditioned_ildlt<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_cg_preconditioned_ildlt<MATRIX, VECTOR>>();
     else if (bgeot::casecmp(name, "gmres/ilu") == 0)
-      p.reset(new linear_solver_gmres_preconditioned_ilu<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_gmres_preconditioned_ilu<MATRIX, VECTOR>>();
     else if (bgeot::casecmp(name, "gmres/ilut") == 0)
-      p.reset(new linear_solver_gmres_preconditioned_ilut<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_gmres_preconditioned_ilut<MATRIX, VECTOR>>();
     else if (bgeot::casecmp(name, "gmres/ilutp") == 0)
-      p.reset(new linear_solver_gmres_preconditioned_ilutp<MATRIX, VECTOR>);
+      return std::make_shared
+	<linear_solver_gmres_preconditioned_ilutp<MATRIX, VECTOR>>();
     else if (bgeot::casecmp(name, "auto") == 0)
-      p = default_linear_solver<MATRIX, VECTOR>(md);
+      return default_linear_solver<MATRIX, VECTOR>(md);
     else
       GMM_ASSERT1(false, "Unknown linear solver");
-    return p;
+    return std::shared_ptr<abstract_linear_solver<MATRIX, VECTOR>>();
   }
 
   inline rmodel_plsolver_type rselect_linear_solver(const model &md,
