@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2013-2015 Yves Renard, Konstantinos Poulios.
+ Copyright (C) 2013-2016 Yves Renard, Konstantinos Poulios.
 
  This file is a part of GetFEM++
 
@@ -50,40 +50,6 @@ namespace getfem {
       }
   }
 
-  void vectorize_base_tensor(const base_tensor &t, base_matrix &vt,
-                             size_type ndof, size_type qdim, size_type N) {
-    GMM_ASSERT1(qdim == N || qdim == 1, "mixed intrinsic vector and "
-                "tensorised fem is not supported");
-    gmm::resize(vt, ndof, N);
-    ndof = (ndof*qdim)/N;
-    if (qdim == 1) {
-      gmm::clear(vt);
-      base_tensor::const_iterator it = t.begin();
-      for (size_type i = 0; i < ndof; ++i, ++it)
-        for (size_type j = 0; j < N; ++j) vt(i*N+j, j) = *it;
-    } else if (qdim == N) {
-      gmm::copy(t.as_vector(), vt.as_vector());
-    }
-  }
-
-  void vectorize_grad_base_tensor(const base_tensor &t, base_tensor &vt,
-                                         size_type ndof, size_type qdim,
-                                         size_type N) {
-    GMM_ASSERT1(qdim == N || qdim == 1, "mixed intrinsic vector and "
-                  "tensorised fem is not supported");
-    vt.adjust_sizes(bgeot::multi_index(ndof, N, N));
-    ndof = (ndof*qdim)/N;
-    if (qdim == 1) {
-      gmm::clear(vt.as_vector());
-      base_tensor::const_iterator it = t.begin();
-      for (size_type k = 0; k < N; ++k)
-        for (size_type i = 0; i < ndof; ++i, ++it)
-          for (size_type j = 0; j < N; ++j) vt(i*N+j, j, k) = *it;
-    } else if (qdim == N) {
-      gmm::copy(t.as_vector(), vt.as_vector());
-    }
-  }
-
   //=========================================================================
   //
   //  Structure which store the contact boundaries, rigid obstacles and
@@ -117,7 +83,7 @@ namespace getfem {
     return i;
   }
 
-  void multi_contact_frame::extend_vectors(void) {
+  void multi_contact_frame::extend_vectors() {
     dal::bit_vector iU, ilambda;
     for (size_type i = 0; i < contact_boundaries.size(); ++i) {
       size_type ind_U = contact_boundaries[i].ind_U;
@@ -141,7 +107,7 @@ namespace getfem {
     }
   }
 
-  void multi_contact_frame::normal_cone_simplification(void) {
+  void multi_contact_frame::normal_cone_simplification() {
     if (nodes_mode) {
       scalar_type threshold = ::cos(cut_angle);
       for (size_type i = 0; i < boundary_points_info.size(); ++i) {
@@ -225,7 +191,7 @@ namespace getfem {
     if (!found) sfi.push_back(face_info(ib, ie, iff));
   }
 
-  void multi_contact_frame::clear_aux_info(void) {
+  void multi_contact_frame::clear_aux_info() {
     boundary_points = std::vector<base_node>();
     boundary_points_info = std::vector<boundary_point>();
     element_boxes.clear();
@@ -433,7 +399,7 @@ namespace getfem {
           else {
             pintegration_method pim = mim.int_method_of_element(cv);
             GMM_ASSERT1(pim, "Integration method should be defined");
-            pfp = fppool(pf_s,&(pim->approx_method()->integration_points()));
+            pfp = fppool(pf_s, pim->approx_method()->pintegration_points());
             nbptf = pim->approx_method()->nb_points_on_face(v.f());
             indpt.resize(nbptf); indpfp.resize(nbptf);
             for (short_type ip = 0; ip < nbptf; ++ip)
@@ -475,7 +441,7 @@ namespace getfem {
       }
   }
 
-  void multi_contact_frame::compute_potential_contact_pairs_delaunay(void) {
+  void multi_contact_frame::compute_potential_contact_pairs_delaunay() {
 
     compute_boundary_points();
     normal_cone_simplification();
@@ -583,7 +549,7 @@ namespace getfem {
   }
 
 
-  void multi_contact_frame::compute_influence_boxes(void) {
+  void multi_contact_frame::compute_influence_boxes() {
     fem_precomp_pool fppool;
     bool avert = false;
     base_matrix G;
@@ -608,7 +574,7 @@ namespace getfem {
           size_type cv = v.cv();
           bgeot::pgeometric_trans pgt = m.trans_of_convex(cv);
           pfem pf_s = mfu.fem_of_element(cv);
-          pfem_precomp pfp = fppool(pf_s, &(pgt->geometric_nodes()));
+          pfem_precomp pfp = fppool(pf_s, pgt->pgeometric_nodes());
           if (!ref_conf)
             slice_vector_on_basic_dof_of_element(mfu, U, cv, coeff);
           bgeot::vectors_to_base_matrix
@@ -684,7 +650,7 @@ namespace getfem {
       }
   }
 
-  void multi_contact_frame::compute_potential_contact_pairs_influence_boxes(void) {
+  void multi_contact_frame::compute_potential_contact_pairs_influence_boxes() {
     compute_influence_boxes();
     compute_boundary_points(!self_contact); // vraiment necessaire ?
     normal_cone_simplification();
@@ -862,7 +828,7 @@ namespace getfem {
   // - Case J3 of valid/invalid contact situations is not really taken into
   //   account. How to take it into account in a cheap way ?
 
-  void multi_contact_frame::compute_contact_pairs(void) {
+  void multi_contact_frame::compute_contact_pairs() {
     base_matrix G, grad(N,N);
     model_real_plain_vector coeff;
     base_small_vector a(N-1), ny(N);
@@ -1327,8 +1293,9 @@ namespace getfem {
       mutable model_real_plain_vector U_unred; // Unreduced displacement
       bool slave;
  
-      contact_boundary(void) {}
-      contact_boundary(size_type r, const mesh_fem *mf, const std::string dn,
+      contact_boundary()
+        : region(-1), mfu(0), dispname(""), U(0), U_unred(0), slave(false) {}
+      contact_boundary(size_type r, const mesh_fem *mf, const std::string &dn,
                        bool sl)
         : region(r), mfu(mf), dispname(dn), slave(sl) {}
     };
@@ -1338,7 +1305,8 @@ namespace getfem {
       size_type ind_element;   // Element number
       short_type ind_face;     // Face number in element
       base_small_vector mean_normal;   // Mean outward normal unit vector
-      face_box_info(void) {}
+      face_box_info()
+        : ind_boundary(-1), ind_element(-1), ind_face(-1), mean_normal(0) {}
       face_box_info(size_type ib, size_type ie,
                     short_type iff, const base_small_vector &n)
         : ind_boundary(ib), ind_element(ie), ind_face(iff), mean_normal(n) {}
@@ -1351,10 +1319,70 @@ namespace getfem {
     typedef std::map<const mesh *, std::vector<size_type> > mesh_boundary_cor;
     mesh_boundary_cor boundary_for_mesh;
     
-    struct obstacle {
-      ga_function f;
-      ga_function der_f;
+    class obstacle {
+      const model *md;
+      const ga_workspace *parent_workspace;
+      std::string expr;
+
       mutable base_vector X;
+      mutable ga_function f, der_f;
+      mutable bool compiled;
+
+      void compile() const {
+        if (md)
+          f = ga_function(*md, expr);
+        else if (parent_workspace)
+          f = ga_function(*parent_workspace, expr);
+        else
+          f = ga_function(expr);
+        size_type N = gmm::vect_size(X);
+        f.workspace().add_fixed_size_variable("X", gmm::sub_interval(0, N), X);
+        if (N >= 1) f.workspace().add_macro("x", "X(1)");
+        if (N >= 2) f.workspace().add_macro("y", "X(2)");
+        if (N >= 3) f.workspace().add_macro("z", "X(3)");
+        if (N >= 4) f.workspace().add_macro("w", "X(4)");
+        f.compile();
+        der_f = f;
+        der_f.derivative("X");
+        compiled = true;
+      }
+
+    public:
+
+      base_vector &point() const { return X; }
+
+      const base_tensor &eval() const {
+        if (!compiled) compile();
+        return f.eval();
+      }
+      const base_tensor &eval_derivative() const {
+        if (!compiled) compile();
+        return der_f.eval();
+      }
+
+      obstacle()
+        : md(0), parent_workspace(0), expr(""), X(0), f(), der_f() {}
+      obstacle(const model &md_, const std::string &expr_, size_type N)
+        : md(&md_), parent_workspace(0), expr(expr_), X(N),
+          f(), der_f(), compiled(false) {}
+      obstacle(const ga_workspace &parent_workspace_,
+               const std::string &expr_, size_type N)
+        : md(0), parent_workspace(&parent_workspace_), expr(expr_), X(N),
+          f(), der_f(), compiled(false) {}
+      obstacle(const obstacle &obs)
+        : md(obs.md), parent_workspace(obs.parent_workspace), expr(obs.expr),
+          X(obs.X), f(), der_f(), compiled(false) {}
+      obstacle &operator =(const obstacle& obs) {
+        md = obs.md;
+        parent_workspace = obs.parent_workspace;
+        expr = obs.expr;
+        X = obs.X;
+        f = ga_function();
+        der_f = ga_function();
+        compiled = false;
+        return *this;
+      }
+      ~obstacle() {}
     };
 
     std::vector<obstacle> obstacles;
@@ -1363,7 +1391,7 @@ namespace getfem {
     mutable std::vector<face_box_info> face_boxes_info;
 
 
-    void compute_face_boxes(void) const { // called by init
+    void compute_face_boxes() const { // called by init
       fem_precomp_pool fppool;
       base_matrix G;
       model_real_plain_vector coeff;
@@ -1371,7 +1399,7 @@ namespace getfem {
       face_boxes_info.resize(0);
 
       for (size_type i = 0; i < contact_boundaries.size(); ++i) {
-        const contact_boundary &cb =  contact_boundaries[i];
+        const contact_boundary &cb = contact_boundaries[i];
         if (! cb.slave) {
           size_type bnum = cb.region;
           const mesh_fem &mfu = *(cb.mfu);
@@ -1391,7 +1419,7 @@ namespace getfem {
             size_type cv = v.cv();
             bgeot::pgeometric_trans pgt = m.trans_of_convex(cv);
             pfem pf_s = mfu.fem_of_element(cv);
-            pfem_precomp pfp = fppool(pf_s, &(pgt->geometric_nodes()));
+            pfem_precomp pfp = fppool(pf_s, pgt->pgeometric_nodes());
             slice_vector_on_basic_dof_of_element(mfu, U, cv, coeff);
             bgeot::vectors_to_base_matrix
               (G, mfu.linked_mesh().points_of_convex(cv));
@@ -1452,34 +1480,12 @@ namespace getfem {
 
     void add_rigid_obstacle(const model &md, const std::string &expr,
                             size_type N) {
-      obstacles.push_back(obstacle());
-      obstacles.back().f = ga_function(md, expr);
-      gmm::resize(obstacles.back().X, N);
-      obstacles.back().f.workspace().add_fixed_size_variable
-        ("X", gmm::sub_interval(0, N), obstacles.back().X);
-      if (N >= 1) obstacles.back().f.workspace().add_macro("x", "X(1)");
-      if (N >= 2) obstacles.back().f.workspace().add_macro("y", "X(2)");
-      if (N >= 3) obstacles.back().f.workspace().add_macro("z", "X(3)");
-      if (N >= 4) obstacles.back().f.workspace().add_macro("w", "X(4)");
-      obstacles.back().f.compile();
-      obstacles.back().der_f = obstacles.back().f;
-      obstacles.back().der_f.derivative("X");
+      obstacles.push_back(obstacle(md, expr, N));
     }
 
-    void add_rigid_obstacle(const ga_workspace &workspace,
+    void add_rigid_obstacle(const ga_workspace &parent_workspace,
                             const std::string &expr, size_type N) {
-      obstacles.push_back(obstacle());
-      obstacles.back().f = ga_function(workspace, expr);
-      gmm::resize(obstacles.back().X, N);
-      obstacles.back().f.workspace().add_fixed_size_variable
-        ("X", gmm::sub_interval(0, N), obstacles.back().X);
-      if (N >= 1) obstacles.back().f.workspace().add_macro("x", "X(1)");
-      if (N >= 2) obstacles.back().f.workspace().add_macro("y", "X(2)");
-      if (N >= 3) obstacles.back().f.workspace().add_macro("z", "X(3)");
-      if (N >= 4) obstacles.back().f.workspace().add_macro("w", "X(4)");
-      obstacles.back().f.compile();
-      obstacles.back().der_f = obstacles.back().f;
-      obstacles.back().der_f.derivative("X");
+     obstacles.push_back(obstacle(parent_workspace, expr, N));
     }
 
     void add_contact_boundary(const model &md, const mesh &m,
@@ -1487,13 +1493,14 @@ namespace getfem {
                               size_type region, bool slave) {
       const mesh_fem *mf = 0;
       if (md.variable_group_exists(dispname)) {
-        const std::vector<std::string> &t = md.variable_group(dispname);
-        for (size_type i = 0; i < t.size(); ++i) {
-          const mesh_fem *mf2 = md.pmesh_fem_of_variable(t[i]);
+        for (const std::string &t : md.variable_group(dispname)) {
+          const mesh_fem *mf2 = md.pmesh_fem_of_variable(t);
           if (mf2 && &(mf2->linked_mesh()) == &m)
             { mf = mf2; break; }
         }
-      } else mf = md.pmesh_fem_of_variable(dispname);
+      } else
+        mf = md.pmesh_fem_of_variable(dispname);
+
       GMM_ASSERT1(mf, "Displacement should be a fem variable");
       contact_boundary cb(region, mf, dispname, slave);
       boundary_for_mesh[&(mf->linked_mesh())]
@@ -1506,13 +1513,14 @@ namespace getfem {
                               size_type region, bool slave) {
       const mesh_fem *mf = 0;
       if (workspace.variable_group_exists(dispname)) {
-        const std::vector<std::string> &t = workspace.variable_group(dispname);
-        for (size_type i = 0; i < t.size(); ++i) {
-          const mesh_fem *mf2 = workspace.associated_mf(t[i]);
+        for (const std::string &t : workspace.variable_group(dispname)) {
+          const mesh_fem *mf2 = workspace.associated_mf(t);
           if (mf2 && &(mf2->linked_mesh()) == &m)
             { mf = mf2; break; }
         }
-      } else mf = workspace.associated_mf(dispname);
+      } else
+        mf = workspace.associated_mf(dispname);
+
       GMM_ASSERT1(mf, "Displacement should be a fem variable");
       contact_boundary cb(region, mf, dispname, slave);
       boundary_for_mesh[&(mf->linked_mesh())]
@@ -1530,27 +1538,23 @@ namespace getfem {
       // GMM_ASSERT1(mf, "Internal error");
       // const mesh &m_x = mf->linked_mesh();
 
-      mesh_boundary_cor::const_iterator it =  boundary_for_mesh.find(&m_x);
+      mesh_boundary_cor::const_iterator it = boundary_for_mesh.find(&m_x);
       GMM_ASSERT1(it != boundary_for_mesh.end(), "Raytracing interpolate "
                   "transformation: Mesh with no declared contact boundary");
-      const std::vector<size_type> &boundaries_ind = it->second;
-      for (size_type i = 0; i < boundaries_ind.size(); ++i) {
-        const contact_boundary &cb =  contact_boundaries[boundaries_ind[i]];
+      for (const size_type &boundary_ind : it->second) {
+        const contact_boundary &cb = contact_boundaries[boundary_ind];
         const std::string &dispname_x
-          =  workspace.variable_in_group(cb.dispname, m_x);
+          = workspace.variable_in_group(cb.dispname, m_x);
         if (!ignore_data || !(workspace.is_constant(dispname_x)))
           vars.insert(var_trans_pair(dispname_x, ""));
       }
 
-      for (size_type i = 0; i < contact_boundaries.size(); ++i) {
-        const contact_boundary &cb =  contact_boundaries[i];
+      for (const contact_boundary &cb : contact_boundaries) {
         if (!(cb.slave)) {
           if (expand_groups && workspace.variable_group_exists(cb.dispname)
               && (!ignore_data || !(workspace.is_constant(cb.dispname)))) {
-            const std::vector<std::string> &t
-              = workspace.variable_group(cb.dispname);
-          for (size_type j = 0; j < t.size(); ++j)
-            vars.insert(var_trans_pair(t[j], interpolate_name));
+            for (const std::string &t : workspace.variable_group(cb.dispname))
+              vars.insert(var_trans_pair(t, interpolate_name));
           } else {
             if (!ignore_data || !(workspace.is_constant(cb.dispname)))
               vars.insert(var_trans_pair(cb.dispname, interpolate_name));
@@ -1560,8 +1564,7 @@ namespace getfem {
     }
 
     void init(const ga_workspace &workspace) const {
-      for (size_type i = 0; i < contact_boundaries.size(); ++i) {
-        const contact_boundary &cb =  contact_boundaries[i];
+      for (const contact_boundary &cb : contact_boundaries) {
         const mesh_fem &mfu = *(cb.mfu);
         const std::string dispname_x
           =  workspace.variable_in_group(cb.dispname, mfu.linked_mesh());
@@ -1577,11 +1580,11 @@ namespace getfem {
       compute_face_boxes();
     };
 
-    void finalize(void) const {
+    void finalize() const {
       face_boxes.clear();
       face_boxes_info = std::vector<face_box_info>();
-      for (size_type i = 0; i < contact_boundaries.size(); ++i)
-        contact_boundaries[i].U_unred = model_real_plain_vector();
+      for (const contact_boundary &cb : contact_boundaries)
+        cb.U_unred = model_real_plain_vector();
     }
 
     int transform(const ga_workspace &workspace, const mesh &m_x,
@@ -1603,16 +1606,15 @@ namespace getfem {
       mesh_boundary_cor::const_iterator it =  boundary_for_mesh.find(&m_x);
       GMM_ASSERT1(it != boundary_for_mesh.end(),
                   "Mesh with no declared contact boundary");
-      const std::vector<size_type> &boundaries_ind = it->second;
       size_type ib_x = size_type(-1);
-      for (size_type i = 0; i < boundaries_ind.size(); ++i) {
-        const contact_boundary &cb =  contact_boundaries[boundaries_ind[i]];
+      for (const size_type &boundary_ind : it->second) {
+        const contact_boundary &cb = contact_boundaries[boundary_ind];
         if (m_x.region(cb.region).is_in(cv_x, face_x))
-          { ib_x = boundaries_ind[i]; break; }
+          { ib_x = boundary_ind; break; }
       }
       GMM_ASSERT1(ib_x != size_type(-1),
                   "No contact region found for this point");
-      const contact_boundary &cb_x =  contact_boundaries[ib_x];
+      const contact_boundary &cb_x = contact_boundaries[ib_x];
       const mesh_fem &mfu_x = *(cb_x.mfu); 
       pfem pfu_x = mfu_x.fem_of_element(cv_x);
       size_type N = mfu_x.linked_mesh().dim();
@@ -1653,44 +1655,47 @@ namespace getfem {
       size_type irigid_obstacle(-1);
       for (size_type i = 0; i < obstacles.size(); ++i) {
         const obstacle &obs = obstacles[i];
-        gmm::copy(pt_x, obs.X);
-        const base_tensor &t = obs.f.eval();
+        gmm::copy(pt_x, obs.point());
+        const base_tensor &t = obs.eval();
         
         GMM_ASSERT1(t.size() == 1, "Obstacle level set function as to be "
                     "a scalar valued one");
         d1 = t[0];
         // cout << "d1 = " << d1 << endl;
         if (gmm::abs(d1) < release_distance && d1 < d0) {
-          const base_tensor &t_der = obs.der_f.eval();
+          const base_tensor &t_der = obs.eval_derivative();
           GMM_ASSERT1(t_der.size() == n_x.size(), "Bad derivative size");
-          if (gmm::vect_sp(t_der.as_vector(), n_x) < scalar_type(0)) 
-            { d0 = d1; irigid_obstacle = i; gmm::copy(t_der.as_vector(),n_y); }
+          if (gmm::vect_sp(t_der.as_vector(), n_x) < scalar_type(0)) {
+            d0 = d1;
+            irigid_obstacle = i;
+            gmm::copy(t_der.as_vector(), n_y);
+          }
         }
       }
 
       if (irigid_obstacle != size_type(-1)) {
         // cout << "Testing obstacle " << irigid_obstacle << endl;
         const obstacle &obs = obstacles[irigid_obstacle];
-        gmm::copy(pt_x, obs.X);
+        gmm::copy(pt_x, obs.point());
         gmm::copy(pt_x, pt_y);
         size_type nit = 0, nb_fail = 0;
         scalar_type alpha(0), beta(0);
         d1 = d0;
         
         while (gmm::abs(d1) > 1E-13 && ++nit < 50 && nb_fail < 3) {
-          if (nit != 1) gmm::copy(obs.der_f.eval().as_vector(), n_y);
+          if (nit != 1) gmm::copy(obs.eval_derivative().as_vector(), n_y);
 
           for (scalar_type lambda(1); lambda >= 1E-3; lambda/=scalar_type(2)) {
             alpha = beta - lambda * d1 / gmm::vect_sp(n_y, n_x);
-            gmm::add(pt_x, gmm::scaled(n_x, alpha), obs.X);
-            d2 = obs.f.eval()[0];
+            gmm::add(pt_x, gmm::scaled(n_x, alpha), obs.point());
+            d2 = obs.eval()[0];
             if (gmm::abs(d2) < gmm::abs(d1)) break;
           }
           if (gmm::abs(beta - d1 / gmm::vect_sp(n_y, n_x)) > scalar_type(500))
             nb_fail++;
           beta = alpha; d1 = d2;
         }
-        gmm::copy(obs.X, pt_y);
+        gmm::copy(obs.point(), pt_y);
 
         if (gmm::abs(d1) > 1E-8) {
            GMM_WARNING1("Raytrace on rigid obstacle failed");
@@ -1698,10 +1703,12 @@ namespace getfem {
         else if (gmm::vect_dist2(pt_y, pt_x) <= release_distance) {
           n_y /= gmm::vect_norm2(n_y);
           d0 = gmm::vect_dist2(pt_y, pt_x) * gmm::sgn(d0);
-          stored_pt_y = stored_pt_y_ref = pt_y; stored_n_y = n_y, 
+          stored_pt_y = stored_pt_y_ref = pt_y;
+          stored_n_y = n_y;
           stored_signed_distance = d0;
           first_pair_found = true;
-        } else irigid_obstacle = size_type(-1);
+        } else
+          irigid_obstacle = size_type(-1);
       }
 
       //
@@ -1718,11 +1725,10 @@ namespace getfem {
       // Iteration on potential contact pairs and application
       // of selection criteria
       //
-      bgeot::rtree::pbox_set::iterator it_cp = bset.begin();
-      for (; it_cp != bset.end(); ++it_cp) {
-        face_box_info &fbox_y = face_boxes_info[(*it_cp)->id];
+      for (const auto &pbox : bset) {
+        face_box_info &fbox_y = face_boxes_info[pbox->id];
         size_type ib_y = fbox_y.ind_boundary;
-        const contact_boundary &cb_y =  contact_boundaries[ib_y];
+        const contact_boundary &cb_y = contact_boundaries[ib_y];
         const mesh_fem &mfu_y = *(cb_y.mfu);
         const mesh &m_y = mfu_y.linked_mesh();
         size_type cv_y = fbox_y.ind_element;
@@ -1786,13 +1792,12 @@ namespace getfem {
         size_type nbfail = 0, niter = 0;
         for (;residual > 2E-12 && niter <= 30; ++niter) {
           
-          for (size_type subiter(0);;) {
+          for (size_type subiter(0); subiter <= 4; ++subiter) {
             pps(a, hessa);
             det = gmm::abs(gmm::lu_inverse(hessa, false));
             if (det > 1E-15) break;
             for (size_type i = 0; i < N-1; ++i)
               a[i] += gmm::random() * 1E-7;
-            if (++subiter > 4) break;
           }
           if (det <= 1E-15) break;
           // Computation of the descent direction
@@ -1877,13 +1882,15 @@ namespace getfem {
       }
 
       int ret_type = 0;
-      *m_t = 0; cv = size_type(-1); face_num = short_type(-1);
+      *m_t = 0;
+      cv = size_type(-1);
+      face_num = short_type(-1);
       if (irigid_obstacle != size_type(-1)) {
         P_ref = stored_pt_y; N_y = stored_n_y;
         ret_type = 2;
       } else if (first_pair_found) {
         *m_t = stored_m_y; cv = stored_cv_y; face_num = stored_face_y;
-        P_ref = stored_pt_y_ref;
+        P_ref = stored_pt_y_ref; N_y = stored_n_y;
         ret_type = 1;
       }
 
@@ -1973,27 +1980,25 @@ namespace getfem {
                   der_x(i, j) -= M2(j, k) * vgrad_base_ux(i, l, k)
                     * n_x[l] * stored_signed_distance;
 
-
           const std::string &dispname_x
             = workspace.variable_in_group(cb_x.dispname, m_x);
 
-          for (std::map<var_trans_pair, base_tensor>::iterator itd
-                 = derivatives.begin(); itd != derivatives.end(); ++itd) {
-            if (dispname_x.compare(itd->first.first) == 0 &&
-                itd->first.second.size() == 0) {
-              itd->second.adjust_sizes(ndof_ux, N);
-              gmm::copy(der_x.as_vector(), itd->second.as_vector());
+          for (auto&& d : derivatives) {
+            if (dispname_x.compare(d.first.varname) == 0 &&
+                d.first.transname.size() == 0) {
+              d.second.adjust_sizes(ndof_ux, N);
+              gmm::copy(der_x.as_vector(), d.second.as_vector());
             } else if (ret_type == 1 &&
-                       stored_dispname.compare(itd->first.first) == 0 &&
-                       itd->first.second.size() != 0) {
-              itd->second.adjust_sizes(ndof_uy, N);
-              gmm::copy(der_y.as_vector(), itd->second.as_vector());
-            } else itd->second.adjust_sizes(0, 0);
+                       stored_dispname.compare(d.first.varname) == 0 &&
+                       d.first.transname.size() != 0) {
+              d.second.adjust_sizes(ndof_uy, N);
+              gmm::copy(der_y.as_vector(), d.second.as_vector());
+            } else
+              d.second.adjust_sizes(0, 0);
           }
         } else {
-          for (std::map<var_trans_pair, base_tensor>::iterator itd
-                 = derivatives.begin(); itd != derivatives.end(); ++itd)
-            itd->second.adjust_sizes(0, 0);
+          for (auto&& d : derivatives)
+            d.second.adjust_sizes(0, 0);
         }
       }
       return ret_type;
@@ -2005,13 +2010,15 @@ namespace getfem {
 
   void add_raytracing_transformation
   (model &md, const std::string &transname, scalar_type d) {
-    pinterpolate_transformation p=new raytracing_interpolate_transformation(d);
+    pinterpolate_transformation
+      p = std::make_shared<raytracing_interpolate_transformation>(d);
     md.add_interpolate_transformation(transname, p);
   }
 
   void add_raytracing_transformation
   (ga_workspace &workspace, const std::string &transname, scalar_type d) {
-    pinterpolate_transformation p=new raytracing_interpolate_transformation(d);
+    pinterpolate_transformation
+      p = std::make_shared<raytracing_interpolate_transformation>(d);
     workspace.add_interpolate_transformation(transname, p);
   }
 
@@ -2164,11 +2171,12 @@ namespace getfem {
   // Coulomb_friction_coupled_projection(lambda, n, Vs, g, f, r)
   struct Coulomb_friction_coupled_projection : public ga_nonlinear_operator {
     bool result_size(const arg_list &args, bgeot::multi_index &sizes) const {
-      if (args.size() != 6 || args[1]->size() != args[0]->size()
-          || args[2]->size() != args[0]->size()
-          || args[3]->size() != 1 || args[4]->size() > 3 || args[4]->size() == 0
-          || args[5]->size() != 1 ) return false;
-      ga_init_vector(sizes, args[0]->sizes()[0]);
+      if (args.size() != 6) return false;
+      size_type N =  args[0]->size();
+      if (N == 0 || args[1]->size() != N || args[2]->size() != N
+	  || args[3]->size() != 1 || args[4]->size() > 3
+	  || args[4]->size() == 0 || args[5]->size() != 1 ) return false;
+      ga_init_vector(sizes, N);
       return true;
     }
     
@@ -2334,20 +2342,21 @@ namespace getfem {
     }
   };
 
-  static bool init_predef_operators(void) {
+  static bool init_predef_operators() {
 
     ga_predef_operator_tab &PREDEF_OPERATORS
       = dal::singleton<ga_predef_operator_tab>::instance();
     
     PREDEF_OPERATORS.add_method("Transformed_unit_vector",
-                                new Transformed_unit_vector());
+                                std::make_shared<Transformed_unit_vector>());
     PREDEF_OPERATORS.add_method("Coulomb_friction_coupled_projection",
-                                new Coulomb_friction_coupled_projection());
+                    std::make_shared<Coulomb_friction_coupled_projection>());
 
     return true;
    }
 
-  static bool predef_operators_initialized = init_predef_operators();
-
+  // declared in getfem_generic_assembly.cc
+  bool predef_operators_contact_initialized
+    = init_predef_operators();
 
 }  /* end of namespace getfem.                                             */
