@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2004-2012 Yves Renard, Konstantinos Poulios.
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2004-2016 Yves Renard, Konstantinos Poulios.
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 
@@ -24,23 +24,10 @@
 #include "getfem/getfem_contact_and_friction_common.h"
 #include "getfem/getfem_assembling.h"
 
-#include <getfem/getfem_arch_config.h>
-#if GETFEM_HAVE_MUPARSER_MUPARSER_H
-#include <muParser/muParser.h>
-#elif GETFEM_HAVE_MUPARSER_H
-#include <muParser.h>
-#endif
-
-#ifdef GETFEM_HAVE_QHULL_QHULL_H
-#include <getfem/getfem_mesher.h>
-#else
+#ifndef GETFEM_HAVE_QHULL_QHULL_H
 #include <getfem/bgeot_kdtree.h>
 #endif
 
-#ifdef _MSC_VER
-#define xor ^
-#endif
-
 namespace getfem {
 
   typedef bgeot::convex<base_node>::dref_convex_pt_ct dref_convex_pt_ct;
@@ -158,7 +145,7 @@ namespace getfem {
           base_node node1 = cn1->mf->point_of_basic_dof(cn1->dof);
           bgeot::index_node_pair ipt;
           scalar_type dist2 = tree2.nearest_neighbor(ipt, node1);
-          if (ipt.i >= 0 && dist2 < (*this)[ii1].dist2) {
+          if (ipt.i != size_type(-1) && dist2 < (*this)[ii1].dist2) {
             (*this)[ii1].cn_s = *cn1;
             (*this)[ii1].cn_m = cnl2[ipt.i];
             (*this)[ii1].dist2 = dist2;
@@ -173,7 +160,7 @@ namespace getfem {
           base_node node2 = cn2->mf->point_of_basic_dof(cn2->dof);
           bgeot::index_node_pair ipt;
           scalar_type dist2 = tree1.nearest_neighbor(ipt, node2);
-          if (ipt.i >= 0 && dist2 < (*this)[ii2].dist2) {
+          if (ipt.i != size_type(-1) && dist2 < (*this)[ii2].dist2) {
             (*this)[ii2].cn_s = *cn2;
             (*this)[ii2].cn_m = cnl1[ipt.i];
             (*this)[ii2].dist2 = dist2;
@@ -206,7 +193,7 @@ namespace getfem {
           for (size_type iv2 = iv1 + 1; iv2 < nb_vertices; ++iv2) {
             size_type v2 = facet_vertices[iv2];
             bool v2_on_surface1 = (v2 < size1);
-            if (v1_on_surface1 xor v2_on_surface1) {
+            if (v1_on_surface1 ^ v2_on_surface1) {
               bool already_in = false;
               size_type vv1 = (v1_on_surface1 ? v1 : v2);
               size_type vv2 = (v2_on_surface1 ? v1 : v2);
@@ -377,10 +364,12 @@ namespace getfem {
           std::vector<size_type> ind(0);
           for (size_type j = 0; j < P; ++j)
             if (j != i ) ind.push_back(j);
-          scalar_type det = gmm::lu_det(gmm::sub_matrix(bases_product,
-                                                        gmm::sub_interval(0, P-1),
-                                                        gmm::sub_index(ind)      ) );
-          gmm::add(gmm::scaled(gmm::mat_col(KK, i), (i % 2) ? -det : +det ), un);
+          gmm::sub_index SUBI(ind);
+          scalar_type det
+            = gmm::lu_det(gmm::sub_matrix(bases_product,
+                                          gmm::sub_interval(0, P-1), SUBI));
+          gmm::add(gmm::scaled(gmm::mat_col(KK, i), (i % 2) ? -det : +det ),
+                   un);
         }
       }
       // normalizing
@@ -429,7 +418,8 @@ namespace getfem {
         base_node master_node = cn_m->mf->point_of_basic_dof(cn_m->dof);
         base_node un_sel(3), proj_node_sel(3), proj_node_ref_sel(3);
         scalar_type is_in_min = 1e5;  //FIXME
-        size_type cv_sel = 0, fc_sel = 0;
+        size_type cv_sel = 0;
+        short_type fc_sel = 0;
         std::vector<size_type>::iterator cv;
         std::vector<short_type>::iterator fc;
         for (cv = cn_m->cvs.begin(), fc = cn_m->fcs.begin();
@@ -580,6 +570,8 @@ namespace getfem {
       is_init = true;
     }
 
+    void set_stationary(void) { really_stationary = true; }
+
     void precomp(const model_real_plain_vector &u1,
                  const model_real_plain_vector &u2,
                  const model_real_plain_vector &lambda_n,
@@ -782,7 +774,7 @@ namespace getfem {
 					  gmm::mat_nrows(T_t_u1));
 	    model_real_sparse_matrix tmp4(gmm::mat_ncols(T_t_u2),
 					  gmm::mat_nrows(T_t_u2));
-	    
+
 	    for (size_type i=0; i < nbc; ++i) {
 	      gmm::sub_interval SUBI(i*d, d);
 	      scalar_type th = - (std::min(vt0, RLN[i])) * friction_coeff[i];
@@ -802,7 +794,7 @@ namespace getfem {
 		    if (two_variables)
 		      gmm::add(gmm::scaled(gmm::mat_row(BT2,i*d+k),
 			   vg[k]*friction_coeff[i]), gmm::mat_col(T_u2_n, i));
-		  
+
 		    gmm::copy(gmm::scaled(gmm::mat_row(BBT1,i*d+k),
 					  -r*friction_coeff[i]*vg[k]),
 			      gmm::mat_col(tmp5, i*d+k));
@@ -823,7 +815,7 @@ namespace getfem {
 	    gmm::add(gmm::transposed(tmp3), T_t_u1);
 	    if (two_variables)
 	      gmm::add(gmm::transposed(tmp4), T_t_u2);
-	    
+
 	    if (augmentation_version == 2) {
 	      model_real_sparse_matrix tmp1(gmm::mat_ncols(BN1),
 					    gmm::mat_ncols(BN1));
@@ -983,7 +975,7 @@ namespace getfem {
           }
 	  for (size_type i = 0; i < nbc; ++i) {
 	    rlambda_n[i] = (lambda_n[i] - RLN[i]) / (r * alpha[i]);
-	    if (!contact_only) 
+	    if (!contact_only)
 	      for (size_type k = 0; k < d; ++k)
 		rlambda_t[i*d+k]
 		  = (lambda_t[i*d+k] - RLT[i*d+k]) / (r * alpha[i]);
@@ -1007,7 +999,7 @@ namespace getfem {
           }
 	  for (size_type i = 0; i < nbc; ++i) {
 	    rlambda_n[i] = (lambda_n[i] - RLN[i]) / (r * alpha[i]);
-	    if (!contact_only) 
+	    if (!contact_only)
 	      for (size_type k = 0; k < d; ++k)
 		rlambda_t[i*d+k]
 		  = (lambda_t[i*d+k] - RLT[i*d+k]) / (r * alpha[i]);
@@ -1040,7 +1032,7 @@ namespace getfem {
           }
 	  for (size_type i = 0; i < nbc; ++i) {
 	    rlambda_n[i] /= alpha[i];
-	    if (!contact_only) 
+	    if (!contact_only)
 	      for (size_type k = 0; k < d; ++k) rlambda_t[i*d+k] /= alpha[i];
 	  }
           break;
@@ -1074,7 +1066,7 @@ namespace getfem {
           if (two_variables) gmm::mult_add(BBN2, u2, rlambda_n);
 	  for (size_type i = 0; i < nbc; ++i) {
 	    rlambda_n[i] /= alpha[i];
-	    if (!contact_only) 
+	    if (!contact_only)
 	      for (size_type k = 0; k < d; ++k) rlambda_t[i*d+k] /= alpha[i];
 	  }
           break;
@@ -1092,13 +1084,13 @@ namespace getfem {
                                         model::real_veclist &,
                                         size_type /* region */,
                                         build_version version) const {
+      if (MPI_IS_MASTER()) {
       GMM_ASSERT1(mims.size() == 0, "Contact brick need no mesh_im");
       size_type nbvar = 2 + (contact_only ? 0 : 1) + (two_variables ? 1 : 0);
       GMM_ASSERT1(vl.size() == nbvar,
                   "Wrong number of variables for contact brick");
-           size_type nbdl = 3 + (contact_only ? 0 : 1) + (Tresca_version ? 1 : 0)
-        + (friction_dynamic_term ? 1 : 0);
-     
+      size_type nbdl = 3 + (contact_only ? 0 : 1) + (Tresca_version ? 1 : 0)
+        + (friction_dynamic_term ? 2 : 0);
       GMM_ASSERT1(dl.size() == nbdl, "Wrong number of data for contact brick, "
                   << dl.size() << " should be " << nbdl);
 
@@ -1174,13 +1166,16 @@ namespace getfem {
       basic_asm_real_tangent_terms
         (u1, u2, lambda_n, lambda_t, md.real_variable(dl[np_wt1]),
          md.real_variable(dl[np_wt2]), matl, vecl, version);
+      }
 
     }
 
     Coulomb_friction_brick(int aug_version, bool contact_only_,
                            bool two_variables_=false,
                            bool Tresca_version_=false,
-                           bool Hughes_stabilized_=false) {
+                           bool Hughes_stabilized_=false,
+                           bool friction_dynamic_term_=false) {
+
       if (aug_version == 4 && contact_only_) aug_version = 3;
       augmentation_version = aug_version;
       GMM_ASSERT1(aug_version >= 1 && aug_version <= 4,
@@ -1192,7 +1187,7 @@ namespace getfem {
       is_init = false;
       Tresca_version = Tresca_version_;
       really_stationary = false;   // for future version ...
-      friction_dynamic_term = false;  // for future version ...
+      friction_dynamic_term = friction_dynamic_term_;
       two_variables = two_variables_;
       Hughes_stabilized = Hughes_stabilized_;
       set_flags("Coulomb friction brick", false /* is linear*/,
@@ -1208,6 +1203,12 @@ namespace getfem {
       is_init = false;
     }
 
+    void set_BN2(CONTACT_B_MATRIX &BN2_) {
+      gmm::resize(BN2, gmm::mat_nrows(BN2_), gmm::mat_ncols(BN2_));
+      gmm::copy(BN2_, BN2);
+      is_init = false;
+    }
+
     void set_DN(CONTACT_B_MATRIX &DN_) {
       gmm::resize(DN, gmm::mat_nrows(DN_), gmm::mat_ncols(DN_));
       gmm::copy(DN_, DN);
@@ -1237,6 +1238,15 @@ namespace getfem {
   };
 
 
+  void contact_brick_set_stationary(model &md, size_type indbrick) {
+    pbrick pbr = md.brick_pointer(indbrick);
+    md.touch_brick(indbrick);
+    Coulomb_friction_brick *p = dynamic_cast<Coulomb_friction_brick *>
+      (const_cast<virtual_brick *>(pbr.get()));
+    GMM_ASSERT1(p, "Wrong type of brick");
+    p->set_stationary();
+  }
+
   CONTACT_B_MATRIX &contact_brick_set_BN
   (model &md, size_type indbrick) {
     pbrick pbr = md.brick_pointer(indbrick);
@@ -1288,10 +1298,11 @@ namespace getfem {
    const std::string &dataname_r, CONTACT_B_MATRIX &BN,
    std::string dataname_gap, std::string dataname_alpha,
    int aug_version, bool Hughes_stabilized) {
-    Coulomb_friction_brick *pbr_
-      = new Coulomb_friction_brick(aug_version, true, false,false, Hughes_stabilized);
+
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version, true, false, false, Hughes_stabilized);
+    pbrick pbr(pbr_);
     pbr_->set_BN1(BN);
-    pbrick pbr = pbr_;
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1320,6 +1331,55 @@ namespace getfem {
     return md.add_brick(pbr, vl, dl, tl, model::mimlist(), size_type(-1));
   }
 
+  //=========================================================================
+  //  Add a frictionless contact condition with BN, r, alpha given for
+  //  two deformable bodies
+  //=========================================================================
+
+  size_type add_basic_contact_brick_two_deformable_bodies
+  (model &md, const std::string &varname_u1, const std::string &varname_u2,
+   const std::string &multname_n,
+   const std::string &dataname_r, CONTACT_B_MATRIX &BN1, CONTACT_B_MATRIX &BN2,
+   std::string dataname_gap, std::string dataname_alpha,
+   int aug_version, bool Hughes_stabilized) {
+
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version, true, true, false, Hughes_stabilized);
+    pbrick pbr(pbr_);
+    pbr_->set_BN1(BN1);
+    pbr_->set_BN2(BN2);
+
+    model::termlist tl;
+    tl.push_back(model::term_description(varname_u1, varname_u1, false));
+    tl.push_back(model::term_description(varname_u2, varname_u2, false));
+    tl.push_back(model::term_description(varname_u1, multname_n, false));
+    tl.push_back(model::term_description(multname_n, varname_u1, false));
+    tl.push_back(model::term_description(varname_u2, multname_n, false));
+    tl.push_back(model::term_description(multname_n, varname_u2, false));
+    tl.push_back(model::term_description(multname_n, multname_n, false));
+    model::varnamelist dl(1, dataname_r);
+
+    if (dataname_gap.size() == 0) {
+      dataname_gap = md.new_name("contact_gap_on_" + varname_u1);
+      md.add_initialized_fixed_size_data
+        (dataname_gap, model_real_plain_vector(1, scalar_type(0)));
+    }
+    dl.push_back(dataname_gap);
+
+    if (dataname_alpha.size() == 0) {
+      dataname_alpha = md.new_name("contact_parameter_alpha_on_"+ multname_n);
+      md.add_initialized_fixed_size_data
+        (dataname_alpha, model_real_plain_vector(1, scalar_type(1)));
+    }
+    dl.push_back(dataname_alpha);
+
+    model::varnamelist vl(1, varname_u1);
+    vl.push_back(varname_u2);
+    vl.push_back(multname_n);
+
+    return md.add_brick(pbr, vl, dl, tl, model::mimlist(), size_type(-1));
+  }
+
 
   //=========================================================================
   //  Add a contact with friction condition with BN, r, alpha given.
@@ -1331,13 +1391,17 @@ namespace getfem {
    CONTACT_B_MATRIX &BN, CONTACT_B_MATRIX &BT,
    std::string dataname_friction_coeff,
    std::string dataname_gap, std::string dataname_alpha,
-   int aug_version, bool Tresca_version, std::string dataname_threshold, bool Hughes_stabilized) {
-    Coulomb_friction_brick *pbr_
-      = new Coulomb_friction_brick(aug_version,false, false,
-                                   Tresca_version, Hughes_stabilized);
+   int aug_version, bool Tresca_version, const std::string dataname_threshold,
+   std::string dataname_gamma, std::string dataname_wt, bool Hughes_stabilized) {
+
+    bool dynamic_terms = (dataname_gamma.size() > 0);
+
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version,false, false, Tresca_version, Hughes_stabilized,
+       dynamic_terms);
+    pbrick pbr(pbr_);
     pbr_->set_BN1(BN);
     pbr_->set_BT1(BT);
-    pbrick pbr = pbr_;
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1364,6 +1428,10 @@ namespace getfem {
     }
     dl.push_back(dataname_alpha);
     dl.push_back(dataname_friction_coeff);
+    if (dataname_gamma.size()) {
+      dl.push_back(dataname_gamma);
+      dl.push_back(dataname_wt);
+    }
     if (Tresca_version)
       dl.push_back(dataname_threshold);
 
@@ -1398,6 +1466,7 @@ namespace getfem {
                                         model::real_veclist &,
                                         size_type region,
                                         build_version version) const {
+      if (MPI_IS_MASTER()) {
       GMM_ASSERT1(mims.size() == 1, "This contact brick needs one mesh_im");
       size_type nbvar = 2 + (contact_only ? 0 : 1);
       GMM_ASSERT1(vl.size() == nbvar,
@@ -1458,11 +1527,18 @@ namespace getfem {
           if ((i % (d+1)) == 0) alpha[j++] = MM(id, id) / l;
 
 
-#if GETFEM_HAVE_MUPARSER_MUPARSER_H || GETFEM_HAVE_MUPARSER_H
+        getfem::ga_workspace gw;
+        getfem::ga_function f(gw, obstacle);
 
+        size_type N = d+1;
+        getfem::model_real_plain_vector pt(N);
+        gw.add_fixed_size_constant("X", pt);
+        if (N >= 1) gw.add_macro("x", "X(1)");
+        if (N >= 2) gw.add_macro("y", "X(2)");
+        if (N >= 3) gw.add_macro("z", "X(3)");
+        if (N >= 4) gw.add_macro("w", "X(4)");
 
-        mu::Parser parser;
-        parser.SetExpr(obstacle);
+        f.compile();
 
         gmm::resize(gap, nbc);
         gmm::resize(BN1, nbc, mf_u1.nb_dof());
@@ -1471,53 +1547,39 @@ namespace getfem {
           gmm::resize(BT1, d*nbc, mf_u1.nb_dof());
           gmm::clear(BT1);
         }
-        base_node pt(d+1), grad(d+1), ut[3];
-
-        static std::string varn[4] = {"x", "y", "z", "w"};
-        for (size_type k = 0; k <= d; ++k)
-          parser.DefineVar(varn[k], &pt[k]);
+        base_node grad(d+1), ut[3];
 
         i = 0; j = 0;
         for (dal::bv_visitor id(dofs); !id.finished(); ++id, ++i) {
           if ((i % (d+1)) == 0) {
             gmm::copy(mf_u1.point_of_basic_dof(id), pt);
-            try {
 
-              // Computation of gap
-              gap[j] = scalar_type(parser.Eval());
+            // Computation of gap
+            gap[j] = (f.eval())[0];
 
-              // computation of BN
-              size_type cv = mf_u1.first_convex_of_basic_dof(id);
-              scalar_type eps
-                = mf_u1.linked_mesh().convex_radius_estimate(cv) * 1E-3;
-              for (size_type k = 0; k <= d; ++k) {
-                pt[k] += eps;
-                grad[k] = (scalar_type(parser.Eval()) - gap[j]) / eps;
-                pt[k] -= eps;
-              }
-              // unit normal vector
-              base_node un = - grad / gmm::vect_norm2(grad);
+            // computation of BN
+            size_type cv = mf_u1.first_convex_of_basic_dof(id);
+            scalar_type eps
+              = mf_u1.linked_mesh().convex_radius_estimate(cv) * 1E-3;
+            for (size_type k = 0; k <= d; ++k) {
+              pt[k] += eps;
+              grad[k] = ((f.eval())[0] - gap[j]) / eps;
+              pt[k] -= eps;
+            }
+            // unit normal vector
+            base_node un = - grad / gmm::vect_norm2(grad);
+
+            for (size_type k = 0; k <= d; ++k)
+              BN1(j, id + k) = un[k];
+
+            // computation of BT
+            if (!contact_only) {
+
+              orthonormal_basis_to_unit_vec(d, un, ut);
 
               for (size_type k = 0; k <= d; ++k)
-                BN1(j, id + k) = un[k];
-
-              // computation of BT
-              if (!contact_only) {
-
-                orthonormal_basis_to_unit_vec(d, un, ut);
-
-                for (size_type k = 0; k <= d; ++k)
-                  for (size_type nn = 0; nn < d; ++nn)
-                    BT1(j*d+nn, id + k) = ut[nn][k];
-              }
-
-            } catch (mu::Parser::exception_type &e) {
-              std::cerr << "Message  : " << e.GetMsg()   << std::endl;
-              std::cerr << "Formula  : " << e.GetExpr()  << std::endl;
-              std::cerr << "Token    : " << e.GetToken() << std::endl;
-              std::cerr << "Position : " << e.GetPos()   << std::endl;
-              std::cerr << "Errc     : " << e.GetCode()  << std::endl;
-              GMM_ASSERT1(false, "Error in signed distance expression");
+                for (size_type nn = 0; nn < d; ++nn)
+                  BT1(j*d+nn, id + k) = ut[nn][k];
             }
             ++j;
           }
@@ -1531,13 +1593,6 @@ namespace getfem {
           GMM_ASSERT1(gmm::vect_size(md.real_variable(vl[2])) == nbc*d,
                       "Wrong size of multiplier for the friction condition");
 
-#else
-
-        GMM_ASSERT1(false, "Muparser is not installed, "
-                    "You cannot use this contact brick");
-
-#endif
-
         is_init = false;
       }
       else
@@ -1574,6 +1629,7 @@ namespace getfem {
       basic_asm_real_tangent_terms
         (u1, u1, lambda_n, lambda_t, md.real_variable(dl[np_wt1]),
          md.real_variable(dl[np_wt1]), matl, vecl, version);
+      }
 
     }
 
@@ -1593,8 +1649,8 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname_u,
    const std::string &multname_n, const std::string &dataname_r,
    size_type region, const std::string &obstacle, int aug_version) {
-    pbrick pbr
-      = new Coulomb_friction_brick_rigid_obstacle(aug_version, true, obstacle);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_rigid_obstacle>
+      (aug_version, true, obstacle);
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1620,8 +1676,8 @@ namespace getfem {
    const std::string &multname_n, const std::string &multname_t,
    const std::string &dataname_r, const std::string &dataname_friction_coeff,
    size_type region, const std::string &obstacle, int aug_version) {
-    pbrick pbr
-      = new Coulomb_friction_brick_rigid_obstacle(aug_version,false,obstacle);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_rigid_obstacle>
+      (aug_version, false, obstacle);
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1673,7 +1729,7 @@ namespace getfem {
                                         model::real_veclist &,
                                         size_type region,
                                         build_version version) const {
-
+      if (MPI_IS_MASTER()) {
       GMM_ASSERT1(mims.size() == 2, "This contact brick needs two mesh_im");
       const mesh_im &mim1 = *mims[0];
       const mesh_im &mim2 = *mims[1];
@@ -1788,6 +1844,7 @@ namespace getfem {
       const model_real_plain_vector dummy_wt;
       basic_asm_real_tangent_terms
         (u1, u2, lambda_n, lambda_t, dummy_wt, dummy_wt, matl, vecl, version);
+      }
     }
 
     Coulomb_friction_brick_nonmatching_meshes
@@ -1814,8 +1871,8 @@ namespace getfem {
 
     bool two_variables = (varname_u1.compare(varname_u2) != 0);
 
-    pbrick pbr = new Coulomb_friction_brick_nonmatching_meshes
-           (aug_version, true, two_variables, rg1, rg2, slave1, slave2);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_nonmatching_meshes>
+      (aug_version, true, two_variables, rg1, rg2, slave1, slave2);
 
     // Calculate multipliers size
     const mesh_fem &mf_u1 = md.mesh_fem_of_variable(varname_u1);
@@ -1836,7 +1893,7 @@ namespace getfem {
       multname_n = md.new_name("contact_multiplier");
     else
       GMM_ASSERT1(multname_n.compare(md.new_name(multname_n)) == 0,
-                  "The given name for the multiplier is alraedy reserved in the model");
+                  "The given name for the multiplier is already reserved in the model");
     md.add_fixed_size_variable(multname_n, nbc);
 
     model::termlist tl;
@@ -1885,8 +1942,8 @@ namespace getfem {
 
     bool two_variables = (varname_u1.compare(varname_u2) != 0);
 
-    pbrick pbr = new Coulomb_friction_brick_nonmatching_meshes
-           (aug_version, false, two_variables, rg1, rg2, slave1, slave2);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_nonmatching_meshes>
+	       (aug_version, false, two_variables, rg1, rg2, slave1, slave2);
 
     // Calculate multipliers size
     const mesh_fem &mf_u1 = md.mesh_fem_of_variable(varname_u1);
@@ -1907,13 +1964,13 @@ namespace getfem {
       multname_n = md.new_name("contact_normal_multiplier");
     else
       GMM_ASSERT1(multname_n.compare(md.new_name(multname_n)) == 0,
-                  "The given name for the multiplier is alraedy reserved in the model");
+                  "The given name for the multiplier is already reserved in the model");
     md.add_fixed_size_variable(multname_n, nbc);
     if (multname_t.size() == 0)
       multname_t = md.new_name("contact_tangent_multiplier");
     else
       GMM_ASSERT1(multname_t.compare(md.new_name(multname_t)) == 0,
-                  "The given name for the multiplier is alraedy reserved in the model");
+                  "The given name for the multiplier is already reserved in the model");
     md.add_fixed_size_variable(multname_t, nbc * (mf_u1.get_qdim() - 1) ); // ??
 
     model::termlist tl;
