@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2000-2015 Julien Pommier
+ Copyright (C) 2000-2016 Julien Pommier
 
  This file is a part of GetFEM++
 
@@ -91,6 +91,9 @@ namespace getfem {
       case 23: { /* 4TH ORDER TRIANGLE */
         pgt = bgeot::simplex_geotrans(2, 4);
       } break;
+      case 27: { /* 4TH ORDER LINE */
+        pgt = bgeot::simplex_geotrans(1, 4);
+      } break;
       default: { /* UNKNOWN .. */
         /* higher order elements : to be done .. */
         GMM_ASSERT1(false, "gmsh element type " << type << " is unknown.");
@@ -156,6 +159,9 @@ namespace getfem {
       case 23: { /* 4TH ORDER TRIANGLE */
         nodes.resize(15);
       } break;
+      case 27: { /* 4TH ORDER LINE */
+        nodes.resize(5);
+      } break;
       default: { /* UNKNOWN .. */
         /* higher order elements : to be done .. */
         GMM_ASSERT1(false, "the gmsh element type " << type << " is unknown..");
@@ -166,10 +172,35 @@ namespace getfem {
     bool operator<(const gmsh_cv_info& other) const {
       unsigned this_dim = (type == 15) ? 0 : pgt->dim();
       unsigned other_dim = (other.type == 15) ? 0 : other.pgt->dim();
+      if (this_dim == other_dim) return region < other.region;
       return this_dim > other_dim;
     }
   };
 
+  std::map<std::string, size_type> read_region_names_from_gmsh_mesh_file(std::istream& f)
+  {
+    std::map<std::string, size_type> region_map;
+    bgeot::read_until(f, "$PhysicalNames");
+    size_type nb_regions;
+    f >> nb_regions;
+    size_type rt,ri;
+    std::string region_name;
+    for (size_type region_cnt=0; region_cnt < nb_regions; ++ region_cnt) {
+      f >> rt >> ri;
+      std::getline(f, region_name);
+      /* trim the string to the quote character front and back*/
+      size_t pos = region_name.find_first_of("\"");
+      if (pos != region_name.npos) {
+        region_name.erase(0, pos+1);
+        pos = region_name.find_last_of("\"");
+        region_name.erase(pos);
+      }
+      region_map[region_name] = ri;
+    }
+
+    return region_map;
+  }
+
   /*
      Format version 1 [for gmsh version < 2.0].
      structure: $NOD list_of_nodes $ENDNOD $ELT list_of_elt $ENDELT
@@ -180,8 +211,8 @@ namespace getfem {
      structure: $Nodes list_of_nodes $EndNodes $Elements list_of_elt
      $EndElements
 
-     Faces that falls in the range of face_region_range will be imported as
-     an independant convex.
+     Lower dimensions elements in the regions of lower_dim_convex_rg will
+     be imported as independant convexes.
 
      If add_all_element_type is set to true, convexes with less dimension
      than highest dimension pgt and are not part of other element's face will
@@ -192,9 +223,11 @@ namespace getfem {
   */
   static void import_gmsh_mesh_file(std::istream& f, mesh& m, int deprecate=0,
                                     std::map<std::string, size_type> *region_map=NULL,
-                                    std::pair<size_type, size_type> *face_region_range=NULL,
+                                    std::set<size_type> *lower_dim_convex_rg=NULL,
                                     bool add_all_element_type = false,
-                                    bool remove_last_dimension = true)
+                                    bool remove_last_dimension = true,
+                                    std::map<size_type, std::set<size_type>> *nodal_map = NULL,
+                                    bool remove_duplicated_nodes = true)
   {
     gmm::stream_standard_locale sl(f);
     /* print general warning */
@@ -227,23 +260,7 @@ namespace getfem {
     /* read the region names */
     if (region_map != NULL) {
       if (version == 2) {
-        bgeot::read_until(f, "$PhysicalNames");
-        size_type nb_regions;
-        f >> nb_regions;
-        size_type rt,ri;
-        std::string region_name;
-        for (size_type region_cnt=0; region_cnt < nb_regions; ++ region_cnt) {
-          f >> rt >> ri;
-          std::getline(f, region_name);
-          /* trim the string to the quote character front and back*/
-          size_t pos = region_name.find_first_of("\"");
-          if (pos != region_name.npos) {
-            region_name.erase(0, pos+1);
-            pos = region_name.find_last_of("\"");
-            region_name.erase(pos);
-          }
-          (*region_map)[region_name] = ri;
-        }
+        *region_map = read_region_names_from_gmsh_mesh_file(f);
       }
     }
     /* read the node list */
@@ -258,7 +275,7 @@ namespace getfem {
       size_type node_id;
       base_node n(3); n[0]=n[1]=n[2]=0.0;
       f >> node_id >> n[0] >> n[1] >> n[2];
-      msh_node_2_getfem_node[node_id] = m.add_point(n);
+      msh_node_2_getfem_node[node_id] = m.add_point(n, 0.0, remove_duplicated_nodes);
     }
 
     if (version == 2)
@@ -470,6 +487,13 @@ namespace getfem {
         ci.nodes[13] = tmp_nodes[8];
         ci.nodes[14] = tmp_nodes[2];
       } break;
+      case 27: { /* Fourth order line */
+      //ci.nodes[0]  = tmp_nodes[0];
+        ci.nodes[1]  = tmp_nodes[2];
+        ci.nodes[2]  = tmp_nodes[3];
+        ci.nodes[3]  = tmp_nodes[4];
+        ci.nodes[4]  = tmp_nodes[1];
+      } break;
       }
     }
     nb_cv = cvlst.size();
@@ -498,11 +522,11 @@ namespace getfem {
         //convexes with lower dimensions
         }
         else {
-          //convex that lies within the range of face_region_range
+          //convex that lies within the regions of lower_dim_convex_rg
           //is imported explicitly as a convex.
-          if (face_region_range != NULL &&
-            ci.region >= face_region_range->first &&
-            ci.region <= face_region_range->second && !is_node){
+          if (lower_dim_convex_rg != NULL &&
+              lower_dim_convex_rg->find(ci.region) != lower_dim_convex_rg->end() &&
+              !is_node){
               size_type ic = m.add_convex(ci.pgt, ci.nodes.begin()); cvok = true;
               m.region(ci.region).add(ic);
           }
@@ -521,14 +545,17 @@ namespace getfem {
                 }
               }
             }
+            if (is_node && (nodal_map != NULL)) (*nodal_map)[ci.region].insert(ci.id);
             //if the convex is not part of the face of others
             if (!cvok)
             {
               if (is_node)
               {
-                GMM_WARNING2("gmsh import ignored a node id: "
-                             << ci.id << " region :" << ci.region <<
-                             " point is not added explicitly as an element.");
+                if (nodal_map == NULL){
+                  GMM_WARNING2("gmsh import ignored a node id: "
+                               << ci.id << " region :" << ci.region <<
+                               " point is not added explicitly as an element.");
+                }
               }
               else if (add_all_element_type){
                 size_type ic = m.add_convex(ci.pgt, ci.nodes.begin());
@@ -536,7 +563,7 @@ namespace getfem {
                 cvok = true;
               }
               else{
-                GMM_WARNING2("gmsh import ignored a element of type "
+                GMM_WARNING2("gmsh import ignored an element of type "
                   << bgeot::name_of_geometric_trans(ci.pgt) <<
                   " as it does not belong to the face of another element");
               }
@@ -1223,10 +1250,10 @@ namespace getfem {
       GMM_ASSERT1(f.good(), "can't open file " << filename);
       /* throw exceptions when an error occurs */
       f.exceptions(std::ifstream::badbit | std::ifstream::failbit);
-      import_mesh(f,format,m);
+      import_mesh(f, format,m);
       f.close();
     }
-    catch (failure_error& exc) {
+    catch (std::logic_error& exc) {
       m.clear();
       throw exc;
     }
@@ -1235,29 +1262,41 @@ namespace getfem {
       GMM_ASSERT1(false, "error while importing " << format
                   << " mesh file \"" << filename << "\" : " << exc.what());
     }
+    catch (std::runtime_error& exc) {
+      m.clear();
+      throw exc;
+    }
   }
 
   void import_mesh_gmsh(std::istream& f, mesh &m,
-                  std::map<std::string, size_type> &region_map,
-                   bool remove_last_dimension)
+                        std::map<std::string, size_type> &region_map,
+                        bool remove_last_dimension,
+                        std::map<size_type, std::set<size_type>> *nodal_map,
+                        bool remove_duplicated_nodes)
   {
-    import_gmsh_mesh_file(f, m, 0, &region_map, nullptr, false, remove_last_dimension);
+    import_gmsh_mesh_file(f, m, 0, &region_map, nullptr, false, remove_last_dimension, nodal_map,
+                          remove_duplicated_nodes);
   }
 
   void import_mesh_gmsh(std::istream& f, mesh& m,
                         bool add_all_element_type,
-                        std::pair<size_type, size_type> *face_region_range,
+                        std::set<size_type> *lower_dim_convex_rg,
                         std::map<std::string, size_type> *region_map,
-                   bool remove_last_dimension)
+                        bool remove_last_dimension,
+                        std::map<size_type, std::set<size_type>> *nodal_map,
+                        bool remove_duplicated_nodes)
   {
-    import_gmsh_mesh_file(f, m, 0, region_map, face_region_range, add_all_element_type, remove_last_dimension);
+    import_gmsh_mesh_file(f, m, 0, region_map, lower_dim_convex_rg, add_all_element_type,
+                          remove_last_dimension, nodal_map, remove_duplicated_nodes);
   }
 
   void import_mesh_gmsh(const std::string& filename, mesh& m,
                         bool add_all_element_type,
-                        std::pair<size_type, size_type> *face_region_range,
+                        std::set<size_type> *lower_dim_convex_rg,
                         std::map<std::string, size_type> *region_map,
-                        bool remove_last_dimension)
+                        bool remove_last_dimension,
+                        std::map<size_type, std::set<size_type>> *nodal_map,
+                        bool remove_duplicated_nodes)
   {
     m.clear();
     try {
@@ -1265,10 +1304,11 @@ namespace getfem {
       GMM_ASSERT1(f.good(), "can't open file " << filename);
       /* throw exceptions when an error occurs */
       f.exceptions(std::ifstream::badbit | std::ifstream::failbit);
-      import_gmsh_mesh_file(f, m, 0, region_map, face_region_range, add_all_element_type, remove_last_dimension);
+      import_gmsh_mesh_file(f, m, 0, region_map, lower_dim_convex_rg, add_all_element_type,
+                            remove_last_dimension, nodal_map, remove_duplicated_nodes);
       f.close();
     }
-    catch (failure_error& exc) {
+    catch (std::logic_error& exc) {
       m.clear();
       throw exc;
     }
@@ -1277,12 +1317,19 @@ namespace getfem {
       GMM_ASSERT1(false, "error while importing " << "gmsh"
                   << " mesh file \"" << filename << "\" : " << exc.what());
     }
+    catch (std::runtime_error& exc) {
+      m.clear();
+      throw exc;
+    }
   }
 
   void import_mesh_gmsh(const std::string& filename,
     mesh& m, std::map<std::string, size_type> &region_map,
-    bool remove_last_dimension) {
-    import_mesh_gmsh(filename, m, false, NULL, &region_map, remove_last_dimension);
+    bool remove_last_dimension,
+    std::map<size_type, std::set<size_type>> *nodal_map,
+    bool remove_duplicated_nodes) {
+    import_mesh_gmsh(filename, m, false, NULL, &region_map, remove_last_dimension, nodal_map,
+                     remove_duplicated_nodes);
   }
 
   void import_mesh(std::istream& f, const std::string& format,
