@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2000-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2000-2016 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,29 +16,29 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 
 #include "getfem/dal_singleton.h"
 #include "getfem/getfem_integration.h"
-#include "getfem/dal_naming_system.h"
 #include "gmm/gmm_dense_lu.h"
 #include "getfem/bgeot_permutations.h"
 #include "getfem/bgeot_geotrans_inv.h"
 #include "getfem/getfem_im_list.h"
+#include "getfem/dal_naming_system.h"
+#include "getfem/getfem_omp.h"
+#include "getfem/getfem_torus.h"
 
 namespace getfem {
 
-  typedef dal::naming_system<integration_method>::param_list im_param_list;
-
   /*
    * dummy integration method 
    */
   static pintegration_method im_none(im_param_list &params,
 			       std::vector<dal::pstatic_stored_object> &) {
     GMM_ASSERT1(params.size() == 0, "IM_NONE does not accept any parameter");
-    return new integration_method();
+    return std::make_shared<integration_method>();
   }
 
   long_scalar_type poly_integration::int_poly(const base_poly &P) const {
@@ -134,8 +134,9 @@ namespace getfem {
     GMM_ASSERT1(n > 0 && n < 100 && double(n) == params[0].num(),
 		"Bad parameters");
     dependencies.push_back(bgeot::simplex_structure(dim_type(n)));
-    return new integration_method
-      (new simplex_poly_integration_(bgeot::simplex_structure(dim_type(n))));
+    ppoly_integration ppi = std::make_shared<simplex_poly_integration_>
+      (bgeot::simplex_structure(dim_type(n)));
+    return std::make_shared<integration_method>(ppi);
   }
 
   /* ******************************************************************** */
@@ -195,8 +196,9 @@ namespace getfem {
     dependencies.push_back(a); dependencies.push_back(b);
     dependencies.push_back(bgeot::convex_product_structure(a->structure(),
 							   b->structure()));
-    return new integration_method(new plyint_mul_structure_(a->exact_method(),
-							  b->exact_method()));
+    ppoly_integration ppi = std::make_shared<plyint_mul_structure_>
+      (a->exact_method(), b->exact_method());
+    return  std::make_shared<integration_method>(ppi);
   }
 
   /* ******************************************************************** */
@@ -336,7 +338,7 @@ namespace getfem {
     for (size_type i = 0; i < pai->nb_points_on_convex(); ++i) {
       pt = (cvr->dir_points_of_face(f))[0];
       for (dim_type j = 0; j < N; ++j)
-	pt += pts[j] * (pai->integration_points()[i])[j];
+	pt += pts[j] * ((*(pai->pintegration_points()))[i])[j];
       add_point(pt, pai->coeff(i) * det, f);
     }
   }
@@ -374,7 +376,7 @@ namespace getfem {
 	  = bgeot::geometric_trans_descriptor(im_desc_tab[i].geotrans_name);
 	dim_type N = pgt->structure()->dim();
 	base_node pt(N);
-	approx_integration *pai = new approx_integration(pgt->convex_ref());
+	auto pai = std::make_shared<approx_integration>(pgt->convex_ref());
 	size_type fr = im_desc_tab[i].firstreal;
 	for (size_type j = 0; j < im_desc_tab[i].nb_points; ++j) {
 	  for (dim_type k = 0; k < N; ++k)
@@ -413,12 +415,12 @@ namespace getfem {
 	pai->valid_method();
         // cerr << "finding " << name << endl;
 
-	integration_method *p = new integration_method(pai);
+	pintegration_method p(std::make_shared<integration_method>(pai));
 	dependencies.push_back(p->approx_method()->ref_convex());
-	dependencies.push_back(&(p->approx_method()->integration_points()));
+	dependencies.push_back(p->approx_method()->pintegration_points());
 	return p;
       }
-    return 0;
+    return pintegration_method();
   }
 
 
@@ -427,11 +429,13 @@ namespace getfem {
   /* ********************************************************************* */
 
   struct Legendre_polynomials {
-    dal::dynamic_array<base_poly> polynomials;
-    dal::dynamic_array< std::vector<long_scalar_type> > roots;
+    std::vector<base_poly> polynomials;
+    std::vector<std::vector<long_scalar_type>> roots;
     int nb_lp;
     Legendre_polynomials() : nb_lp(-1) {}
     void init(short_type de) {
+      polynomials.resize(de+2);
+      roots.resize(de+2);
       if (nb_lp < 0) {
         polynomials[0] = base_poly(1,0);
         polynomials[0].one();
@@ -489,16 +493,16 @@ namespace getfem {
     repartition[1] = nbpt + 1;
     repartition[2] = nbpt + 2; 
     
-    Legendre_polynomials &lp = dal::singleton<Legendre_polynomials>::instance();
-    lp.init(nbpt);
+    Legendre_polynomials Lp;
+    Lp.init(nbpt);
     
     for (short_type i = 0; i < nbpt; ++i) {
       int_points[i].resize(1);
-      long_scalar_type lr = lp.roots[nbpt][i];
-      int_points[i][0] = 0.5 + 0.5 * lr;
-      int_coeffs[i] = (1.0 - gmm::sqr(lr))
+      long_scalar_type lr = Lp.roots[nbpt][i];
+      int_points[i][0] = 0.5 + 0.5 * bgeot::to_scalar(lr);
+      int_coeffs[i] = bgeot::to_scalar((1.0 - gmm::sqr(lr))
 	/ gmm::sqr( long_scalar_type(nbpt)
-		    * (lp.polynomials[nbpt-1].eval(&lr)));
+		    * (Lp.polynomials[nbpt-1].eval(&lr))));
     }
     
     int_points[nbpt].resize(1);
@@ -525,10 +529,11 @@ namespace getfem {
       return int_method_descriptor(name.str());
     }
     else {
-      integration_method *p
-	= new integration_method(new gauss_approx_integration_(short_type(n/2 + 1)));
+      papprox_integration
+	pai = std::make_shared<gauss_approx_integration_>(short_type(n/2 + 1));
+      pintegration_method p = std::make_shared<integration_method>(pai);
       dependencies.push_back(p->approx_method()->ref_convex());
-      dependencies.push_back(&(p->approx_method()->integration_points()));
+      dependencies.push_back(p->approx_method()->pintegration_points());
       return p;
     }
   }
@@ -550,7 +555,7 @@ namespace getfem {
 
     base_node c(nc); 
     if (nc == 0) {
-      add_point(c, LONG_SCAL(1));
+      add_point(c, scalar_type(1));
     }
     else {
       
@@ -615,7 +620,7 @@ namespace getfem {
       
       gmm::lu_solve(M, U, F);
       for (size_type r = 0; r < R; ++r)
-	add_point(nodes[r], U[r]);
+	add_point(nodes[r], bgeot::to_scalar(U[r]));
       
       std::stringstream name2;
       name2 << "IM_NC(" << int(nc-1) << "," << int(k) << ")";
@@ -636,10 +641,12 @@ namespace getfem {
     GMM_ASSERT1(n >= 0 && n < 100 && k >= 0 && k <= 150 &&
 		double(n) == params[0].num() && double(k) == params[1].num(),
 		"Bad parameters");
-    integration_method *p
-      = new integration_method(new Newton_Cotes_approx_integration_(dim_type(n), short_type(k)));
+    papprox_integration
+      pai = std::make_shared<Newton_Cotes_approx_integration_>(dim_type(n),
+							       short_type(k));
+    pintegration_method p = std::make_shared<integration_method>(pai);
     dependencies.push_back(p->approx_method()->ref_convex());
-    dependencies.push_back(&(p->approx_method()->integration_points()));
+    dependencies.push_back(p->approx_method()->pintegration_points());
     return p;
   }
 
@@ -725,11 +732,12 @@ namespace getfem {
     pintegration_method b = params[1].method();
     GMM_ASSERT1(a->type() == IM_APPROX && b->type() == IM_APPROX,
 		"Bad parameters");
-    integration_method *p
-      = new integration_method(new a_int_pro_integration(a->approx_method(),
-							 b->approx_method()));
+    papprox_integration
+      pai = std::make_shared<a_int_pro_integration>(a->approx_method(),
+						    b->approx_method());
+    pintegration_method p = std::make_shared<integration_method>(pai);
     dependencies.push_back(p->approx_method()->ref_convex());
-    dependencies.push_back(&(p->approx_method()->integration_points()));
+    dependencies.push_back(p->approx_method()->pintegration_points());
     return p;
   }
 
@@ -1002,11 +1010,13 @@ namespace getfem {
     int N = a->approx_method()->dim();
     GMM_ASSERT1(N >= 2 && N <= 3 && ip1 >= 0 && ip2 >= 0 && ip1 <= N
 		&& ip2 <= N, "Bad parameters");
-    integration_method *p
-      = new integration_method(new quasi_polar_integration(a->approx_method(),
-							   ip1, ip2));
+
+    papprox_integration
+      pai = std::make_shared<quasi_polar_integration>(a->approx_method(),
+						      ip1, ip2);
+    pintegration_method p = std::make_shared<integration_method>(pai);
     dependencies.push_back(p->approx_method()->ref_convex());
-    dependencies.push_back(&(p->approx_method()->integration_points()));
+    dependencies.push_back(p->approx_method()->pintegration_points());
     return p;
   }
 
@@ -1054,15 +1064,22 @@ namespace getfem {
   }
 
   std::string name_of_int_method(pintegration_method p) {
-    if (!p) return "IM_NONE";
+    if (!(p.get())) return "IM_NONE";
     return dal::singleton<im_naming_system>::instance().shorter_name_of_method(p);
   }
 
+  // allows the add of an integration method.
+  void add_integration_name(std::string name,
+			dal::naming_system<integration_method>::pfunction f) {
+    dal::singleton<im_naming_system>::instance().add_suffix(name, f);
+  }
+
+
   /* Fonctions pour la ref. directe.                                     */
   
   pintegration_method exact_simplex_im(size_type n) {
-    static pintegration_method pim = 0;
-    static size_type d = size_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method, pim, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, -2);
     if (d != n) {
       std::stringstream name;
       name << "IM_EXACT_SIMPLEX(" << n << ")";
@@ -1073,8 +1090,8 @@ namespace getfem {
   }
 
   pintegration_method exact_parallelepiped_im(size_type n) {
-    static pintegration_method pim = 0;
-    static size_type d = size_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method, pim, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, -2);
     if (d != n) {
       std::stringstream name;
       name << "IM_EXACT_PARALLELEPIPED(" << n << ")";
@@ -1085,8 +1102,8 @@ namespace getfem {
   }
 
   pintegration_method exact_prism_im(size_type n) {
-    static pintegration_method pim = 0;
-    static size_type d = size_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method, pim, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, -2);
     if (d != n) {
       std::stringstream name;
       name << "IM_EXACT_PRISM(" << n << ")";
@@ -1101,9 +1118,9 @@ namespace getfem {
   }
 
   static pintegration_method classical_exact_im(bgeot::pconvex_structure cvs) {
-    cvs = cvs->basic_structure();
-    static bgeot::pconvex_structure cvs_last = 0;
-    static pintegration_method im_last = 0;
+    cvs = bgeot::basic_structure(cvs);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pconvex_structure,cvs_last, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method, im_last, 0);
     bool found = false;
 
     if (cvs_last == cvs)
@@ -1153,9 +1170,11 @@ namespace getfem {
     size_type n = cvs->dim();
     std::stringstream name;
 
+    if(bgeot::is_torus_structure(cvs) && n == 3) n = 2;
+
     degree = std::max<dim_type>(degree, 1);
     bgeot::pconvex_structure a, b;
-    if (cvs->basic_structure() == bgeot::simplex_structure(dim_type(n))) {
+    if (bgeot::basic_structure(cvs) == bgeot::simplex_structure(dim_type(n))) {
       /* Identifying P1-simplexes. */
       switch (n) {
       case 0: return int_method_descriptor("IM_NC(0,0)");
@@ -1164,29 +1183,30 @@ namespace getfem {
       case 3: name << "IM_TETRAHEDRON"; break;
       case 4: name << "IM_SIMPLEX4D"; break;
       default: GMM_ASSERT1(false, "no approximate integration method "
-			   "for simplexes of dimension " << n);
+                           "for simplexes of dimension " << n);
       }
       for (size_type k = degree; k < size_type(degree+10); ++k) {
-	pintegration_method im = 0;
-	std::stringstream name2; name2 << name.str() << "(" << k << ")";
-	im = int_method_descriptor(name2.str(), false);
-	if (im) return im;
+        pintegration_method im = 0;
+        std::stringstream name2; name2 << name.str() << "(" << k << ")";
+        im = int_method_descriptor(name2.str(), false);
+        if (im) return im;
       }
       GMM_ASSERT1(false, "could not find an " << name.str()
-		  << " of degree >= " << int(degree));
-    } else if (cvs->is_product(&a,&b)) {
+                  << " of degree >= " << int(degree));
+    } else if (cvs->is_product(&a,&b) ||
+               (bgeot::basic_structure(cvs).get() && bgeot::basic_structure(cvs)->is_product(&a,&b))) {
       name << "IM_PRODUCT(" 
-	   << name_of_int_method(classical_approx_im_(a,degree)) << ","
-	   << name_of_int_method(classical_approx_im_(b,degree)) << ")";
+           << name_of_int_method(classical_approx_im_(a,degree)) << ","
+           << name_of_int_method(classical_approx_im_(b,degree)) << ")";
     } else GMM_ASSERT1(false, "unknown convex structure!");
     return int_method_descriptor(name.str());
   }
 
   pintegration_method classical_approx_im(bgeot::pgeometric_trans pgt,
 					  dim_type degree) {
-    static bgeot::pgeometric_trans pgt_last = 0;
-    static dim_type degree_last;
-    static pintegration_method im_last = 0;
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_last, 0);
+    DEFINE_STATIC_THREAD_LOCAL(dim_type, degree_last);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method, im_last, 0);
     if (pgt_last == pgt && degree == degree_last)
       return im_last;
     im_last = classical_approx_im_(pgt->structure(),degree);
@@ -1196,7 +1216,7 @@ namespace getfem {
   }
 
   pintegration_method im_none(void) { 
-     static pintegration_method im_last = 0;
+     DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pintegration_method,im_last,0);
      if (!im_last) im_last = int_method_descriptor("IM_NONE");
      return im_last;
   }
@@ -1218,7 +1238,7 @@ namespace getfem {
       realsum = exact->exact_method()->int_monomial(idx);
       error = std::max(error, gmm::abs(realsum-sum));
     }
-    return error;
+    return bgeot::to_scalar(error);
   }
 
   papprox_integration get_approx_im_or_fail(pintegration_method pim) {
