@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2012-2015 Andriy Andreykiv
+ Copyright (C) 2012-2016 Andriy Andreykiv
 
  This file is a part of GetFEM++
 
@@ -132,8 +132,7 @@ const level_set_contact::contact_pair_info&
 	level_set_contact::master_contact_body::get_pair_info(
 	const std::string& slave_var_name) const
 {   
-	std::map<std::string, dal::shared_ptr<contact_pair_info> >
-		::const_iterator it = contact_table.find(slave_var_name);
+        auto it = contact_table.find(slave_var_name);
 	if (it!=contact_table.end()) return *(it->second);
 	GMM_ASSERT1(false,"did not find info on slave contact body, \
 					  defined on variable "+slave_var_name);
@@ -143,8 +142,7 @@ level_set_contact::contact_pair_info&
 	level_set_contact::master_contact_body::get_pair_info(
 	const std::string& slave_var_name)
 {   
-	std::map<std::string, dal::shared_ptr<contact_pair_info> >
-		::iterator it = contact_table.find(slave_var_name);
+	auto it = contact_table.find(slave_var_name);
 	if (it!=contact_table.end()) return *(it->second);
 	GMM_ASSERT1(false,"did not find info on slave contact body, \
 					  defined on variable "+slave_var_name);
@@ -154,8 +152,8 @@ level_set_contact::contact_pair_info&
 level_set_contact::face_type level_set_contact::master_contact_body::
 	ext_face_of_elem(size_type i) const
 {
-	std::map<size_type,face_type>::const_iterator it = border_faces.find(i);
-	if(it!=border_faces.end()) return it->second;
+	auto it = border_faces.find(i);
+	if (it!=border_faces.end()) return it->second;
 	GMM_ASSERT1(false,"did not find a face, corresponding to element "<<i);
 }
 
@@ -174,19 +172,19 @@ void level_set_contact::master_contact_body::
 	size_type assumed_contact_elems = getfem::mesh_region::free_region_id(get_mesh());
 	getfem::mesh_region& contact_elems = get_mesh().region(assumed_contact_elems);
 	getfem::mesh_region& boundary_elems = get_mesh().region(BOUNDARY_ELEMENTS);
-	dal::shared_ptr<getfem::mr_visitor> i;
+	std::shared_ptr<getfem::mr_visitor> i;
 	getfem::mesh_region outer_faces;
 	outer_faces.clear();
 	getfem::outer_faces_of_mesh(get_mesh(), outer_faces);
 
 	if (assumed_contact_region==size_type(-1)){ //all faces will be searched for contact
-		i.reset(new getfem::mr_visitor(outer_faces));
+	  i = std::make_shared<getfem::mr_visitor>(outer_faces);
 	}
 	else // only specified faces will be searched
 	{  
 		getfem::mesh_region& assumed_region = get_mesh().
 			region(assumed_contact_region);
-		i.reset(new getfem::mr_visitor(assumed_region));
+		i = std::make_shared<getfem::mr_visitor>(assumed_region);
 	}
 
 	for (; !i->finished(); ++(*i)){
@@ -226,8 +224,7 @@ void level_set_contact::master_contact_body::
 
 	//register contact pair
 	contact_table[scb.get_var_name()] = 
-		dal::shared_ptr<contact_pair_info>
-		(new contact_pair_info(*this,scb,mult_name,assumed_contact_elems)); 
+	  std::make_shared<contact_pair_info>(*this,scb,mult_name,assumed_contact_elems);
 
 }
 
@@ -257,7 +254,7 @@ void level_set_contact::master_contact_body::clear_all_contact_history()
 
 void level_set_contact::master_contact_body::clear_contact_history()
 {
-	std::map<std::string, dal::shared_ptr<contact_pair_info> >::
+	std::map<std::string, std::shared_ptr<contact_pair_info> >::
 		iterator it = contact_table.begin();
 	for(;it!=contact_table.end();it++)
 		it->second->clear_contact_history();
@@ -266,7 +263,7 @@ void level_set_contact::master_contact_body::clear_contact_history()
 bool level_set_contact::master_contact_body::master_contact_changed()
 {
 	bool contact_surfaces_changed = false;
-	std::map<std::string, dal::shared_ptr<contact_pair_info> >::
+	std::map<std::string, std::shared_ptr<contact_pair_info> >::
 		iterator it = contact_table.begin();
 	for(;it!=contact_table.end();it++)
 		if (it->second->contact_changed()) 
@@ -275,13 +272,13 @@ bool level_set_contact::master_contact_body::master_contact_changed()
 	return contact_surfaces_changed;
 }
 
-dal::shared_ptr<getfem::mesh_im> level_set_contact::master_contact_body::
+std::shared_ptr<getfem::mesh_im> level_set_contact::master_contact_body::
 	build_mesh_im_on_boundary(size_type region_id)
 {
 
-	dal::shared_ptr<getfem::mesh_im> pmim_contact;
+	std::shared_ptr<getfem::mesh_im> pmim_contact;
 
-		pmim_contact.reset(new mesh_im(get_mesh()));
+	pmim_contact = std::make_shared<mesh_im>(get_mesh());
 		if (mult_mim_order!=size_type(-1)){
 			pmim_contact->set_integration_method
                           (get_mesh().region(region_id).index(),
@@ -314,15 +311,15 @@ mcb(_mcb), scb(_scb)
 		Umaster=mcb.get_model().real_variable(mcb.get_var_name());
 	// size_type dof_check = Umaster.size();
 	// size_type node_check = mcb.get_mesh().nb_points();
-	def_master.reset(new getfem::temporary_mesh_deformator<>
-		(mcb.get_mesh_fem(),Umaster));
+	def_master = std::make_shared<getfem::temporary_mesh_deformator<>>
+		(mcb.get_mesh_fem(),Umaster);
 	mcb.is_deformed=true;
 	if (&mcb.get_mesh()!=&scb.get_mesh()){ 
 		//  not deforming the slave if the master and the slave are the same
 		const model_real_plain_vector& 
 			Uslave=scb.get_model().real_variable(scb.get_var_name());
-		def_slave.reset(new getfem::temporary_mesh_deformator<>
-			(scb.get_mesh_fem(),Uslave));
+		def_slave = std::make_shared<getfem::temporary_mesh_deformator<>>
+			(scb.get_mesh_fem(),Uslave);
 		scb.is_deformed=true;
 	}
 	if (ud == FULL_UPDATE) mcb.update_for_slave(scb.get_var_name());
@@ -341,16 +338,12 @@ level_set_contact::contact_pair_info::contact_pair_info(
 	const std::string& _mult_name,
 	size_type _GIVEN_CONTACT_REGION) :
 
-master_cb(underformed_mcb),
+        master_cb(underformed_mcb),
 	slave_cb(underformed_scb),
 	mult_name(_mult_name),
 	GIVEN_CONTACT_REGION(_GIVEN_CONTACT_REGION),
 
 	ACTIVE_CONTACT_REGION(getfem::mesh_region::free_region_id(master_cb.get_mesh())),
-	pmim_contact(0),
-	ifem_srf(0),
-	pinterpolated_fem(0),
-	pinterpolated_fem_U(0),
 	members_are_computed(false),
 	init_cont_detect_done(false)
 
@@ -524,17 +517,21 @@ void level_set_contact::contact_pair_info::update() const
 		"Internal error: Contact area is empty");
 
 	//pinterpolated_fem for level set
-	pinterpolated_fem.reset(new mesh_fem(master_cb.get_mesh()));
-	if (ifem_srf.get()!=0) getfem::del_interpolated_fem(ifem_srf);
-	ifem_srf=getfem::new_interpolated_fem(
-		slave_cb.get_ls_mesh_fem(),*pmim_contact);
+	pinterpolated_fem = std::make_shared<mesh_fem>(master_cb.get_mesh());
+
+	pinterpolated_fem_U = std::shared_ptr<mesh_fem>();
+
+	if (ifem_srf.get()) getfem::del_interpolated_fem(ifem_srf);
+
+	ifem_srf = getfem::new_interpolated_fem(slave_cb.get_ls_mesh_fem(),
+						*pmim_contact, 0, dal::bit_vector(), false);
 	pinterpolated_fem->set_finite_element(
 		master_cb.get_mesh().region(ACTIVE_CONTACT_REGION).index(),ifem_srf);
 	pinterpolated_fem->set_qdim(1);
 
 
 	//pinterpolated_fem_U
-	pinterpolated_fem_U.reset(new mesh_fem(master_cb.get_mesh()));
+	pinterpolated_fem_U = std::make_shared<mesh_fem>(master_cb.get_mesh());
 	pinterpolated_fem_U->set_finite_element(master_cb.get_mesh().
 		region(ACTIVE_CONTACT_REGION).index(),ifem_srf);
 	pinterpolated_fem_U->set_qdim(master_cb.get_mesh().dim());
@@ -550,7 +547,7 @@ void level_set_contact::contact_pair_info::update() const
 			= ifem_srf->index_of_global_dof(icv, j);}
 	}
 
-	slave_ls_dofs.reset(new gmm::unsorted_sub_index(index));
+	slave_ls_dofs = std::make_shared<gmm::unsorted_sub_index>(index);
 
 	//slave_U_dofs
 	std::vector<size_type> indexU(pinterpolated_fem_U->nb_dof());
@@ -558,7 +555,7 @@ void level_set_contact::contact_pair_info::update() const
 	for(size_type d=0;d<dim;d++)
 		for(size_type i=0;i<pinterpolated_fem->nb_dof();i++)
 			indexU[dim*i+d] = dim*index[i]+d;
-	slave_U_dofs.reset(new gmm::unsorted_sub_index(indexU));
+	slave_U_dofs = std::make_shared<gmm::unsorted_sub_index>(indexU);
 
 	members_are_computed=true;
 }
@@ -569,11 +566,9 @@ getfem::size_type level_set_contact::add_level_set_normal_contact_brick(
 	model& md, 
 	master_contact_body& mcb, 
 	slave_contact_body& scb,
-	size_type rg)
-{
+	size_type rg) {
 	//level set contact class
-	getfem::pbrick pbr = 
-		new level_set_contact_brick(md,mcb,scb,rg);
+  getfem::pbrick pbr = std::make_shared<level_set_contact_brick>(md,mcb,scb,rg);
 
 	//term description
 	const std::string& name_Um = mcb.get_var_name();
