@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2012-2015 Yves Renard, Konstantinos Poulios
+ Copyright (C) 2012-2016 Yves Renard, Konstantinos Poulios
 
  This file is a part of GetFEM++
 
@@ -309,7 +309,7 @@ namespace getfem {
     return false;
   }
 
-  void projected_fem::update_from_context(void) const {
+  void projected_fem::update_from_context() const {
     fictx_cv = size_type(-1);
     dim_ = dim_type(-1);
 
@@ -350,7 +350,7 @@ namespace getfem {
       papprox_integration pai = pim->approx_method();
       bgeot::pgeometric_trans pgt = mim_target.linked_mesh().trans_of_convex(cv);
       bgeot::pgeotrans_precomp pgp =
-        bgeot::geotrans_precomp(pgt, &(pai->integration_points()), 0);
+        bgeot::geotrans_precomp(pgt, pai->pintegration_points(), 0);
       dal::bit_vector dofs;
       size_type last_cv = size_type(-1); // refers to the source mesh
       short_type last_f = short_type(-1); // refers to the source mesh
@@ -440,8 +440,8 @@ namespace getfem {
     }
     /** setup global dofs, with dummy coordinates */
     base_node P(dim()); gmm::fill(P,1./20);
-    node_tab_.resize(max_dof);
-    std::fill(node_tab_.begin(), node_tab_.end(), P);
+    std::vector<base_node> node_tab_(max_dof, P);
+    pspt_override = bgeot::store_point_tab(node_tab_);
     pspt_valid = false;
     dof_types_.resize(max_dof);
     std::fill(dof_types_.begin(), dof_types_.end(),
@@ -483,13 +483,6 @@ namespace getfem {
               mim_target.linked_mesh().structure_of_convex(cv)->nb_faces()));
   }
 
-  bgeot::pstored_point_tab projected_fem::node_tab(size_type)
-    const {
-    if (!pspt_valid)
-      { pspt = bgeot::store_point_tab(node_tab_); pspt_valid = true; }
-    return pspt;
-  }
-
   void projected_fem::base_value(const base_node &, base_tensor &) const
   { GMM_ASSERT1(false, "No base values, real only element."); }
   void projected_fem::grad_base_value(const base_node &,
@@ -532,8 +525,8 @@ namespace getfem {
     std::map<size_type,gausspt_projection_data>::iterator git;
     git = e.gausspt.find(c.ii());
     if (c.have_pgp() &&
-        (&c.pgp()->get_point_tab()
-         == &e.pim->approx_method()->integration_points()) &&
+        (c.pgp()->get_ppoint_tab()
+         == e.pim->approx_method()->pintegration_points()) &&
         git != e.gausspt.end()) {
       gausspt_projection_data &gppd = git->second;
       if (gppd.iflags & 1) {
@@ -625,8 +618,8 @@ namespace getfem {
     std::map<size_type,gausspt_projection_data>::iterator git;
     git = e.gausspt.find(c.ii());
     if (c.have_pgp() &&
-        (&c.pgp()->get_point_tab()
-         == &e.pim->approx_method()->integration_points()) &&
+        (c.pgp()->get_ppoint_tab()
+         == e.pim->approx_method()->pintegration_points()) &&
         git != e.gausspt.end()) {
       gausspt_projection_data &gppd = git->second;
       if (gppd.iflags & 1) {
@@ -716,8 +709,8 @@ namespace getfem {
       std::map<size_type,gausspt_projection_data>::iterator git;
       git = e.gausspt.find(c.ii());
       if (c.have_pgp() &&
-          (&c.pgp()->get_point_tab()
-           == &e.pim->approx_method()->integration_points()) &&
+          (c.pgp()->get_ppoint_tab()
+           == e.pim->approx_method()->pintegration_points()) &&
           git != e.gausspt.end()) {
         gausspt_projection_data &gppd = git->second;
         if (gppd.iflags & 1) {
@@ -834,9 +827,11 @@ namespace getfem {
   pfem new_projected_fem(const mesh_fem &mf_source_, const mesh_im &mim_target_,
                          size_type rg_source_, size_type rg_target_,
                          dal::bit_vector blocked_dofs_, bool store_val) {
-    pfem pf = new projected_fem(mf_source_, mim_target_, rg_source_, rg_target_,
-                                blocked_dofs_, store_val);
-    dal::add_stored_object(new special_projfem_key(pf), pf);
+    pfem pf = std::make_shared<projected_fem>
+      (mf_source_, mim_target_, rg_source_, rg_target_,blocked_dofs_,store_val);
+    dal::pstatic_stored_object_key
+      pk = std::make_shared<special_projfem_key>(pf);
+    dal::add_stored_object(pk, pf);
     return pf;
   }
 
